#!/usr/bin/env python
## Set to work with python/3.4.1
## i, j, and k are 0 based
## Ocean levels: -50m (k=16) to -600m (k=8)
## July-September average last 30 years
## Assuming each netCDF contains 1200 months (100 years) 


from scipy.io import netcdf
import numpy as np
import csv
import sys


ncFile = sys.argv[1]           ## CLIOm.* filename
year = sys.argv[2]             ## Orbital year used for header
totalYears = int(sys.argv[3])       ## loveYearStep
yearsToAverage = int(sys.argv[4])   ## loveAve



## First Month
firstJul = (totalYears-yearsToAverage)*12+6
firstAug = (totalYears-yearsToAverage)*12+7
firstSep = (totalYears-yearsToAverage)*12+8


## netCDF shape: (1200 , 20  ,65 ,120)
##                month,level,lat,lon
f = netcdf.netcdf_file(ncFile, 'r')
temp = f.variables['temp']


##### NA #####
## kref xmin xmax ymin ymax xref yref
## 1 -135.0 -20.0 72.0 85.0 -135.0 72.0
## 2 -45.0 -15.0 59.0 85.0 -15.0 76.0
## 3 -78.0 -45.0 59.0 76.0 -48.0 59.0
## 4 -75.0 -58.0 60.0 75.0 -75.0 75.0
## 5 -60.0 -45.0 45.0 60.0 -60.0 60.0
## 6 -65.0 -50.0 40.0 48.0 -65.0 40.0

##### EA #####
## kref xmin xmax ymin ymax xref yref
## 1 -14.0 5.0 52.0 62.0 -9.0 52.0
## 2 2.0 17.0 63.0 73.0 2.0 63.0
## 3 17.0 40.0 65.0 73.0 17.0 73.0
## 4 10.0 100.0 76.0 82.0 58.0 81.0


tempOutNA = np.empty((6,10))
tempOutEA = np.empty((4,10))

indxListNA = [(98,59),(104,52),(99,47),(99,58),(98,51),(98,46)]
indxListEA = [(106,44),(107,48),(107,52),(106,55)]

indx = 0
for i,j in indxListNA:
    tempOutNA[indx][0] = indx+1
    for k in range(1,10):
        tempOutNA[indx][k] = ( np.mean(temp[firstJul:(totalYears*12)-1:12,17-k,j,i])+np.mean(temp[firstAug:(totalYears*12)-1:12,17-k,j,i])+np.mean(temp[firstSep:(totalYears*12)-1:12,17-k,j,i]) )/3
    indx+=1

indx = 0
for i,j in indxListEA:
    tempOutEA[indx][0] = indx+1
    for k in range(1,10):
        tempOutEA[indx][k] = ( np.mean(temp[firstJul:(totalYears*12)-1:12,17-k,j,i])+np.mean(temp[firstAug:(totalYears*12)-1:12,17-k,j,i])+np.mean(temp[firstSep:(totalYears*12)-1:12,17-k,j,i]) )/3
    indx+=1


## Header: Year NoOfkref NoOfOceanLevels id
np.savetxt('out.na',tempOutNA,fmt='%d %2.1f %2.1f %2.1f %2.1f %2.1f %2.1f %2.1f %2.1f %2.1f',header='%s 6 %d 752' % (year,temp[8,8:17,j,i].size))
np.savetxt('out.ea',tempOutEA,fmt='%d %2.1f %2.1f %2.1f %2.1f %2.1f %2.1f %2.1f %2.1f %2.1f',header='%s 4 %d 753' % (year,temp[8,8:17,j,i].size))
