######################################################################
## NOTE:
##
## This script is called by "cnvLoveGSM.sh"
##
## Bias calculator and field generator based on 4 different methods,
## then applies to LOVECLIM generated fields before passing to GSM
## methods, by adding additional lines to the "ffConvLoveGSM.jnl".
## 
## Fields: Temperature and Precipitation
##
## Export these variables befoer call:
##                                    ${biasMethod}
## >> Call before cnvLoveGSM.sh <<
##
######################################################################

######################################################################
### ERROR HANDLING
##  err: Determines important errors from unimportant errors
##       Set it to a non-zero value before commands to stop
##        if they fail, and zero after them again
##       e.g.: err=1
##             command; err=0

err=0
error() {
    if [ ${err} -ne '0' ]; then
        echo "file $(basename $0) failed at line $1"
        exit 1
    fi
}
trap 'error ${LINENO}' ERR
######################################################################

log()
{
    eval $@
    if [[ ${debug} == 1 ]]; then
        echo $@
    fi
}


cat<<EOF
######################################################################
### bias.sh

EOF

case ${biasMethod} in
    '1')
	##
	## Delta change method
	## [Hay et. al, 2000; Graham et al., 2007; Sperna Weiland et al., 2010]
	##
	## T  = T_paleoLove + ( T_PDobs - T_PDLove )
	## Pr = Pr_paleoLove x ( Pr_PDobs / Pr_PDLove )
	##
	
	## PDclim.nc:
	## Contains both observation and LOVECLIM temp & precip.
        ## tempObs, precObs, tempLove, precLove
	## 12 months climatology data

	echo "use PDclim.nc" >> ffConvLoveGSM.jnl

	if [[ ${biasT} == '1' ]]; then
	    echo "let t2mCorrected = t2m[l=1:12, k=1] + ( tempObs[l=1:12, k=1] - tempLove[l=1:12, k=1] )" >> ffConvLoveGSM.jnl
	else
	    echo "let t2mCorrected = t2m[l=1:12]" >> ffConvLoveGSM.jnl
	fi

	if [[ ${biasP} == '1' ]]; then
	    echo "let precCorrected = prec[l=1:12] * ( precObs[l=1:12] / precLove[l=1:12] )" >> ffConvLoveGSM.jnl
	else
	    echo "let precCorrected = prec[l=1:12]" >> ffConvLoveGSM.jnl
	fi
	;;
    
    *)
	;;
esac


cat<<EOF

### bias.sh
######################################################################

EOF

