#!/usr/bin/env python
####################################################################
###
### Solhouette Orography
### H  = w2 . H* + (1-w2) . H_mean
###
### H* = w1 . H_max + (1-w1) . ( H_sx + H_sy ) / 2
###   H_sx = mean(max_i)
###   H_sy = mean(max_j)
### H_mean = Zigma[ w_i,j . a_i,j ] / Zigma[ w_i,j ]
###
####################################################################


import numpy as np
from numpy.testing import assert_array_equal, assert_array_almost_equal

topoGSM1D = np.genfromtxt("topoGSM.dat",delimiter="\n")
topoGSM2D = np.reshape(topoGSM1D, (720,720))
surfGSM2D = np.ceil( topoGSM2D / ( topoGSM2D + 1 ) )
iGSM,jGSM = topoGSM2D.shape

iLOVE = 32
jLOVE = 64

## Size of x and y steps in fine (GSM) res. grid
stepiGSM = 180.0 / iGSM
stepjGSM = 360.0 / jGSM

## Size of x and y steps in coarse (LOVECLIM) res. grid
stepiLOVE = 180.0 / iLOVE
stepjLOVE = 360.0 / jLOVE

## Arrays to store LOVECLIM res. data and weight factors
Hmax     =  np.zeros((iLOVE, jLOVE))
Hmin     =  np.zeros((iLOVE, jLOVE))+10000.0

I = 0
Iincrease = False

for i in np.arange(0, iGSM-1):

    J = 0
    Jincrease = False
    for j in np.arange(0, jGSM-1):

        Hmax[I, J] = topoGSM2D[i, j] if topoGSM2D[i, j] > Hmax[I, J] else Hmax[I, J]
        Hmin[I, J] = topoGSM2D[i, j] if topoGSM2D[i, j] < Hmin[I, J] else Hmin[I, J]

        if (i+1) * stepiGSM > (I+1) * stepiLOVE:
            Hmax[I+1, J] = topoGSM2D[i, j] if topoGSM2D[i, j] > Hmax[I+1, J] else Hmax[I+1, J]
            Hmin[I+1, J] = topoGSM2D[i, j] if topoGSM2D[i, j] < Hmin[I+1, J] else Hmin[I+1, J]
            Iincrease = True
            if (j+1) * stepiGSM > (J+1) * stepjLOVE:
                Hmax[I+1, J+1] = topoGSM2D[i, j] if topoGSM2D[i, j] > Hmax[I+1, J+1] else Hmax[I+1, J+1]
                Hmin[I+1, J+1] = topoGSM2D[i, j] if topoGSM2D[i, j] < Hmin[I+1, J+1] else Hmin[I+1, J+1]

        if (j+1) * stepjGSM > (J+1) * stepjLOVE:
            Hmax[I, J+1] = topoGSM2D[i, j] if topoGSM2D[i, j] > Hmax[I, J+1] else Hmax[I, J+1]
            Hmin[I, J+1] = topoGSM2D[i, j] if topoGSM2D[i, j] < Hmin[I, J+1] else Hmin[I, J+1]
            Jincrease = True

        if Jincrease:
            J += 1
            Jincrease = False

    if Iincrease:
        I += 1
        Iincrease = False


## Save hMax and hMin
np.savetxt('hMax.dat', Hmax, "%6.2f")
np.savetxt('hMin.dat', Hmin, "%6.2f")
