#!/bin/env bash
####################################################################
###
### Applies one of the three methods to generate LOVECLIM resolution
### orography.
### 1. Simple Average
### 2. Envelope Method
### 3. Silhouette Method
###
####################################################################

log()
{
    eval $@
    if [[ ${debug} == 1 ]]; then
	echo $@
    fi
}


cat<<EOF
####################################################################
### hMaxMin.sh

EOF


log met=$1

${ferret} <<EOF
use topoGSMres.nc

list/clob/nohead/file=topoGSM.dat/order=xy/format=(f10.1) topogsmres
exit
EOF

log python ${execCpl}/hMaxMin.py

sed -i '/^$/d' hMax.dat
sed -i '/^$/d' hMin.dat

${ferret} <<EOF
define axis/x=0E:5.625W:5.625/mod/unit=degree xcoarse
define axis/y=87.1875S:87.1875N:5.625/unit=degree ycoarse
define grid/x=xcoarse/y=ycoarse grlove

file/col=64/var=hMax/order=xy/grid=grlove hMax.dat
file/col=64/var=hMin/order=xy/grid=grlove hMin.dat

save/clob/file="bergMax.nc" hMax[d=1]
save/clob/file="bergMin.nc" hMin[d=2]

exit
EOF


log ncecat bergMax.nc bergMax2.nc
log ncrename -v HMAX,h -v YCOARSE,lat -v XCOARSE,lon -d YCOARSE,lat -d XCOARSE,lon -d record,time bergMax2.nc
log ncks -A -v time ${cplCndDir}/topo/berg.PD.nc bergMax2.nc
log ncap2 -O -s "time=${nextOrby}" bergMax2.nc newBergMax.nc

log ncecat bergMin.nc bergMin2.nc
log ncrename -v HMIN,h -v YCOARSE,lat -v XCOARSE,lon -d YCOARSE,lat -d XCOARSE,lon -d record,time bergMin2.nc
log ncks -A -v time ${cplCndDir}/topo/berg.PD.nc bergMin2.nc
log ncap2 -O -s "time=${nextOrby}" bergMin2.nc newBergMin.nc

rm bergMax.nc bergMax2.nc
rm bergMin.nc bergMin2.nc
rm hMax.dat hMin.dat


cat<<EOF

### hMaxMin
####################################################################

EOF
