############################################################
### Copies the Loveclim model into the computational 
##         node

##############################################################
### ERROR HANDLING
##  err: Determines important errors from unimportant errors
##       Set it to a non-zero value before commands to stop
##        if they fail, and zero after them again
##       e.g.: err=1
##             command; err=0

err=0
error() {
    if [ ${err} -ne '0' ]; then
        echo "file $(basename $0) failed at line $1"
        exit 1
    fi
}
trap 'error ${LINENO}' ERR
##############################################################


log()
{
    eval $@
    if [[ ${debug} == 1 ]]; then
	echo $@
    fi
}


cat <<EOF
############################################################
## prepLove

EOF

err=1

mkdir -p ${execLove}/OUTPUT

spnDir=${strDir}/${no}/spinup
if [[ ${cnt} == '0' ]]; then
    mkdir -p ${spnDir}/nc \
	${spnDir}/dat \
	${spnDir}/ic \
	${spnDir}/ocean
fi

mainDir=${strDir}/${no}/main/loveclim
if [[ ${cnt} == '0' ]]; then
    mkdir -p ${mainDir}/nc \
	${mainDir}/dat \
	${mainDir}/ic \
	${mainDir}/ocean \
	${mainDir}/stat
fi


## rsync sometimes fail, and just needs a resubmit, so ...
err=0
errRsync=1
while [[ ${errRsync} != '0' ]]; do
    log rsync -au --delete ${loveSrcDir}/ ${scrLoveCopy}/
    errRsync=$?
done

err=1
log rsync -au --delete ${scrLoveCopy}/* ${execLove}


##############################################################################
## Set albedos (for both original and continue run)
file=RUN/V1.3/expdir/exp.param

sed -i "s/albin=$/albin=${albin}/" ${execLove}/${file}
sed -i "s/albis=$/albis=${albis}/" ${execLove}/${file}
sed -i "s/albice=$/albice=${albice}/" ${execLove}/${file}
sed -i "s/alphd=$/alphd=${alphd}/" ${execLove}/${file}
sed -i "s/alphdi=$/alphdi=${alphdi}/" ${execLove}/${file}
sed -i "s/alphs=$/alphs=${alphs}/" ${execLove}/${file}
sed -i "s/cgren=$/cgren=${cgren}/" ${execLove}/${file}

##############################################################################
## Set precipitation threshold in atmphys0.f (for both original and continue run)
atmphys=LOVECLIM/V1.3/ecbilt/sources/atmphys0.f
sed -i "s/ec_detqmax=$/ec_detqmax=${precth}\*qmax/" ${execLove}/${atmphys}

##############################################################################
## Set cloud cover file to be used (for both original and continue run)
cloudDir=LOVECLIM/V1.3/ecbilt/inputdata/lwrpar/profiles

## prepLoveEns.sh runs only at the start of a full run (before
## spin-up), therefore there is no need to enable any LOVECLIM
## output variables.
## Output variables will be enabled at the start of the main run
## in coupler using loveOutVarEnableEns.sh


## Update GHG.dat year based on ${loveYear}
sed -n -e "/^$((-1*orby)) /,\$p" ${cplCndDir}/GHG/GHG.dat | \
    awk -v "loveYear=${loveYear}" '$1=loveYear+NR-1' > \
    ${execLove}/LOVECLIM/V1.3/scenario/Default/GHG.dat.full
## LOVECLIM does not accept large GHG.dat files, so break them to
## files with lines no more than 10000
sed -n '1,10000p' ${execLove}/LOVECLIM/V1.3/scenario/Default/GHG.dat.full > \
    ${execLove}/LOVECLIM/V1.3/scenario/Default/GHG.dat

# awk -v "loveYear=${loveYear}" '$1=loveYear+NR-1 {print}' ${cplCndDir}/GHG/GHG`printf %06d $((-1*orby))`.dat > \
#     ${execLove}/LOVECLIM/V1.3/scenario/Default/GHG.dat

if [[ ${cnt} == '0' ]]; then
    log cp ${cplCndDir}/IC/${it}/berg.nc \
	${execLove}/LOVECLIM/V1.3/scenario/Default/berg.nc
    log cp ${cplCndDir}/IC/${it}/icemask.nc \
	${execLove}/LOVECLIM/V1.3/scenario/Default/icemask.nc

    log cp ${cplCndDir}/IC/PD/berg.nc \
	${execLove}/LOVECLIM/V1.3/scenario/Default/orgberg.nc
    log cp ${cplCndDir}/IC/PD/icemask.nc \
	${execLove}/LOVECLIM/V1.3/scenario/Default/orgicemask.nc

    log cp ${cplCndDir}/IC/${it}/${ic}.tar.gz \
	${execLove}/RUN/V1.3/expdir/ref/ic/
    log cd ${execLove}/RUN/V1.3/expdir/ref/ic/
    log tar -xzf ${ic}.tar.gz

    ## Copy initial berg.nc, icemask.nc, and IC to storage drive
    log cp ${execLove}/LOVECLIM/V1.3/scenario/Default/berg.nc \
	${spnDir}/nc/berg${orby}.nc
    log cp ${execLove}/LOVECLIM/V1.3/scenario/Default/icemask.nc \
	${spnDir}/nc/icemask${orby}.nc
    log cp ${execLove}/RUN/V1.3/expdir/ref/ic/${ic}.tar.gz \
	${spnDir}/ic/

elif [[ ${cnt} == '1' ]]; then
    log cp ${mainDir}/nc/${berg} \
	${execLove}/LOVECLIM/V1.3/scenario/Default/berg.nc
    log cp ${mainDir}/nc/${icemask} \
	${execLove}/LOVECLIM/V1.3/scenario/Default/icemask.nc
    
    log cp ${strDir}/${no}/spinup/nc/berg*.nc \
	${execLove}/LOVECLIM/V1.3/scenario/Default/orgberg.nc
    log cp ${strDir}/${no}/spinup/nc/icemask*.nc \
	${execLove}/LOVECLIM/V1.3/scenario/Default/orgicemask.nc

    log tar -xzf ${mainDir}/ic/${ic}.tar.gz \
	-C ${execLove}/RUN/V1.3/expdir/ref/ic/
fi



## CLOUD #########################################################
## Enable/disable cloud paramterization
sed -i "s/cloud=$/cloud=${cloud}/" ${execLove}/RUN/V1.3/expdir/exp.param

## Copy the cloud cover file and execute write.x
log cp ${cplCndDir}/cloud/${cloudFile} \
    ${execLove}/LOVECLIM/V1.3/ecbilt/inputdata/lwrpar/profiles/ccECBILT.asc
log cd ${execLove}/LOVECLIM/V1.3/ecbilt/inputdata/lwrpar/profiles/
log ./write.x
##################################################################


err=0

cd ${execLove}
## No erro check for Loveclim "make" yet!
log bash make 2> /dev/null

err=1

sed -i "s/start_y=.*/start_y=${loveYear}/" ${execLove}/RUN/V1.3/expdir/exp.param
sed -i "s/imsk=$/imsk=${imask}/" ${execLove}/RUN/V1.3/expdir/exp.param
sed -i "s/bering=.*/bering=0.3/" ${execLove}/RUN/V1.3/expdir/exp.param  ## Should be used for inception/PD runs only at first step

if [[ ${orbv} -eq '1' ]] || [[ ${iter} -eq '1' ]]; then
    cd ${execCpl}/forcing
    log eccf=`echo ${orby} | ./orbParmSeparate.o | awk '{print $1}' | sed -n '1p'`
    log oblf=`echo ${orby} | ./orbParmSeparate.o | awk '{print $1}' | sed -n '2p'`
    log omwebf=`echo ${orby} | ./orbParmSeparate.o | awk '{print $1}' | sed -n '3p'`

    ## Update exp.param
    sed -i "s/eccf=.*/eccf=${eccf};oblf=${oblf};omwebf=${omwebf}/" ${execLove}/RUN/V1.3/expdir/exp.param
fi

## GSM can only run for 100 years in first step (iter=1).
## Also use appropriate land model runoff script.
if [[ ${spn} -eq 1 ]]; then
    log lengthInDays=$((100*360))
    cp ${execLove}/LOVECLIM/V1.3/lbm/sources/landmodel0.f.org ${execLove}/LOVECLIM/V1.3/lbm/sources/landmodel0.f
elif [[ ${iter} -eq 1 ]]; then
    log lengthInDays=$((100*360))
    cp ${execLove}/LOVECLIM/V1.3/lbm/sources/landmodel0.f.org ${execLove}/LOVECLIM/V1.3/lbm/sources/landmodel0.f
else
    log lengthInDays=$((loveYearStep*360))
    cp ${execLove}/LOVECLIM/V1.3/lbm/sources/landmodel0.f.runoffMasked ${execLove}/LOVECLIM/V1.3/lbm/sources/landmodel0.f
fi
sed -i "s/lengthInDays=.*/lengthInDays=${lengthInDays}/" ${execLove}/RUN/V1.3/expdir/exp.param


cat <<EOF

## prepLoveEns
############################################################

EOF
