##############################################################
### Prepares Loveclim for next time-step
##
##  exported needs:
##                 ${loveSrcDir}
##                 ${execDir}
##                 ${strDir}
##                 ${no}
##                 ${orby}
##############################################################

##############################################################
### ERROR HANDLING
##  err: Determines important errors from unimportant errors
##       Set it to a non-zero value before commands to stop
##        if they fail, and zero after them again
##       e.g.: err=1
##             command; err=0

err=0
error() {
    if [ ${err} -ne '0' ]; then
        echo "file $(basename $0) failed at line $1"
        exit 1
    fi
}
trap 'error ${LINENO}' ERR
##############################################################

log()
{
    eval $@
    if [[ ${debug} == 1 ]]; then
	echo $@
    fi
}

cat<<EOF
##############################################################
### prepLoveNextEns

EOF


err=1

if [[ ${ghg} == '1' ]]; then
    ## Check if GHG.dat includes loveYear
    err=0
    grep "^${loveYear}" ${execLove}/LOVECLIM/V1.3/scenario/Default/GHG.dat > \
	/dev/null
    if [[ $? != '0' ]]; then
	echo "#### Updating GHG.dat ####"
	lineNo=`grep -n "^${loveYear}" ${execLove}/LOVECLIM/V1.3/scenario/Default/GHG.dat.full | cut -d: -f1`
	sed -n "${lineNo},$((lineNo-1+10000))p" ${execLove}/LOVECLIM/V1.3/scenario/Default/GHG.dat.full > \
	    ${execLove}/LOVECLIM/V1.3/scenario/Default/GHG.dat
    fi
    err=1

    # awk -v "loveYear=${loveYear}" '$1=loveYear+NR-1 {print}' ${cplCndDir}/GHG/GHG`printf %06d $((-1*orby))`.dat > \
    # 	${execLove}/LOVECLIM/V1.3/scenario/Default/GHG.dat
fi

if [[ ${orbv} -eq '1' ]]; then
    cd ${execCpl}/forcing
    log eccf=`echo ${orby} | ./orbParmSeparate.o | awk '{print $1}' | sed -n '1p'`
    log oblf=`echo ${orby} | ./orbParmSeparate.o | awk '{print $1}' | sed -n '2p'`
    log omwebf=`echo ${orby} | ./orbParmSeparate.o | awk '{print $1}' | sed -n '3p'`

    # if [[ ${iter} -eq 2 ]]; then
	## Update exp.param
	sed -i "s/eccf=.*/eccf=${eccf};oblf=${oblf};omwebf=${omwebf}/" ${execLove}/RUN/V1.3/expdir/exp.param
    # else
    # 	## Update run.param in run directory
    # 	sed -i "s/eccf=.*;oblf/eccf=${eccf};oblf/" ${execLove}/RUN/V1.3/expdir/run/run.param
    # 	sed -i "s/oblf=.*;omwebf/oblf=${oblf};omwebf/" ${execLove}/RUN/V1.3/expdir/run/run.param
    # 	sed -i "s/omwebf=.*/omwebf=${omwebf}/" ${execLove}/RUN/V1.3/expdir/run/run.param
	
    # 	## Update namelist-ecbilt in run/repo
    # 	sed -i "s/eccf.*/eccf=${eccf}/" ${execLove}/RUN/V1.3/expdir/run/repo/namelist-ecbilt
    # 	sed -i "s/oblf.*/oblf=${oblf}/" ${execLove}/RUN/V1.3/expdir/run/repo/namelist-ecbilt
    # 	sed -i "s/omwebf.*/omwebf=${omwebf}/" ${execLove}/RUN/V1.3/expdir/run/repo/namelist-ecbilt
    # fi
fi


# if [[ ${cnt} != '1' ]]; then

    ## Copy restart files for next Loveclim run
    cd ${execLove}/RUN/V1.3/expdir/run/repo/
    log lastIC=`ls -d ic* | tail -n1`
    ## Check if the new ic* has the correct size, otherwise stop!
    icSize=`du ${lastIC} | awk '{print $1}'`
    if [[ `echo "${icSize} < 10000" | bc` == '1' ]]; then
	echo "Wrong ic directory size (size = ${icSize})"
	exit 1
    else
	log cp -r ${lastIC} ${execLove}/RUN/V1.3/expdir/ref/ic
    fi

# fi


## Bering Strait update
## For high res. GSM:
## x = 169.75W , 169.25W , 168.75W
## y = 65.625N , 65.875N , 66.125N
if [[ ${bs} -eq 1 ]] && [[ ${spn} -eq 0 ]]; then
    log bsVal=`cat ${execGSM}/bs.dat | bc`
else
    log bsVal=0.3
fi



## Prepare LOVECLIM exp.param with recompiling files at each start
sed -i "s/start_y=.*/start_y=${loveYear}/" ${execLove}/RUN/V1.3/expdir/exp.param

## iter=1 -> dOrby=100. Use next line to restore dOrby
sed -i "s/lengthInDays=.*/lengthInDays=$((loveYearStep*360))/" ${execLove}/RUN/V1.3/expdir/exp.param
sed -i "s/bering=.*/bering=${bsVal}/" ${execLove}/RUN/V1.3/expdir/exp.param

## Use appropriate land model runoff script
if [[ ${spn} -eq 0 ]]; then
    cp ${execLove}/LOVECLIM/V1.3/lbm/sources/landmodel0.f.runoffMasked ${execLove}/LOVECLIM/V1.3/lbm/sources/landmodel0.f
fi

log rm -r ${execLove}/RUN/V1.3/expdir/run
log rm -r ${execLove}/OUTPUT


## Procedure to avoid LOVECLIM recompilation at each start
# if [[ ${iter} -eq 2 ]]; then
#     sed -i "s/start_y=.*/start_y=${loveYear}/" ${execLove}/RUN/V1.3/expdir/exp.param

#     ## iter=1 -> dOrby=100. Use next line to restore dOrby
#     sed -i "s/lengthInDays=.*/lengthInDays=$((dOrby*360))/" ${execLove}/RUN/V1.3/expdir/exp.param
#     sed -i "s/bering=.*/bering=${bsVal}/" ${execLove}/RUN/V1.3/expdir/exp.param

#     rm -r ${execLove}/RUN/V1.3/expdir/run
# else
#     sed -i "s/start_y=.*/start_y=${loveYear}/" ${execLove}/RUN/V1.3/expdir/run/run.param
#     sed -i "s/bering=.*/bering=${bsVal}/" ${execLove}/RUN/V1.3/expdir/run/run.param
#     sed -i "s/ntimer=.*/ntimer=$((loveYear*360+1))/" ${execLove}/RUN/V1.3/expdir/run/launch_r1
#     sed -i 's/1 0/3 0/' ${execLove}/RUN/V1.3/expdir/run/launch_r1
# fi


err=0


# rm -r ${execLove}/RUN/V1.3/expdir/run
# rm -r ${execLove}/OUTPUT
# echo "rm -r ${execLove}/RUN/V1.3/expdir/run ${execLove}/OUTPUT"


cat<<EOF

### prepLoveNextEns
##############################################################

EOF
