cat<<EOF
############################################################
### readParam.sh

EOF

## Reads the param file, and passes the values to 'cpl'

# 1. Loveclim enable
love=`sed -n '1p' ${paramFile} | awk '{print $1}'`
# 2. GSM enable
gsm=`sed -n '2p' ${paramFile} | awk '{print $1}'`
# 3. Starting orbital year
orby=`sed -n '3p' ${paramFile} | awk '{print $1}'`
# 4. Variable or fixed orbital forcing
orbv=`sed -n '4p' ${paramFile} | awk '{print $1}'`
# 5. Total number of years to run
noy=`sed -n '5p' ${paramFile} | awk '{print $1}'`
# 6. ghg
ghg=`sed -n '6p' ${paramFile} | awk '{print $1}'`
# 7. Initial loveclim restart directory
ic=`sed -n '7p' ${paramFile} | awk '{print $1}'`
# 8. GSM initialization directory
gsmIC=`sed -n '8p' ${paramFile} | awk '{print $1}'`
# 9. Topography file
berg=`sed -n '9p' ${paramFile} | awk '{print $1}'`
# 10. Icemask file
icemask=`sed -n '10p' ${paramFile} | awk '{print $1}'`
# 11. Enable/Disable bias correction for temperature
biasT=`sed -n '11p' ${paramFile} | awk '{print $1}'`
# 12. Enable/Disable bias correction for precipitation
biasP=`sed -n '12p' ${paramFile} | awk '{print $1}'`
# 13. Bias correction method to use (1/2/3/4)
biasMethod=`sed -n '13p' ${paramFile} | awk '{print $1}'`
# 14. fort file (for GSM only runs)
fort=`sed -n '14p' ${paramFile} | awk '{print $1}'`
# 15. Loveclim years to run
loveYearStep=`sed -n '15p' ${paramFile} | awk '{print $1}'`
# 16. Loveclim years to average over
loveAve=`sed -n '16p' ${paramFile} | awk '{print $1}'`
# 17. Upscaling method
upscl=`sed -n '17p' ${paramFile} | awk '{print $1}'`
# 18. Lapse-rate
lps=`sed -n '18p' ${paramFile} | awk '{print $1}'`


#################################################################################
## Set albedos
file=RUN/V1.3/expdir/exp.param
startLine=`grep -n "ALBEDO" ${paramFile} | awk -F':' '{print $1}'`

# 1. Albedo of melting ice in the arctic
albin=`sed -n "$((startLine+1))p" ${paramFile} | awk '{print $1}'`
sed -i "s/albin=$/albin=${albin}/" ${execLove}/${file}

# 2. Albedo of melting ice in the antarctic
albis=`sed -n "$((startLine+2))p" ${paramFile} | awk '{print $1}'`
sed -i "s/albis=$/albis=${albis}/" ${execLove}/${file}

# 3. Albedo of melting ice
albice=`sed -n "$((startLine+3))p" ${paramFile} | awk '{print $1}'`
sed -i "s/albice=$/albice=${albice}/" ${execLove}/${file}

# 4. Albedo of snow
alphd=`sed -n "$((startLine+4))p" ${paramFile} | awk '{print $1}'`
sed -i "s/alphd=$/alphd=${alphd}/" ${execLove}/${file}

# 5. Albedo of thick bare ice
alphdi=`sed -n "$((startLine+5))p" ${paramFile} | awk '{print $1}'`
sed -i "s/alphdi=$/alphdi=${alphdi}/" ${execLove}/${file}

# 6. Albedo of melting snow
alphs=`sed -n "$((startLine+6))p" ${paramFile} | awk '{print $1}'`
sed -i "s/alphs=$/alphs=${alphs}/" ${execLove}/${file}

# 7. Correction of the snow or ice albedo to take into account effects of cloudiness
cgren=`sed -n "$((startLine+7))p" ${paramFile} | awk '{print $1}'`
sed -i "s/cgren=$/cgren=${cgren}/" ${execLove}/${file}


#################################################################################
## Set precipitation threshold in atmphys0.f
atmphys=LOVECLIM/V1.3/ecbilt/sources/atmphys0.f
startLine=`grep -n "ATMPHYS0.f" ${paramFile} | awk -F':' '{print $1}'`

# 1. Precipitation threshold
precth=`sed -n "$((startLine+1))p" ${paramFile} | awk '{print $1}'`
sed -i "s/ec_detqmax=$/ec_detqmax=${precth}\*qmax/" ${execLove}/${atmphys}


#################################################################################
## Set cloud cover file to be used
cloudDir=LOVECLIM/V1.3/ecbilt/inputdata/lwrpar/profiles
startLine=`grep -n "CLOUD" ${paramFile} | awk -F':' '{print $1}'`

# 1. Cloud radiation parameterization
cloud=`sed -n "$((startLine+1))p" ${paramFile} | awk '{print $1}'`

# 2. Cloud file name
cloudFile=`sed -n "$((startLine+2))p" ${paramFile} | awk '{print $1}'`


#################################################################################
## Enable/Disable variables based on parameter file
atmdiag=LOVECLIM/V1.3/ecbilt/sources/atmdiag0.f
startLine=`grep -n "LOVECLIM ATMOSPHERE VIRIABLES" ${paramFile} | \
    awk -F':' '{print $1}'`

# 1. surface temperature
ts=`sed -n "$((startLine+1))p" ${paramFile} | awk '{print $1}'`
if [[ ${ts} -eq '1' ]]; then
    sed -i '/!T\ surface\ temperature/ s/c\$\$\$//' ${execLove}/${atmdiag}
fi
# 2. 2m temperature
t2m=`sed -n "$((startLine+2))p" ${paramFile} | awk '{print $1}'`
if [[ ${t2m} -eq '1' ]]; then
    sed -i '/!T\ 2m\ temperature/ s/c\$\$\$//' ${execLove}/${atmdiag}
fi
# 3. 650 hPa temperature
t4g=`sed -n "$((startLine+3))p" ${paramFile} | awk '{print $1}'`
if [[ ${t4g} -eq '1' ]]; then
    sed -i '/!T\ 650\ hPa\ temperature/ s/c\$\$\$//' ${execLove}/${atmdiag}
fi
# 4. evaporation
evap=`sed -n "$((startLine+4))p" ${paramFile} | awk '{print $1}'`
if [[ ${evap} -eq '1' ]]; then
    sed -i '/!T\ evaporation/ s/c\$\$\$//' ${execLove}/${atmdiag}
fi
# 5. precipitation
prec=`sed -n "$((startLine+5))p" ${paramFile} | awk '{print $1}'`
if [[ ${prec} -eq '1' ]]; then
    sed -i '/!T\ precipitation/ s/c\$\$\$//' ${execLove}/${atmdiag}
fi
# 6. surface pressure
sp=`sed -n "$((startLine+6))p" ${paramFile} | awk '{print $1}'`
if [[ ${sp} -eq '1' ]]; then
    sed -i '/!T\ surface\ pressure/ s/c\$\$\$//' ${execLove}/${atmdiag}
fi
# 7. 350 hPa temperature
t2g=`sed -n "$((startLine+7))p" ${paramFile} | awk '{print $1}'`
if [[ ${t2g} -eq '1' ]]; then
    sed -i '/!T\ 350\ hPa\ temperature/ s/c\$\$\$//' ${execLove}/${atmdiag}
fi
# 8. stratosphere temperature
t0g=`sed -n "$((startLine+8))p" ${paramFile} | awk '{print $1}'`
if [[ ${t0g} -eq '1' ]]; then
    sed -i '/!T\ stratosphere\ temperature/ s/c\$\$\$//' ${execLove}/${atmdiag}
fi
# 9. dynamic precipitation
dyprec=`sed -n "$((startLine+9))p" ${paramFile} | awk '{print $1}'`
if [[ ${dyprec} -eq '1' ]]; then
    sed -i '/!T\ dynamic\ precipitation/ s/c\$\$\$//' ${execLove}/${atmdiag}
fi
# 10. convection precipitation
coprec=`sed -n "$((startLine+10))p" ${paramFile} | awk '{print $1}'`
if [[ ${coprec} -eq '1' ]]; then
    sed -i '/!T\ convection\ precipitation/ s/c\$\$\$//' ${execLove}/${atmdiag}
fi
# 11. total snow
snow=`sed -n "$((startLine+11))p" ${paramFile} | awk '{print $1}'`
if [[ ${snow} -eq '1' ]]; then
    sed -i '/!T\ total\ snow/ s/c\$\$\$//' ${execLove}/${atmdiag}
fi
# 12. land snow depth
snowa=`sed -n "$((startLine+12))p" ${paramFile} | awk '{print $1}'`
if [[ ${snowa} -eq '1' ]]; then
    sed -i '/!T\ land\ snow\ depth/ s/c\$\$\$//' ${execLove}/${atmdiag}
fi
# 13. geopotential height
gph=`sed -n "$((startLine+13))p" ${paramFile} | awk '{print $1}'`
if [[ ${gph} -eq '1' ]]; then
    sed -i '/!T\ geopotential\ height/ s/c\$\$\$//' ${execLove}/${atmdiag}
fi
# 14. relative humidity
rh=`sed -n "$((startLine+14))p" ${paramFile} | awk '{print $1}'`
if [[ ${rh} -eq '1' ]]; then
    sed -i '/!T\ relative\ humidity/ s/c\$\$\$//' ${execLove}/${atmdiag}
fi


cat<<EOF

### readParam.sh
############################################################

EOF
