#!/bin/env bash
####################################################################
###
### Applies one of the three methods to generate LOVECLIM resolution
### orography.
### 1. Simple Average
### 2. Envelope Method
### 3. Silhouette Method
###
####################################################################

log()
{
    eval $@
    if [[ ${debug} == 1 ]]; then
	echo $@
    fi
}


cat<<EOF
####################################################################
### regrid.sh

EOF


log met=$1

${ferret} <<EOF
use topoGSMres.nc

list/clob/nohead/file=topoGSM.dat/order=xy/format=(f10.1) topogsmres
exit
EOF

if [[ ${met} == '1' ]]; then
    log python ${execCpl}/regrid_simpleAvg.py
elif [[ ${met} == '2' ]]; then
    log python ${execCpl}/regrid_envelope.py
elif [[ ${met} == '3' ]]; then
    log python ${execCpl}/regrid_silhouette.py
fi

sed -i '/^$/d' h.dat
sed -i '/^$/d' sfric.dat

${ferret} <<EOF
define axis/x=0E:5.625W:5.625/mod/unit=degree xcoarse
define axis/y=87.1875S:87.1875N:5.625/unit=degree ycoarse
define grid/x=xcoarse/y=ycoarse grlove

file/col=64/var=h/order=xy/grid=grlove h.dat
file/col=64/var=sfric/order=xy/grid=grlove sfric.dat

save/clob/file="berg.nc" h[d=1], sfric[d=2]

exit
EOF


log ncecat berg.nc berg2.nc
log ncrename -v H,h -v SFRIC,sfric -v YCOARSE,lat -v XCOARSE,lon -d YCOARSE,lat -d XCOARSE,lon -d record,time berg2.nc
log ncks -A -v time ${cplCndDir}/topo/berg.PD.nc berg2.nc
log ncap2 -O -s "time=${nextOrby}" berg2.nc newBerg.nc

rm berg.nc berg2.nc
rm topoGSM.dat h.dat sfric.dat


cat<<EOF

### regrid.sh
####################################################################

EOF
