#!/usr/bin/env python

import numpy as np
from numpy.testing import assert_array_equal, assert_array_almost_equal

topoGSM1D = np.genfromtxt("topoGSM.dat",delimiter="\n")
topoGSM2D = np.reshape(topoGSM1D, (720,720))
surfGSM2D = np.ceil( topoGSM2D / ( topoGSM2D + 1 ) )
iGSM,jGSM = topoGSM2D.shape

iLOVE = 32
jLOVE = 64

## Size of x and y steps in fine (GSM) res. grid
stepiGSM = 180.0 / iGSM
stepjGSM = 360.0 / jGSM

## Size of x and y steps in coarse (LOVECLIM) res. grid
stepiLOVE = 180.0 / iLOVE
stepjLOVE = 360.0 / jLOVE

## Arrays to store LOVECLIM res. data and weight factors
topoLOVE  =  np.zeros((iLOVE, jLOVE))
surfLOVE  =  np.zeros((iLOVE, jLOVE))
w         =  np.zeros((iLOVE, jLOVE))

I = 0
for i in np.arange(0, iGSM-1):
    wi1 = 1
    wi2 = 0

    if (i+1) * stepiGSM > (I+1) * stepiLOVE:
        wi2 = ( (i+1) * stepiGSM - (I+1) * stepiLOVE ) / stepiGSM
        wi1 = 1 - wi2

    J = 0
    for j in np.arange(0, jGSM-1):
        wj1 = 1
        wj2 = 0

        if (j+1) * stepjGSM > (J+1) * stepjLOVE:
            wj2 = ( (j+1) * stepjGSM - (J+1) * stepjLOVE ) / stepjGSM
            wj1 = 1 - wj2

        topoLOVE[I, J] += wi1 * wj1 * topoGSM2D[i, j]
        surfLOVE[I, J] += wi1 * wj1 * surfGSM2D[i, j]
        w[I, J] += wi1 * wj1

        try:
            topoLOVE[I+1, J] += wi2 * wj1 * topoGSM2D[i, j]
            surfLOVE[I+1, J] += wi2 * wj1 * surfGSM2D[i, j]
            w[I+1, J] += wi2 * wj1
        except:
            pass
        try:
            topoLOVE[I, J+1] += wi1 * wj2 * topoGSM2D[i, j]
            surfLOVE[I, J+1] += wi1 * wj2 * surfGSM2D[i, j]
            w[I, J+1] += wi1 * wj2
        except:
            pass
        try:
            topoLOVE[I+1, J+1] += wi2 * wj2 * topoGSM2D[i, j]
            surfLOVE[I+1, J+1] += wi2 * wj2 * surfGSM2D[i, j]
            w[I+1, J+1] += wi2 * wj2
        except:
            pass


        if wj2 != 0:
            J += 1

    if wi2 != 0:
        I += 1


## Apply weight and save
h     = np.zeros((iLOVE, jLOVE))
sfric = np.zeros((iLOVE, jLOVE))
for i in range(iLOVE):
    for j in range(jLOVE):
        h[i,j] = topoLOVE[i,j] / w[i,j]
        sfric[i,j] = surfLOVE[i,j] / w[i,j]

np.savetxt('h.dat', h, "%6.2f")
np.savetxt('sfric.dat', sfric, "%6.2f")




###################################################################################
###################################################################################


## Slope 10x5
iLOVE = 72  ## lat 2.5 deg
jLOVE = 72  ## lon 5.0 deg

## Size of x and y steps in fine (GSM) res. grid
stepiGSM = 180.0 / iGSM
stepjGSM = 360.0 / jGSM

## Size of x and y steps in coarse (LOVECLIM) res. grid
stepiLOVE = 180.0 / iLOVE
stepjLOVE = 360.0 / jLOVE


## Arrays to store LOVECLIM res. data and weight factors
topoLOVE  =  np.zeros((iLOVE, jLOVE))
surfLOVE  =  np.zeros((iLOVE, jLOVE))
w         =  np.zeros((iLOVE, jLOVE))

I = 0
for i in np.arange(0, iGSM-1):
    wi1 = 1
    wi2 = 0

    if (i+1) * stepiGSM > (I+1) * stepiLOVE:
        wi2 = ( (i+1) * stepiGSM - (I+1) * stepiLOVE ) / stepiGSM
        wi1 = 1 - wi2

    J = 0
    for j in np.arange(0, jGSM-1):
        wj1 = 1
        wj2 = 0

        if (j+1) * stepjGSM > (J+1) * stepjLOVE:
            wj2 = ( (j+1) * stepjGSM - (J+1) * stepjLOVE ) / stepjGSM
            wj1 = 1 - wj2

        topoLOVE[I, J] += wi1 * wj1 * topoGSM2D[i, j]
        surfLOVE[I, J] += wi1 * wj1 * surfGSM2D[i, j]
        w[I, J] += wi1 * wj1

        try:
            topoLOVE[I+1, J] += wi2 * wj1 * topoGSM2D[i, j]
            surfLOVE[I+1, J] += wi2 * wj1 * surfGSM2D[i, j]
            w[I+1, J] += wi2 * wj1
        except:
            pass
        try:
            topoLOVE[I, J+1] += wi1 * wj2 * topoGSM2D[i, j]
            surfLOVE[I, J+1] += wi1 * wj2 * surfGSM2D[i, j]
            w[I, J+1] += wi1 * wj2
        except:
            pass
        try:
            topoLOVE[I+1, J+1] += wi2 * wj2 * topoGSM2D[i, j]
            surfLOVE[I+1, J+1] += wi2 * wj2 * surfGSM2D[i, j]
            w[I+1, J+1] += wi2 * wj2
        except:
            pass


        if wj2 != 0:
            J += 1

    if wi2 != 0:
        I += 1


## Apply weight and save
h2     = np.zeros((iLOVE, jLOVE+1))
for i in range(iLOVE):
    for j in range(jLOVE):
        h2[i,j] = topoLOVE[i,j] / w[i,j]
    h2[i,jLOVE] = h2[i,0]

np.savetxt('h2.dat', h, "%6.2f")


iLOVE = 36  ## lat  5.0 deg
jLOVE = 36  ## lon 10.0 deg

slopex     = np.zeros((iLOVE, jLOVE))
slopey     = np.zeros((iLOVE, jLOVE))

R = 6371000  ## Radius if Earth [m]
dy = R*(np.pi*(180/iLOVE/2)/180)

for i in range(1,iLOVE-1):
    ## dx assumed to be the same for two neighbour rows in 2x res.
    ## 2.8125: 5.625/2 (2x res. -> 0.5*dx)
    lat = np.pi*(-90+i*(180/iLOVE))/180
    r = R * np.cos(lat)
    dx = r*(np.pi*(360/jLOVE/2)/180)

    for j in range(jLOVE):
        slopex[i,j] = ( (h2[2*i,2*j]-h2[2*i,2*j-1]) + (h2[2*i+1,2*j]-h2[2*i+1,2*j-1]) + (h2[2*i,2*j+2]-h2[2*i,2*j+1]) + (h2[2*i+1,2*j+2]-h2[2*i+1,2*j+1]) ) / (4*dx)
        slopey[i,j] = ( (h2[2*i,2*j]-h2[2*i-1,2*j]) + (h2[2*i,2*j+1]-h2[2*i-1,2*j+1]) + (h2[2*i+2,2*j]-h2[2*i+1,2*j]) + (h2[2*i+2,2*j+1]-h2[2*i+1,2*j+1]) ) / (4*dy)

np.savetxt('slopex10x5.dat', slopex, "%0.10f")
np.savetxt('slopey10x5.dat', slopey, "%0.10f")


###################################################################################


## Slope 15x7.5
iLOVE = 48  ## lat 3.75 deg
jLOVE = 48  ## lon 7.5  deg

## Size of x and y steps in fine (GSM) res. grid
stepiGSM = 180.0 / iGSM
stepjGSM = 360.0 / jGSM

## Size of x and y steps in coarse (LOVECLIM) res. grid
stepiLOVE = 180.0 / iLOVE
stepjLOVE = 360.0 / jLOVE


## Arrays to store LOVECLIM res. data and weight factors
topoLOVE  =  np.zeros((iLOVE, jLOVE))
surfLOVE  =  np.zeros((iLOVE, jLOVE))
w         =  np.zeros((iLOVE, jLOVE))

I = 0
for i in np.arange(0, iGSM-1):
    wi1 = 1
    wi2 = 0

    if (i+1) * stepiGSM > (I+1) * stepiLOVE:
        wi2 = ( (i+1) * stepiGSM - (I+1) * stepiLOVE ) / stepiGSM
        wi1 = 1 - wi2

    J = 0
    for j in np.arange(0, jGSM-1):
        wj1 = 1
        wj2 = 0

        if (j+1) * stepjGSM > (J+1) * stepjLOVE:
            wj2 = ( (j+1) * stepjGSM - (J+1) * stepjLOVE ) / stepjGSM
            wj1 = 1 - wj2

        topoLOVE[I, J] += wi1 * wj1 * topoGSM2D[i, j]
        surfLOVE[I, J] += wi1 * wj1 * surfGSM2D[i, j]
        w[I, J] += wi1 * wj1

        try:
            topoLOVE[I+1, J] += wi2 * wj1 * topoGSM2D[i, j]
            surfLOVE[I+1, J] += wi2 * wj1 * surfGSM2D[i, j]
            w[I+1, J] += wi2 * wj1
        except:
            pass
        try:
            topoLOVE[I, J+1] += wi1 * wj2 * topoGSM2D[i, j]
            surfLOVE[I, J+1] += wi1 * wj2 * surfGSM2D[i, j]
            w[I, J+1] += wi1 * wj2
        except:
            pass
        try:
            topoLOVE[I+1, J+1] += wi2 * wj2 * topoGSM2D[i, j]
            surfLOVE[I+1, J+1] += wi2 * wj2 * surfGSM2D[i, j]
            w[I+1, J+1] += wi2 * wj2
        except:
            pass


        if wj2 != 0:
            J += 1

    if wi2 != 0:
        I += 1


## Apply weight and save
h2     = np.zeros((iLOVE, jLOVE+1))
for i in range(iLOVE):
    for j in range(jLOVE):
        h2[i,j] = topoLOVE[i,j] / w[i,j]
    h2[i,jLOVE] = h2[i,0]

np.savetxt('h2.dat', h, "%6.2f")


iLOVE = 24  ## lat  7.5 deg
jLOVE = 24  ## lon 15.0 deg

slopex     = np.zeros((iLOVE, jLOVE))
slopey     = np.zeros((iLOVE, jLOVE))

R = 6371000  ## Radius if Earth [m]
dy = R*(np.pi*(180/iLOVE/2)/180)

for i in range(1,iLOVE-1):
    ## dx assumed to be the same for two neighbour rows in 2x res.
    ## 2.8125: 5.625/2 (2x res. -> 0.5*dx)
    lat = np.pi*(-90+i*(180/iLOVE))/180
    r = R * np.cos(lat)
    dx = r*(np.pi*(360/jLOVE/2)/180)

    for j in range(jLOVE):
        slopex[i,j] = ( (h2[2*i,2*j]-h2[2*i,2*j-1]) + (h2[2*i+1,2*j]-h2[2*i+1,2*j-1]) + (h2[2*i,2*j+2]-h2[2*i,2*j+1]) + (h2[2*i+1,2*j+2]-h2[2*i+1,2*j+1]) ) / (4*dx)
        slopey[i,j] = ( (h2[2*i,2*j]-h2[2*i-1,2*j]) + (h2[2*i,2*j+1]-h2[2*i-1,2*j+1]) + (h2[2*i+2,2*j]-h2[2*i+1,2*j]) + (h2[2*i+2,2*j+1]-h2[2*i+1,2*j+1]) ) / (4*dy)

np.savetxt('slopex15x75.dat', slopex, "%0.10f")
np.savetxt('slopey15x75.dat', slopey, "%0.10f")


###################################################################################


## Slope 20x10
iLOVE = 36  ## lat 10.0 deg
jLOVE = 36  ## lon 20.0 deg

## Size of x and y steps in fine (GSM) res. grid
stepiGSM = 180.0 / iGSM
stepjGSM = 360.0 / jGSM

## Size of x and y steps in coarse (LOVECLIM) res. grid
stepiLOVE = 180.0 / iLOVE
stepjLOVE = 360.0 / jLOVE


## Arrays to store LOVECLIM res. data and weight factors
topoLOVE  =  np.zeros((iLOVE, jLOVE))
surfLOVE  =  np.zeros((iLOVE, jLOVE))
w         =  np.zeros((iLOVE, jLOVE))

I = 0
for i in np.arange(0, iGSM-1):
    wi1 = 1
    wi2 = 0

    if (i+1) * stepiGSM > (I+1) * stepiLOVE:
        wi2 = ( (i+1) * stepiGSM - (I+1) * stepiLOVE ) / stepiGSM
        wi1 = 1 - wi2

    J = 0
    for j in np.arange(0, jGSM-1):
        wj1 = 1
        wj2 = 0

        if (j+1) * stepjGSM > (J+1) * stepjLOVE:
            wj2 = ( (j+1) * stepjGSM - (J+1) * stepjLOVE ) / stepjGSM
            wj1 = 1 - wj2

        topoLOVE[I, J] += wi1 * wj1 * topoGSM2D[i, j]
        surfLOVE[I, J] += wi1 * wj1 * surfGSM2D[i, j]
        w[I, J] += wi1 * wj1

        try:
            topoLOVE[I+1, J] += wi2 * wj1 * topoGSM2D[i, j]
            surfLOVE[I+1, J] += wi2 * wj1 * surfGSM2D[i, j]
            w[I+1, J] += wi2 * wj1
        except:
            pass
        try:
            topoLOVE[I, J+1] += wi1 * wj2 * topoGSM2D[i, j]
            surfLOVE[I, J+1] += wi1 * wj2 * surfGSM2D[i, j]
            w[I, J+1] += wi1 * wj2
        except:
            pass
        try:
            topoLOVE[I+1, J+1] += wi2 * wj2 * topoGSM2D[i, j]
            surfLOVE[I+1, J+1] += wi2 * wj2 * surfGSM2D[i, j]
            w[I+1, J+1] += wi2 * wj2
        except:
            pass


        if wj2 != 0:
            J += 1

    if wi2 != 0:
        I += 1


## Apply weight and save
h2     = np.zeros((iLOVE, jLOVE+1))
for i in range(iLOVE):
    for j in range(jLOVE):
        h2[i,j] = topoLOVE[i,j] / w[i,j]
    h2[i,jLOVE] = h2[i,0]

np.savetxt('h2.dat', h, "%6.2f")


iLOVE = 18  ## lat 10.0 deg
jLOVE = 18  ## lon 20.0 deg

slopex     = np.zeros((iLOVE, jLOVE))
slopey     = np.zeros((iLOVE, jLOVE))

R = 6371000  ## Radius if Earth [m]
dy = R*(np.pi*(180/iLOVE/2)/180)

for i in range(1,iLOVE-1):
    ## dx assumed to be the same for two neighbour rows in 2x res.
    ## 2.8125: 5.625/2 (2x res. -> 0.5*dx)
    lat = np.pi*(-90+i*(180/iLOVE))/180
    r = R * np.cos(lat)
    dx = r*(np.pi*(360/jLOVE/2)/180)

    for j in range(jLOVE):
        slopex[i,j] = ( (h2[2*i,2*j]-h2[2*i,2*j-1]) + (h2[2*i+1,2*j]-h2[2*i+1,2*j-1]) + (h2[2*i,2*j+2]-h2[2*i,2*j+1]) + (h2[2*i+1,2*j+2]-h2[2*i+1,2*j+1]) ) / (4*dx)
        slopey[i,j] = ( (h2[2*i,2*j]-h2[2*i-1,2*j]) + (h2[2*i,2*j+1]-h2[2*i-1,2*j+1]) + (h2[2*i+2,2*j]-h2[2*i+1,2*j]) + (h2[2*i+2,2*j+1]-h2[2*i+1,2*j+1]) ) / (4*dy)

np.savetxt('slopex20x10.dat', slopex, "%0.10f")
np.savetxt('slopey20x10.dat', slopey, "%0.10f")


