#!/bin/bash
# ---------------------------
#comments:
# This script can generate huge files.
# EX: 500-yr simulation:

# iveget=1 :
#----------
#  75M     mod_veget2D.nc
#  2.4M    mod_vegetGL.nc
#  1.2M    mod_vegetZON.nc

# iatmos=1 : (pic à 1.6Gb)
#-----------
#  1.9M    mod_atmos3DT4EVOWP.nc
#  1.5M    mod_atmos3DU3EVOWP.nc
#  5.2M    mod_atmosEVOWP.nc
#  94M     mod_atmosEVOYL.nc

# iocean=1 : 
#-----------
#  330M  mod_ocean0015003D.nc
#  330M  mod_ocean0016003D.nc
#  330M  mod_ocean0017003D.nc
#  330M  mod_ocean0018003D.nc
#  330M  mod_ocean0019003D.nc
#  825M  mod_ocean2D.nc
#  9.6M  mod_oceanVBASIN.nc
#  895K  mod_oceanBASIN1D.nc
#  408K  mod_oceanEVOA.nc
#  806K  mod_oceanEVOA_modminmax.nc
#  total=2.5Gb
#-------------------------------------------------------------------------------
uname -n

#get argument and global variabe
texp=$1
time=$2
timefin=$3
long=$(( ${timefin} - ${time} ))
dt=$4
iatmos=$5
iocean=$6 ; ioceanevo=$7
iveget=$8
exp={EXP}
Rundir={RUNDIR}
paramdir=${Rundir}/expdir/$texp/$texp.param
ncodir={NCODIR}
cdodir={CDODIR}

#----------------
#Environment:

                   tmpdir1=/home/astr
[ -d /scratch ] && tmpdir1=scratch

tmpdir=/$tmpdir1/${USER}/post_${texp}_$exp
src=${Rundir}/tools/atlas/src
post=${Rundir}/tools/atlas/src/post

source ${paramdir}
#----------------

[ ! -d /scratch ] && mkdir -p /$tmpdir1/${USER}/
rm -rf $tmpdir
mkdir -p $tmpdir
cd $tmpdir

[ ${time} -lt 100000 ] && time0=0$time
[ ${time} -lt 10000 ] && time0=00$time
[ ${time} -lt 1000 ] && time0=000$time

last=$(( ${timefin} - ${dt} ))
[ ${last} -lt 100000 ] && last0=0$last
[ ${last} -lt 10000 ] && last0=00$last
[ ${last} -lt 1000 ] && last0=000$last

#----------------
#Functions:

function ERR {

 $@

  [ $? -ne 0 ] && echo "ERROR: $@" && rm -rf && exit

   SIZE "$2" ; SIZE "$3" ; SIZE "$4" ; SIZE "$5" ; SIZE "$6" ; SIZE "$7"

   }

function SIZE {


 if [ ${#1} -gt 0 ] && [ -f "$1" ] ; then
    size=`du -sk "$1" | awk '{print $1}'`
    [ $size -ge 5000000 ] && echo "ERROR: size of file $1 > 5Go" && exit
 fi

}

climato() 
{
for month in {"01","02","03","04","05","06","07","08","09","10","11","12"}; do
  ${ncodir}/ncra -F -d time,$month,,12 $1 temp$month.nc
done
${ncodir}/ncrcat -O -h temp*nc $2
rm -rf temp??.nc
}

################################1.ATMOSPHERE########################## --> 4 fichiers en sortie
if [ $iatmos -eq 1 ] ; then
 echo "######ATMOSPHERE#########"

 file1=atmmmyl
 file2=atmmmwp
 comp=atmos
 type=atmos

 echo "postprocessing of MODELE 2D -3D"
 #1.1 MODELE 2D - 3D (niveaux U3 et T4)
 #-------------
 file_mod=mod_${comp}

 listcat=" "
 listcat2=" "
 listcat3=" "
 listcat4=" "
 xx=$time
 compt=0

 rsync -azq ${SavingPath}/LOVECLIM${loveclimver}/${texp}/${exp}/output/${type}/* .

 while [ ${xx} -lt ${timefin} ] ; do
   [ ${xx} -lt 100000 ] && xx0=0$xx
   [ ${xx} -lt 10000 ] && xx0=00$xx
   [ ${xx} -lt 1000 ] && xx0=000$xx

   [ ! -f ${file1}${xx0}_001.nc ] && echo "${file1}${xx0}_001.nc not found" && exit
   [ ! -f ${file2}${xx0}_001.nc ] && echo "${file2}${xx0}_001.nc not found" && exit

   #atmmmyl pour pp et ts
   ERR ${cdodir}/cdo selvar,ts,pp ${file1}${xx0}_001.nc modele${xx0}.bin
   
   #atmmmwp pour plus de variables --> 	ts pp snow evap tsr alb ttr bm runoffl q r
   ERR ${cdodir}/cdo selvar,ts,pp,snow,evap,tsr,alb,ttr,bm,runoffl,q,r ${file2}${xx0}_001.nc modelewp${xx0}.bin

   #atmmmwp pour 3D niveaux U3 et T4 --> geopg et tair
   ERR ${cdodir}/cdo selvar,geopg ${file2}${xx0}_001.nc modele3D${xx0}U3.bin
   ERR ${cdodir}/cdo selvar,tair  ${file2}${xx0}_001.nc modele3D${xx0}T4.bin

   listcat="${listcat} modele${xx0}.bin"
   listcat2="${listcat2} modelewp${xx0}.bin"
   listcat3="${listcat3} modele3D${xx0}T4.bin"
   listcat4="${listcat4} modele3D${xx0}U3.bin"
   xx=$((${xx}+${dt}))
   compt=$((${compt}+1))
 done

 if [ ${compt} -gt 1 ] ; then
  ERR ${ncodir}/ncrcat -O -h ${listcat} out1.nc
  ${ncodir}/ncap2 -O -F -s 'for(*i=1; i<=$time.size; i++)time(i)=i;' out1.nc ${file_mod}EVOYL.nc
  ERR ${ncodir}/ncrcat -O -h ${listcat2} out1.nc
  ERR climato out1.nc ${file_mod}EVOWP.nc
  ERR ${ncodir}/ncrcat -O -h ${listcat3} out1.nc
  ERR climato out1.nc ${file_mod}3DT4EVOWP.nc
  ERR ${ncodir}/ncrcat -O -h ${listcat4} out1.nc
  ERR climato out1.nc ${file_mod}3DU3EVOWP.nc
 else
  mv modele${xx0}.bin ${file_mod}EVOYL.nc
  mv modelewp${xx0}.bin ${file_mod}EVOWP.nc
  mv modele3D${xx0}T4.bin ${file_mod}3DT4EVOWP.nc
  mv modele3D${xx0}U3.bin ${file_mod}3DU3EVOWP.nc
 fi

 rm -rf atmmm*nc
 rm -rf *.bin
 rm -rf out?.nc
 rm -rf icesheet*
 rm -rf oc*
fi
################################2.OCEAN##########################
if [ $iocean -eq 1 ] ; then
 echo "######OCEAN#########"

 file0=evolu
 file1=CLIO3m
 file2=CLIO3a
 comp=ocean
 type=ocean
 prog_mod3D=cres3D
 prog_mod2Dz=cres2Dz
 prog_mod2D=cres
 prog_mod1Dz=cres1Dz
 prog_mod0D=read_evolu
 inputfile0D=${file0D}.${texp}00${time}


 #2.1 MODELE 2D surface - 3D - 2D vert/zonal - 1D zonal
 #-----------------------------------------------------

 file_mod=mod_${comp}
 
 listcat=" "
 listcat2=" "
 listcat3=" "
 listcat4=" "
 xx=$last
 [ $ioceanevo -eq 1 ] && xx=$time
 compt=0

 rsync -azq ${SavingPath}/LOVECLIM${loveclimver}/${texp}/${exp}/output/${type}/* .

 while [ ${xx} -lt ${timefin} ] ; do
   xx0=$xx
   [ ${xx} -lt 100000 ] && xx0=0$xx
   [ ${xx} -lt 10000 ] && xx0=00$xx
   [ ${xx} -lt 1000 ] && xx0=000$xx

   [ ! -f ${file1}.${xx0}_001.nc ] && echo "${file1}.${xx0}_001.nc not found" && exit
   [ ! -f ${file2}.${xx0}_001.nc ] && echo "${file2}.${xx0}_001.nc not found" && exit
   mv ${file1}.${xx0}_001.nc ${file1}.${texp}.${xx0}_001.nc
   mv ${file2}.${xx0}_001.nc ${file2}.${texp}.${xx0}_001.nc

   #surface:
   ERR ${src}/${comp}/${prog_mod2D} ${file1} ${texp} ${xx0} ${dt} ${Rundir}
   mv ${file1}.${texp}.${xx0}_0012D.bin modele${xx0}.bin

   #vertical 2D:
   ERR ${src}/${comp}/${prog_mod2Dz} ${file2} ${texp} ${xx0} ${dt} ${Rundir}
   mv ${file2}.${texp}.${xx0}_0012Dz.bin modele${xx0}2Dz.bin

   #3D:
   ERR ${src}/${comp}/${prog_mod3D} ${file2} ${texp} ${xx0} ${dt} ${Rundir}
   mv ${file2}.${texp}.${xx0}_0013D.bin modele${xx0}3D.bin
   #ERR ${post}/graph modele${xx0}3D.bin ${file_mod}${xx0}3D.nc

   #1D:
   ERR ${src}/${comp}/${prog_mod1Dz} ${file2} ${texp} ${xx0} ${dt} ${Rundir}
   mv ${file2}.${texp}.${xx0}_0011Dz.bin modele${xx0}1D.bin

   listcat="${listcat} modele${xx0}.bin"
   listcat2="${listcat2} modele${xx0}2Dz.bin"
   listcat3="${listcat3} modele${xx0}3D.bin"
   listcat4="${listcat4} modele${xx0}1D.bin"
   xx=$((${xx}+${dt}))
   compt=$((${compt}+1))
   rm -rf ${file1}.${texp}${xx0}.nc
   rm -rf ${file2}.${texp}${xx0}.nc
 done

 if [ ${compt} -gt 1 ] ; then
  ERR ${post}/cat_file ${compt} ${listcat} modeleevo.bin
  ERR ${post}/cat_file ${compt} ${listcat2} modeleevo2Dz.bin
  ERR ${post}/cat_file ${compt} ${listcat3} modeleevo3D.bin
  ERR ${post}/cat_file ${compt} ${listcat4} modeleevo1D.bin
 else
  mv modele${xx0}.bin modeleevo.bin
  mv modele${xx0}2Dz.bin modeleevo2Dz.bin
  mv modele${xx0}3D.bin modeleevo3D.bin
  mv modele${xx0}1D.bin modeleevo1D.bin
 fi

 ERR ${post}/graph modeleevo.bin ${file_mod}2D.nc
 ERR ${post}/graph modeleevo3D.bin ${file_mod}3D.nc
 ERR ${post}/graph modeleevo2Dz.bin ${file_mod}VBASIN.nc
 ERR ${post}/graph modeleevo1D.bin  ${file_mod}BASIN1D.nc

# rm -rf modele*.bin
 rm -rf choiceg.dat

 #2.2 MODELE0D (evolu)
 #-------------------
 type=veget
 xx=$time

 rsync -azq ${SavingPath}/LOVECLIM${loveclimver}/${texp}/${exp}/output/${type}/evolu* .

 while [ ${xx} -lt ${timefin} ] ; do
  [ ${xx} -lt 100000 ] && xx0=0$xx
  [ ${xx} -lt 10000 ] && xx0=00$xx 
  [ ${xx} -lt 1000 ] && xx0=000$xx

  rm -rf evolu.${xx0}_001.nc
  [ ! -f evolu.${xx0}_001 ] && echo "evolu.${xx0}_001 not found" && exit
  mv evolu.${xx0}_001 evolu.${texp}.${xx0}_001

  xx=$((${xx}+${dt}))
 done

 ERR ${src}/${comp}/post_evolu_an ${texp} ${Rundir}
 ERR ${src}/${comp}/${prog_mod0D} evolu 1
 ERR ${post}/graph evolu.bin ${file_mod}EVOA.nc

 rm -rf evolu.bin

 ERR ${src}/${comp}/post_evolu_max ${texp} ${Rundir}
 ERR ${src}/${comp}/${prog_mod0D} evolu 1
 ERR ${post}/graph evolu.bin ${file_mod}EVOA_modminmax.nc

 rm -rf ${file2}*nc
 rm -rf ${file1}*nc
 rm -rf ${file0}*
 rm -rf cres.in

fi

################################2.VEGET##########################
if [ $iveget -eq 1 ] ; then
 echo "######VEGET#########"

 file1=veget
 file0=evolu
 comp=veget
 type=veget
 prog_mod2D=read_veget2D
 inputfile2D=${file1}.${last0}_001.outp
 prog_mod1D=read_veget1D
 inputfile1D=${file1}.${last0}_001.zav

 file_data=data_${comp}
 file_mod=mod_${comp}

 #2.1 MODELE1D
 #-------------

 listcat=" "
 listcat2=" "
 listcat3=" "
 listcat4=" "
 listcat5=" "
 xx=${time}
 compt=0

 rsync -azq ${SavingPath}/LOVECLIM${loveclimver}/${texp}/${exp}/output/${type}/* .

 while [ ${xx} -lt ${timefin} ] ; do
   xx0=$xx
   [ ${xx} -lt 100000 ] && xx0=0$xx
   [ ${xx} -lt 10000 ] && xx0=00$xx
   [ ${xx} -lt 1000 ] && xx0=000$xx

   echo  ${xx} ${texp} ${type} ${xx0} ${comp}

   [ ! -f evolu.${xx0}_001 ] && echo "evolu.${xx0}_001 not found" && exit
   [ ! -f ${comp}.${xx0}_001.outp ] && echo "${comp}.${xx0}_001.outp not found" && exit
   [ ! -f ${comp}.${xx0}_001.zav ] && echo "${comp}.${xx0}_001.zav not found" && exit

   echo "Convert Evolu Out"
   ${Rundir}/tools/EvoluCat2NetCDF evolu.${xx0}_001 evolu.${xx0}_001.nc

   inputfile1D=${file1}.${xx0}_001.zav
   inputfile2D=${file1}.${xx0}_001.outp

   #Global et zonal:
   ERR ${src}/${comp}/${prog_mod1D} ${inputfile1D} modele0D${xx0}
   #2D:
   ERR ${src}/${comp}/${prog_mod2D} ${inputfile2D} modele2D${xx0}
   mv gdd.nc gdd${xx0}.nc

   compt=$((${compt}+1))
   listcat="${listcat} modele0D${xx0}GL.bin"
   listcat2="${listcat2} modele2D${xx0}.bin"
   listcat3="${listcat3} modele0D${xx0}.bin"
   listcat4="${listcat4} evolu.${xx0}_001.nc"
   listcat5="${listcat5} vegym${xx0}_001.nc"
   listcat6="${listcat6} gdd${xx0}.nc"
   xx=$((${xx}+${dt}))

 done

 if [ ${compt} -gt 1 ] ; then
   ERR ${post}/cat_file ${compt} ${listcat} modele0DGL.bin
   ERR ${post}/cat_file ${compt} ${listcat3} modele0D.bin
   ERR ${post}/cat_file ${compt} ${listcat2} modele2D.bin
   ERR ${ncodir}/ncrcat -O -h ${listcat4} out1.nc
   ${ncodir}/ncap2 -O -F -s 'for(*i=1; i<=$time.size; i++)time(i)=i;' out1.nc ${file_mod}_evolu.nc
   ERR ${ncodir}/ncrcat -O -h ${listcat5} out1.nc
   ${ncodir}/ncap2 -O -F -s 'for(*i=1; i<=$time.size; i++)time(i)=i;' out1.nc ${file_mod}.nc
 else
   mv modele0D${xx0}GL.bin modele0DGL.bin
   mv modele0D${xx0}.bin modele0D.bin
   mv modele2D${xx0}.bin modele2D.bin
   mv evolu.${xx0}_001.nc ${file_mod}_evolu.nc
   mv vegym${xx0}_001.nc ${file_mod}.nc
 fi

 ERR ${ncodir}/ncrcat -O -h ${listcat6} out1.nc
 ${ncodir}/ncap2 -O -F -s 'for(*i=1; i<=$time.size; i++)time(i)=i;' out1.nc ${file_mod}_gdd.nc
 ERR ${post}/graph modele0DGL.bin ${file_mod}GL.nc
 ERR ${post}/graph modele0D.bin ${file_mod}ZON.nc
 ERR ${post}/graph modele2D.bin ${file_mod}2D.nc

 rm -rf *.bin
 rm -rf atmmm*
 rm -rf veget*outp
 rm -rf veget*zav
 rm -rf *_001.nc
 rm -rf out1.nc
 rm -rf ${file0}*
 rm -rf gdd*
fi

cd ..
  
ERR tar -cvf post_${texp}_${exp}.tar post_${texp}_${exp}

#determine loveclim version for storage name
loveclimver=`echo "${loveclimdir}" | sed "s/.*LOVECLIM\///g; s/\///g"`
if [ ${loveclimver:0:1} == "V" ]; then
  loveclimver=${loveclimver:1:$(( ${#loveclimver}-1 ))}
fi
ERR scp post_${texp}_${exp}.tar ${SavingPath}/LOVECLIM${loveclimver}/${texp}/${exp}

rm -rf post_${texp}_${exp}.tar
rm -rf post_${texp}_${exp}
  
