#include <iostream>
#include <iomanip>
#include <sstream>
#include <fstream>
#include <vector>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <sys/time.h>
#include <unistd.h>


using namespace std;

double randg(double sigma)
{
  double x,y,r2;
  static const double m=2./RAND_MAX;
 
  if(sigma==0) return 0.;
  do{
    x=rand()*m-1.;
    y=rand()*m-1.;
    r2=x*x+y*y;
  }while(r2>1.0 || r2==0);
  return sigma*y*sqrt(-2.*log(r2)/r2);
}

int nbValue(ifstream &fichier)
{
  string line;
  getline(fichier, line);
  int nbvalue=0;
  bool debutdemot=false;
  size_t oldPos=0;
  size_t Pos=line.find(" ",0);
  if(Pos!=0) nbvalue++;
  while(Pos!=string::npos)
  {
    if(oldPos!=Pos-1 && line.size()-1!=Pos) nbvalue++;
    oldPos=Pos;
    Pos=line.find(" ",Pos+1);
  }
  fichier.seekg (0, ios::beg);
  return nbvalue;
}

void usage()
{
   cerr<<"Usage :"<<endl;
   cerr<<"\tNoisyVolc [inVOLC.DAT] [outVOLC.DAT] [sigma] (-e Epsylon)"<<endl;
   cerr<<"\tNoisyVolc [outVOLC.DAT] [sigma]  (-e Epsylon)"<<endl; 
   cerr<<"\tThe optional value Epsylon have 0 for default value"<<endl;
}

class volc
{
  public:
    int year;
    int month;
    float vol1;
    float vol2;
    float vol3;
    float vol4;
    float alea;
    volc(){;}
    volc(int Y, int M, float V1, float V2, float V3, float V4){ year=Y; month=M; vol1=V1; vol2=V2; vol3=V3; vol4=V4;}
};



int main(int argc, char *argv[])
{
  float epsylon=0;
  double sigma;
  if(argc>=3 && argc<=6)
  {
    if(argc==6)
    {
      string option=argv[4];
      if(option=="-e")
      {
        epsylon=atof(argv[5]);
      }else{
        usage();
        return 1;
      }
      sigma=atof(argv[3]);
    }else if(argc==5)
    {
      string option=argv[3];
      if(option=="-e")
      {
        epsylon=atof(argv[4]);
      }else{
        usage();
        return 1;
      }
      sigma=atof(argv[2]);
    }else if(argc==4)
    {
      sigma=atof(argv[3]);
    }else{
      sigma=atof(argv[2]); 
    }
  }else{
    usage();
    return 1;
  }
  cout<<"Sigma="<<sigma<<" Epsylon="<<epsylon<<endl;
    
  struct timeval usec;
  gettimeofday(&usec, NULL); 
  srand((unsigned int)usec.tv_usec);
  vector<volc> volcan;
  
  //READ OR GENERATE VOLC.DAT DATA
  ifstream fin(argv[1]);
  bool volcExtended;
  
  if(argc==3)
  {
    for(int y=1; y<=2001; y++)
      for(int m=1; m<=12; m++)
        volcan.push_back(volc(y,m,0,0,0,0));
  }else if(!fin.fail()) 
  {
    //volcExtended est le cas où les données supplémentaire sont dans le fichier de Volcan
    if(nbValue(fin)==7)
    {
      volcExtended=true;
    }else{
      volcExtended=false;
    }
       
    char buf[512];
    while(fin>>buf) //annee
    {
      volc tempvolcan;
      tempvolcan.year=atoi(buf);
      fin>>buf; tempvolcan.month=atoi(buf);
      fin>>buf; tempvolcan.vol1=atof(buf);
      fin>>buf; tempvolcan.vol2=atof(buf);
      fin>>buf; tempvolcan.vol3=atof(buf);
      fin>>buf; tempvolcan.vol4=atof(buf);
      if(volcExtended)
      {
        fin>>buf; tempvolcan.alea=atof(buf);        
      }
      volcan.push_back(tempvolcan);
    }
    
  }else{
      cerr<<"Error during read "<<argv[1]<<" file"<<endl;
  }
 
  
  //COMPUTE MEAN AND NOISE FOR VOLC DATA
  float alea=randg(sigma);
  cout<<"random(t)="<<alea<<endl;
  if(volcExtended)
  {
    cout<<"random(t-1)="<<volcan[0].alea<<endl;
    cout<<"random_total(t)="<<alea<<"+"<<epsylon<<"*"<<volcan[0].alea<<"=";
    alea+=epsylon*volcan[0].alea;
    cout<<alea<<endl;
  }else{
    cout<<"random_total(t)="<<alea<<endl;
  }
  for(int i=0; i<(int)volcan.size(); i++)
  {
    volcan[i].vol1+=alea;
    volcan[i].vol2+=alea;
    volcan[i].vol3+=alea;
    volcan[i].vol4+=alea;
  }
  
  //WRITE VOLC DATA IN A NEW FILE
  ofstream fout;
  if(argc==3)
  {
    fout.open(argv[1]);
  }else{
    fout.open(argv[2]);  
  }
  if(fout)
  {
    for(int i=0; i<(int)volcan.size(); i++)
    {
      fout<<setiosflags(ios::right)<<setw(5)<<volcan[i].year<<setw(3)<<volcan[i].month<<" ";
      fout<<setiosflags(ios::fixed|ios::right)<<setprecision(3)<<setw(8)<<volcan[i].vol1<<" ";
      fout<<setiosflags(ios::fixed|ios::right)<<setprecision(3)<<setw(8)<<volcan[i].vol2<<" ";
      fout<<setiosflags(ios::fixed|ios::right)<<setprecision(3)<<setw(8)<<volcan[i].vol3<<" ";
      fout<<setiosflags(ios::fixed|ios::right)<<setprecision(3)<<setw(8)<<volcan[i].vol4<<" ";
      fout<<setiosflags(ios::fixed|ios::right)<<setprecision(5)<<setw(20)<<alea<<endl;
    }
  }else{
      cerr<<"Error during write "<<argv[2]<<" file"<<endl;
  }
  return 0;
}
