MODULE CLIO_PostProcess
  IMPLICIT NONE
  !IMPLICIT DOUBLE PRECISION (a-h,o-z)
  PRIVATE

  PUBLIC :: readCresum

  ! Module Parameters
  INTEGER, PARAMETER :: nio0=122, njo0=65, mfield=6, nlev=20
  INTEGER, PARAMETER :: jmtt=60, imtt=120
  INTEGER, PARAMETER :: imax=nio0, jmax=njo0
  INTEGER, PARAMETER :: nsmax = 2
  REAL(kind=4), PARAMETER :: spv    = -2.0e3
  REAL(kind=8), PARAMETER :: spvw   = spv
  REAL(kind=8), PARAMETER :: pi     = 3.1415926535897932384d0
  REAL(kind=8), PARAMETER :: radian = pi/180.0
  REAL(kind=8), PARAMETER :: degre  = 180.0/pi
  REAL(kind=8), PARAMETER :: untour = 360.0d0
  REAL(kind=8), PARAMETER :: epsil  = 1.0d-10
  REAL(kind=8), PARAMETER :: zero   = 0.0
  REAL(kind=8), PARAMETER :: one    = 1.0
  REAL(kind=8), PARAMETER :: separ  = 0.5 + epsil
  !--(alpha,beta): (latitude,longitude) of the north pole of the new grid.
  REAL(kind=8), PARAMETER :: alpha = 0.0, beta = -111.0
  !--Define original grids.
  REAL(kind=8), PARAMETER :: xi1 = 25.500, dxi = 3.00
  REAL(kind=8), PARAMETER :: yj1 =-78.500, dyj = 3.00

  CHARACTER(len=7), PARAMETER  :: titv(1:mfield) = (/ &
       & 'uoc    ', 'voc    ', 'woc   ', 'toc   ', 'soc   ', &
       & 'doc     ' & 
       & /)
  CHARACTER(len=7), PARAMETER :: units(1:mfield) = (/ &
       & 'm/s      ', 'm/s      ', 'm/s      ', 'K      ', 'psu', &
       & 'kg/cm3      ' &
       & /)
  CHARACTER(len=7), PARAMETER :: titv2(1:mfield) = (/ &
       & 'u velocity ', 'v velocity ', 'w velocity ', 'temperature', 'salinity', &
       & 'density ' &
       & /)



CONTAINS

  SUBROUTINE readCresum
    !
    ! Reads mean outputs from CLIO.
    !
    INTEGER, PARAMETER :: input = 10, output = 8

    REAL(kind=4), DIMENSION(:,:,:,:), ALLOCATABLE :: wdata, wdatai
    INTEGER,      DIMENSION(:),       ALLOCATABLE :: ityp, nfi
    REAL(kind=4) :: data(1:nio0,1:njo0,1:nlev)
    REAL(kind=8) :: ttlon(1:imtt), ttlat(1:jmtt), xlat1, dxlat, ylon1, dylon
    REAL(kind=8) ::T,S 
    INTEGER :: i,ims1,ims2,irec,j,ja,jj,jcl1,jcl2,jmois,jpxy,compt,l
    INTEGER :: k,kfield,m,month,n,nio0p,njo0p,nmef,nmi,nsnm,nmp,nsa,nsi,nmef2
    LOGICAL :: interpolate, output_in_netcdf
    CHARACTER(len=60) :: tit(1:mfield)
    CHARACTER(len=60)  :: name

    PRINT "('Name of the file')"
    READ(unit=*,fmt="(60A)") name

    OPEN(unit=input,file=Trim(name)//".dat",form="unformatted")

    PRINT "('First sample ?')"
    READ(unit=*,fmt=*) month
    PRINT "('Sample every n month ?')"
    READ(unit=*,fmt=*) nsnm
    PRINT "('Average over n samples?')"
    READ(unit=*,fmt=*) nsa
    PRINT "('Number of samples ?')"
    READ(unit=*,fmt=*) nmp
    PRINT "('Interpolate (T) or not (F) ?')"
    READ(unit=*,fmt=*) interpolate
!   IF (interpolate) THEN
!      PRINT "('Output in NetCDF (T) or GrADS (F) format ?')"
!      READ(unit=*,fmt=*) output_in_netcdf
!   ELSE
       output_in_netcdf = .FALSE.
!   END IF


    compt=0 
    nmef=6
!   READ(unit=input) 
    READ(10)
    READ(unit=input) nmef2
    REWIND(unit=input)
    PRINT*, nmef2

    ALLOCATE(wdata (0:nio0,0:njo0,1:nmef,1:nlev))
    ALLOCATE(wdatai(1:imtt,1:jmtt,1:nmef,1:nlev))
    ALLOCATE(ityp  (1:nmef))
    ALLOCATE(nfi   (1:nmef))
    wdata (:,:,:,:) = spv
    wdatai(:,:,:,:) = spv
    ityp(1)=2
    ityp(2)=2
    ityp(3)=1
    ityp(4)=1
    ityp(5)=1
    ityp(6)=1


    !--Reading of the files

    DO m=1,month-1
       READ(unit=input) ja,jmois
       READ(unit=input) nmef2
       DO kfield=1,nmef
             READ(unit=input) data
       END DO
    END DO

    DO nmi=1,nmp
      wdata(:,:,:,:)=0.
     DO nsi=1,nsa
       READ(unit=input) ja,jmois
       READ(unit=input) nmef2
       PRINT*, "Maintained year,month,nmef: ",ja,jmois,UBound(wdata,3)
       DO kfield=LBound(wdata,3),UBound(wdata,3)
             READ(unit=input) data
          DO k=1,nlev
           DO i=1,UBound(data,1)
            DO j=1,UBound(data,2)
              if(data(i,j,k).eq.-100) data(i,j,k)=spv
            ENDDO
           ENDDO
           END DO
           wdata(LBound(data,1):UBound(data,1),LBound(data,2):UBound(data,2), &
     &    kfield,LBound(data,3):UBound(data,3)) = &
     &    (data(:,:,:)/nsa)+ &
     &     wdata(LBound(data,1):UBound(data,1),LBound(data,2):UBound(data,2), &
     &     kfield,LBound(data,3):UBound(data,3))
           if ((kfield.eq.6).and.(nsi.eq.nsa)) then
           DO k=1,nlev
            DO i=1,UBound(data,1)
             DO j=1,UBound(data,2)
              if (wdata(i,j,4,k).eq.spv) then
                wdata(i,j,kfield,k)=spv 
              else
               T=wdata(i,j,4,k)-273.15
               S=wdata(i,j,5,k)
               wdata(i,j,kfield,k)=999.842594+(T*6.793952e-02)-&
     &         (9.095290e-3*T**2)+(1.001685e-4*T**3)-&
     &         (1.120083e-6*T**4)+(6.536332e-9*T**5)+&
     &         S*(0.824493-(4.0899e-3*T)+(7.6438e-5*&
     &         T**2)-(8.2467e-7*T**3)+(5.3875e-9*&
     &         T**4))+(S*1.5)*(0-5.72466e-3+(1.0227e-4*&
     &         T)-(1.6546e-6*T**2))+4.8314e-4*&
     &         S**2
              endif
             ENDDO
            ENDDO
           ENDDO 
           endif

        END DO
        ityp(1)=2
        ityp(2)=2
        ityp(3)=1
        ityp(4)=1
        ityp(5)=1
        ityp(6)=1


       END DO
       IF (nmi < nmp) THEN 
          DO jj=1,nsnm-1
             READ(unit=input) ja,jmois
             READ(unit=input) nmef2
             DO kfield=LBound(wdata,3),UBound(wdata,3)
                READ(unit=input) nfi(kfield),ityp(kfield),tit(kfield)
                DO j=LBound(data,2),UBound(data,2)
                 DO k=1,nlev
                   READ(unit=input) data(:,j,k)
                 END DO 
                END DO
             END DO
        
          END DO
       END IF
   IF (nmi.eq.nmp) then
     CLOSE(unit=input)
   ENDIF

    ! Cyclic correspondance
    ims1 = 2
    ims2 = imax-1
    jcl1 = 2
!   jcl2 = 28
    jcl2 = 64 
    DO i=LBound(wdata,3),UBound(wdata,3)
      DO k=1,nlev
          DO j=jcl1,jcl2
             wdata(ims1-1, j,i,k) = wdata(ims2,j,i,k)
             wdata(ims2+1, j,i,k) = wdata(ims1,j,i,k)
             if (k.eq.nlev) write(*,*)wdata(ims1-1, j,i,k),i
             wdata(    55,50,i,k) = spv
             wdata(    56,50,i,k) = spv
          END DO
          wdata(:,65,i,k) = spv
          wdata(:,1,i,k) = spv
      END DO
    END DO

    ! Interpolation or not
    IF (interpolate) THEN
       nio0p = imtt
       njo0p = jmtt
       CALL gridtt(ttlon(:),ttlat(:))
       xlat1 = ttlat(1)
       dxlat = ttlat(2) - ttlat(1)
       ylon1 = ttlon(1)
       dylon = ttlon(2) - ttlon(1)

       CALL mercator(Size(ttlon),ttlon(:),Size(ttlat),ttlat(:),Size(wdatai,3), &
            & Size(wdatai,4), &
            & wdata(:,:,:,:),wdatai(:,:,:,:),ityp(:))

  
       if (nmi.eq.1) then
                 do j=1,nio0p
                  do l=1,njo0p
                  enddo
               enddo
       endif

       DO n=LBound(wdatai,3),UBound(wdatai,3)
          IF (ityp(n) == 2) THEN
             PRINT*, "Vector: ",tit(n)
             PRINT*, "Vector: ",tit(n+1)
              DO k=1,nlev
                CALL mercatv(Size(ttlon),ttlon,Size(ttlat),ttlat,Size(wdata,3), & 
             & Size(wdatai,4), &
             &        wdata(:,:,:,:),wdatai(:,:,:,:),ityp(:),n,n+1,k)
              END DO
             ityp(n)   = 1
             ityp(n+1) = 1
          END IF
       END DO
    ELSE
       nio0p = nio0
       njo0p = njo0
       xlat1 =-78.500
       dxlat = 3.0
       ylon1 = 25.500
       dylon = 3.0
    END IF

    ! Writing of the fields
!   IF (output_in_netcdf .AND. interpolate) THEN
!      CALL outputInNetCDF(nfi(:),tit(1:nmef),wdatai(1:nio0p,1:njo0p,:,:))
!   ELSE
       jpxy=nio0p*njo0p*nlev*4
       IF (nmi.eq.1 ) then
      OPEN(unit=output,file="grads_"//Trim(name)//".dat", form="unformatted", &
            & access="direct",recl=jpxy)
       irec = 1
       ENDIF 
          DO i=LBound(wdata,3),UBound(wdata,3)
             IF (interpolate) THEN
                WRITE(unit=output,rec=irec) wdatai(1:nio0p,1:njo0p,i,1:nlev)
                irec = irec + 1
             ELSE
                WRITE(unit=output,rec=irec) wdata (1:nio0p,1:njo0p,i,1:nlev)
                irec = irec + 1
             END IF
          END DO
       
       
!   END IF
    
    END DO
    CLOSE(unit=output)

    OPEN(unit=output,file="grads_"//Trim(name)//".ctl")
       WRITE(unit=output,fmt="('dset ^',a)") "grads_"//Trim(name)//".dat"
       WRITE(unit=output,fmt="('options big_endian')")
       WRITE(unit=output,fmt="('undef ',1p,e12.4)") spv
       WRITE(unit=output,fmt="('title test of grads')")
       WRITE(unit=output,fmt="('xdef ',i4,' linear ',2f7.2)") nio0p,ylon1,dylon
       WRITE(unit=output,fmt="('ydef ',i4,' linear ',2f7.2)") njo0p,xlat1,dxlat
       WRITE(unit=output,fmt="('zdef  20 levels')")
       WRITE(unit=output,fmt="(' 5500     5126.18 4385.22 3661.11 2963.25 2307.36')")
       WRITE(unit=output,fmt="(' 1717.90  1225.11  850.19  588.88  415.07  299.29')")
       WRITE(unit=output,fmt="('  219.86   163.28  121.52   89.75   64.96   45.20')")
       WRITE(unit=output,fmt="('   29.17    15.98 ')") 
       WRITE(unit=output,fmt="('tdef ',i3,' linear 1jan0001 ',i3,'dy')") &
            & nmp,nsnm*30*12
       WRITE(unit=output,fmt="('vars ',i3)") nmef
       DO i=1,nmef
          WRITE(unit=output,fmt="(A7,'   20   99  ',A15,A15)") titv(i),titv2(i),units(i)
       END DO
       WRITE(unit=output,fmt="('endvars')")
       CLOSE(unit=output)


    DEALLOCATE(nfi)
    DEALLOCATE(ityp)
    DEALLOCATE(wdata)
    DEALLOCATE(wdatai)

    RETURN
  END SUBROUTINE readCresum


  SUBROUTINE mercator(imtt,ttlon,jmtt,ttlat,nmef,nlev,wdata,wdatai,ityp)
    INTEGER,      INTENT(in)    :: imtt, jmtt, nmef, ityp(1:nmef),nlev
    REAL(kind=8), INTENT(in)    :: ttlon(1:imtt),ttlat(1:jmtt)
    REAL(kind=4), INTENT(in)    :: wdata (0:nio0,0:njo0,1:nmef,1:nlev)
    REAL(kind=4), INTENT(inout) :: wdatai(1:imtt,1:jmtt,1:nmef,1:nlev)
    !
    !--Interpolation of scalar data placed at the center of the
    !--elements of a two rectangular grid onto one grid.
    !--Longitude-latitude coordinates for both grids.
    !--Data are outputs of the program OTI.
    !--This subroutine is identical to mercat.f except for input and ouputs
    !
    !--M.A.Morales Maqueda, 11-IV-1994.
    !--Modified by H.GOOSSE 15-IV-1994
    !--Modified by H.GOOSSE + M.A.M. Maqueda 15-IV-1994
    !--Modified by H.GOOSSE 16-V-1994
    !  modif : 14/10/94

    REAL(kind=8) :: gwlon(0:imax), gwlat(0:jmax+1)
    REAL(kind=8) :: galat(0:imax), galon(0:jmax+1)
    LOGICAL      :: whigri(LBound(ttlon,1):UBound(ttlon,1),LBound(ttlat,1):UBound(ttlat,1))
    REAL(kind=8) :: dcxw, dcyw, dd, dl, dr, dsxw, dsyw, du, dxa, dxw, dya, dyw
    REAL(kind=8) :: gxw, rd, rl, rr, ru, unsdtx, unsdty, xw, xxx, yw, yyy
    REAL(kind=8) :: valad, valau, valc, valcd, valcl, valcr, valcu
    INTEGER      :: i, iwp, j, jwp, k, nn, nn0, nn1, nprt

    ! Definition of the new grid
    gwlon(:) = (/ (xi1 + dxi*(i-1),i=LBound(gwlon,1),UBound(gwlon,1)) /)
    galat(:) = 90.0 + beta + untour - gwlon(:)
    gwlat(:) = (/ (yj1 + dyj*(j-1),j=LBound(gwlat,1),UBound(gwlat,1)) /)
    galon(:) = 90.0 + gwlat(:)
    !--Compute the correspondance between the grids.
    CALL choiceg(ttlat(:),ttlon(:),whigri(:,:))

    DO j=LBound(whigri,2),UBound(whigri,2)
       DO i=LBound(whigri,1),UBound(whigri,1)
          IF (whigri(i,j)) THEN
             dxw = ttlon(i)
             dyw = ttlat(j)
             gxw = xi1 + Mod(dxw-xi1+untour,untour)
             xxx = (gxw - xi1)/dxi + 0.5
             iwp = NInt(xxx) 
             iwp = Max(0,Min(imax-1,iwp))
             dr  = gwlon(iwp+1) - gxw
             dl  = gxw - gwlon(iwp)
             yyy = (dyw - yj1)/dyj + 0.5
             jwp = NInt(yyy) 
             jwp = Max(0,Min(jmax,jwp))
             du  = gwlat(jwp+1) - dyw
             dd  = dyw - gwlat(jwp)
          ELSE
             yw   = ttlat(j)
             dyw  = yw*radian
             dsyw = Sin(dyw)
             dcyw = Cos(dyw)
             xw   = Mod(ttlon(i)-beta,untour)
             dxw  = xw*radian
             dsxw = Sin(dxw)
             dcxw = Cos(dxw)
             !--Compute coordinates on the ss grid of a point of the aa grid.
             dya  = Asin(dcyw*dcxw)*degre
             dxa  = Atan2(dcyw*dsxw,-dsyw)*degre
             dxa  = Mod(dxa+untour,untour)
             yyy  = (dxa - (90.0 + yj1))/dyj + 0.5
             jwp  = NInt(yyy) 
             jwp  = Max(0,Min(jmax,jwp))
             du   = galon(jwp+1) - dxa
             dd   = dxa - galon(jwp)
             xxx  = (dya - (90.0 + beta + untour - xi1))/(-dxi) + 0.5
             iwp  = NInt(xxx) 
             iwp  = Max(0,Min(imax-1,iwp))
             dr   = galat(iwp+1) - dya
             dl   = dya - galat(iwp)
          END IF


          IF (ttlon(i) >= 369.0 .OR. ttlon(i) <= -1.0) THEN
             nprt = nprt + 1
             IF (nprt >= nn0 .AND.  nprt <= nn1) THEN
                WRITE(unit=99,fmt=*) 'nprt, whigri(i,j) :', nprt, whigri(i,j)
                WRITE(unit=99,fmt="('i,j, iwp, jwp :')")
                WRITE(unit=99,fmt=*) i,j, iwp, jwp
                WRITE(unit=99,fmt="('dl, dr, dd, du :')")
                WRITE(unit=99,fmt=*) dl, dr, dd, du
                IF (whigri(i,j)) THEN
                   WRITE(unit=99,fmt="('dxw, dyw :')")
                   WRITE(unit=99,fmt=*) dxw, dyw
                ELSE
                   WRITE(unit=99,fmt="('dxa, dya :')")
                   WRITE(unit=99,fmt=*) dxa, dya 
                END IF
             END IF
          END IF

          ! Interpolation a partir des 4 voisins iwp/iwp+1,jwp/jwp+1 .
          !--Point (i,j). A point is considered to be a land point 
          !--if the nearest data point is a land point.
          unsdtx = 1.0/(dl + dr)
          rl     = dl*unsdtx
          rr     = dr*unsdtx
          unsdty = 1.0/(dd + du)
          rd     = dd*unsdty
          ru     = du*unsdty
          DO k=LBound(wdatai,3),UBound(wdatai,3)
             IF (ityp(k) == 1) THEN
                DO nn=LBound(wdatai,4),UBound(wdatai,4)
                   valcl = wdata(iwp,  jwp,k,nn)
                   valcr = wdata(iwp+1,jwp,k,nn)
                   if (jwp.lt.1) then
                    valcl = spv
                    valcr = spv   
                   endif
                   if ((iwp+1.eq.122).and.(jwp.gt.28)) then
                      valcl = spv
                      valcr = spv
                   endif  
                   
!                  IF ((valcl.eq.0.).and.(jwp /= 0))PRINT*, iwp,jwp,k,nn
!                  IF(valcl /= spv) PRINT*, iwp,jwp,k,nn 
                   IF (valcl == spv) THEN
                      IF (rl <= 0.5) THEN
                         valad = valcl
                      ELSE
                         valad = valcr
                         valcd = valcr
                      END IF
                   ELSE IF (valcr == spv) THEN
                      IF (rr <= 0.5) THEN
                         valad = valcr
                      ELSE
                         valad = valcl
                         valcd = valcl
                      END IF
                   ELSE
                      valad = valcl*rr + valcr*rl
                      valcd = valad
!                  PRINT*,valad,valcd
                   END IF

                   valcl = wdata(iwp,  jwp+1,k,nn)
                   valcr = wdata(iwp+1,jwp+1,k,nn)
                   if ((iwp+1.eq.122).and.(jwp.gt.28)) then
                      valcl = spv
                      valcr = spv
                   endif

                   IF (valcl == spv) THEN
                      IF (rl <= 0.5) THEN
                         valau = valcl
                      ELSE
                         valau = valcr
                         valcu = valcr
                      END IF
                   ELSE IF (valcr == spv) THEN
                      IF (rr <= 0.5) THEN
                         valau = valcr
                      ELSE
                         valau = valcl
                         valcu = valcl
                      END IF
                   ELSE
                      valau = valcl*rr + valcr*rl
                      valcu = valau
                   END IF

                   IF (valad == spv) THEN
                      IF (rd <= 0.5) THEN
                         wdatai(i,j,k,nn) = spv
                      ELSE IF (valau == spv) THEN
                         wdatai(i,j,k,nn) = spv
                      ELSE
                         valc = valcu
                         wdatai(i,j,k,nn) = valcu
                      END IF
                   ELSE IF (valau == spv) THEN
                      IF (ru <= 0.5) THEN
                         wdatai(i,j,k,nn) = spv
                      ELSE
                         valc = valcd
                         wdatai(i,j,k,nn) = valcd
!                        PRINT*,'toto1'

                      END IF
                   ELSE
                      valc = valcd*ru + valcu*rd
                      wdatai(i,j,k,nn) = valc
                    IF ((nn.eq.20).AND.(i.eq.17).and.(j.eq.42)) then
                   ENDIF

                   END IF
                    


                   nn0=999999.99
                   nn1=999999.99
                   !--Pour verification : 
                   IF (ttlon(i) >= 369.0 .OR. ttlon(i) <= -1.0) THEN
                      IF (nprt >= nn0 .AND. nprt <= nn1) THEN
                         WRITE(unit=99,fmt=*) "wdatai(i,j,1,1) =", wdatai(i,j,1,1)
                      END IF
                   END IF

                END DO
             END IF
          END DO
       END DO
    END DO

    RETURN
  END SUBROUTINE mercator
 

  SUBROUTINE gridtt(ttlon,ttlat)
    REAL(kind=8), INTENT(out) :: ttlon(:),ttlat(:)
    !--Define interpolating grid tt.
    !--Latitudes and longitudes correspond to the centers of the grid elements.
    !
    REAL(kind=8) :: delx, dely, xlong1, xlat1
    INTEGER      :: i, j

    delx=360.0/Real(Size(ttlon,1))
    ttlon(:) = Mod( (/ ((i-1)*delx,i=LBound(ttlon,1),UBound(ttlon,1)) /), 360.0 )
    dely=180.0/Real(Size(ttlat,1))
    ttlat(:) = (/ (-90.0 + (j-1+0.5)*dely,j=LBound(ttlat,1),UBound(ttlat,1)) /)

    RETURN
  END SUBROUTINE gridtt


  SUBROUTINE choiceg(ttlat,ttlong,whigri)
    LOGICAL,      INTENT(out) :: whigri(:,:)
    REAL(kind=8), INTENT(in)  :: ttlong(LBound(whigri,1):UBound(whigri,1))
    REAL(kind=8), INTENT(in)  :: ttlat (LBound(whigri,2):UBound(whigri,2))

    ! This routine determine at which original grid (aa or ww)
    ! each point of tt correspond.

    INTEGER      :: i, j
    REAL(kind=8) :: xoncri

    whigri(:,:) = .TRUE.
    DO i=LBound(whigri,1),UBound(whigri,1)
       DO j=LBound(whigri,2),UBound(whigri,2)
          IF (ttlat(j) >  0.0 .AND. ttlat(j) <= 8.0) THEN
             IF (ttlong(i) >= 290.0  .OR. ttlong(i) <  30.0) whigri(i,j) = .FALSE.
          END IF
          IF (ttlat(j) >  8.0 .AND. ttlat(j) <= 10.0) THEN
             xoncri = 281.0 + (ttlat(j) - 8.0)/(10.0 - 8.0)*(276.0 - 281.0)
             IF (ttlong(i) >= xoncri .OR. ttlong(i) <  30.0) whigri(i,j) = .FALSE.
          END IF
          IF (ttlat(j) > 10.0 .AND. ttlat(j) <= 15.0) THEN
             xoncri = 276.0 + (ttlat(j) - 10.0)/(15.0 - 10.0)*(270.0 - 276.0)
             IF (ttlong(i) >= xoncri .OR. ttlong(i) <  30.0) whigri(i,j) = .FALSE.
          END IF
          IF (ttlat(j) > 15.0 .AND. ttlat(j) <= 20.0) THEN
             xoncri = 270.0 + (ttlat(j) - 15.0)/(20.0 - 15.0)*(260.0 - 270.0)
             IF (ttlong(i) >= xoncri .OR. ttlong(i) <  30.0) whigri(i,j) = .FALSE.
          END IF
          IF (ttlat(j) > 20.0 .AND. ttlat(j) <= 30.0) THEN
             xoncri = 260.0 + (ttlat(j) - 20.0)/(30.0 - 20.0)*(260.0 - 260.0)
             IF (ttlong(i) >= xoncri .OR. ttlong(i) <  30.0) whigri(i,j) = .FALSE.
          END IF
          IF (ttlat(j) > 30.0 .AND. ttlat(j) <= 68.0) THEN
             xoncri = 260.0 + (ttlat(j) - 30.0)/(65.0 - 30.0)*(260.0 - 260.0)
             IF (ttlong(i) >= xoncri .OR. ttlong(i) <  50.0) whigri(i,j) = .FALSE.
          END IF
          IF (ttlat(j) > 67.0 .AND. ttlat(j) <= 90.0) THEN
             IF (ttlong(i) >= 0.0    .OR. ttlong(i) < 360.0) whigri(i,j) = .FALSE.
          END IF
       END DO
    END DO

    RETURN
  END SUBROUTINE choiceg


  SUBROUTINE mercatv(imtt,ttlon,jmtt,ttlat,nmef,nlev,wdata,wdatai,ityp,n1,n2,k)
    INTEGER,      INTENT(in)    :: imtt, jmtt, nmef, ityp(1:nmef), n1, n2, k, nlev
    REAL(kind=8), INTENT(inout)    :: ttlon(1:imtt),ttlat(1:jmtt)
    REAL(kind=4), INTENT(inout)    :: wdata (0:nio0,0:njo0,1:nmef,1:nlev)
    REAL(kind=4), INTENT(inout) :: wdatai(1:imtt,1:jmtt,1:nmef,1:nlev)

    !--Interpolation of scalar data placed at the center of the
    !--elements of a two rectangular grid onto one grid.
    !--Longitude-latitude coordinates for both grids.
    !--Data are outputs of the program OTI.
    !
    !--M.A.Morales Maqueda, 11-IV-1994.
    !--Modified by H.GOOSSE 15-IV-1994
    !--Modified by H.GOOSSE + M.A.M. Maqueda 15-IV-1994
    !--Modified by H.GOOSSE 16-V-1994
    !--Modified by JMC 19/09/95, adapted for vector, (derived from "provect").
    !  modif : 21/09/95

    REAL(kind=8), DIMENSION(1:imtt) :: cxt, sxt !LBound(ttlon,1),UBound(ttlon,1)) :: cxt, sxt
    REAL(kind=8), DIMENSION(1:jmtt) :: cyt, syt !LBound(ttlat,1),UBound(ttlat,1)) :: cyt, syt
    LOGICAL      :: whigri(LBound(ttlon,1):UBound(ttlon,1),LBound(ttlat,1):UBound(ttlat,1))
    REAL(kind=8) :: gwlon(0:imax), gwlat(0:jmax+1)
    REAL(kind=8) :: galat(0:imax), galon(0:jmax+1)
    REAL(kind=8) :: cxw(0:imax), sxw(0:imax), cyw(0:jmax+1), syw(0:jmax+1)
    REAL(kind=8) :: cya(0:imax), sya(0:imax), cxa(0:jmax+1), sxa(0:jmax+1)
    REAL(kind=8) :: valx(0:imax,0:jmax+1), valy(0:imax,0:jmax+1)
    REAL(kind=8) :: valz(0:imax,0:jmax+1)
    REAL(kind=8) :: dd, dl, dr, du, dxa, dxw, dya, dyw
    REAL(kind=8) :: gxw, rd, rl, rr, ru, unsdtx, unsdty
    REAL(kind=8) :: valdw, valg, valup, valxd, valxu, valyd, valyu, valzd, valzu, vvx, vvy, vvz
    REAL(kind=8) :: xxx, yyy, ylim, ylim1, ylim2
    INTEGER      :: i, iwp, j, jwp, nprt, nn0, nn1

    valx(:,:) = 0.0
    valy(:,:) = 0.0
    valz(:,:) = 0.0

    ! Definition de la nouvelle grille

    !--Define interpolating grid ww.
    CALL gridtt(ttlon(:),ttlat(:))
    cxt(:) = Cos(radian*(ttlon(:) - beta))
    sxt(:) = Sin(radian*(ttlon(:) - beta))
    cyt(:) = Cos(radian* ttlat(:)        )
    syt(:) = Sin(radian* ttlat(:)        )

    gwlon(:) = (/ (xi1 + dxi*(i-1),i=LBound(gwlon,1),UBound(gwlon,1)) /)
    galat(:) = 90.0 + beta + untour - gwlon(:)
    cxw(:) = Cos(radian*(gwlon(:)-beta))
    sxw(:) = Sin(radian*(gwlon(:)-beta))
    cya(:) = Cos(radian* galat(:)      )
    sya(:) = Sin(radian* galat(:)      )

    gwlat(:) = (/ (yj1 + dyj*Float(j-1),j=LBound(gwlat,1),UBound(gwlat,1)) /)
    galon(:) = 90.0 + gwlat(:)
    cyw(:) = Cos(radian*gwlat(:))
    syw(:) = Sin(radian*gwlat(:))
    cxa(:) = Cos(radian*galon(:))
    sxa(:) = Sin(radian*galon(:))

    !--Compute de correspondance between grids
    CALL choiceg(ttlat(:),ttlon(:),whigri(:,:))

    !--Calcul des 3 composantes dans le repere fixe .
    ylim1 = 69.0
    ylim2 = 294.0
    !- Critere d'apartenace a Grille AA : y > min[ 69., max(0., 294. - x) ]
    DO j=1,jmax
       DO i=1,imax
          ylim = Min(ylim1,Max(zero,ylim2-gwlon(i)))
          IF (gwlat(j) <= ylim) THEN
             !- WW :
             valx(i,j) = -sxw(i)*wdata(i,j,n1,k) - syw(j)*cxw(i)*wdata(i,j,n2,k)
             valy(i,j) =  cxw(i)*wdata(i,j,n1,k) - syw(j)*sxw(i)*wdata(i,j,n2,k)
             valz(i,j) =  cyw(j)*wdata(i,j,n2,k)
          ELSE
             !- AA :
             valz(i,j) =  sxa(j)*wdata(i,j,n2,k) - sya(i)*cxa(j)*wdata(i,j,n1,k)
             valy(i,j) =  cxa(j)*wdata(i,j,n2,k) + sya(i)*sxa(j)*wdata(i,j,n1,k)
             valx(i,j) = -cya(i)*wdata(i,j,n1,k)
          END IF
       END DO
    END DO

    ! Traitement de la nouvelle grille colonne par colonne.

    !--Main do-loop.
    DO j=LBound(whigri,2),UBound(whigri,2)
       DO i=LBound(whigri,1),UBound(whigri,1)
          !--Debut du traitement de la colonne (i,j) :
          IF (whigri(i,j)) THEN
             dxw = ttlon(i)
             dyw = ttlat(j)
             gxw = xi1 + Mod(dxw-xi1+untour,untour)
             xxx = (gxw - xi1)/dxi + 0.5
             iwp = NInt(xxx)
             iwp = Max(0,Min(imax-1,iwp))
             dr  = gwlon(iwp+1) - gxw
             dl  = gxw - gwlon(iwp)
             yyy = (dyw - yj1)/dyj + 0.5
             jwp = NInt(yyy)
             jwp = Max(0,Min(jmax,jwp))
             du  = gwlat(jwp+1) - dyw
             dd  = dyw - gwlat(jwp)
          ELSE
             !--Compute coordinates on the ss grid of a point of the aa grid.
             dya = Asin(cyt(j)*cxt(i))*degre
             dxa = Atan2(cyt(j)*sxt(i),-syt(j)) * degre
             dxa = Mod(dxa+untour,untour)
             yyy = (dxa - (90.0 + yj1))/dyj + 0.5
             jwp = NInt(yyy)
             jwp = Max(0,Min(jmax,jwp))
             du  = galon(jwp+1) - dxa
             dd  = dxa - galon(jwp)
             xxx = (dya - (90.0 + beta + untour - xi1))/(-dxi) + 0.5
             iwp = NInt(xxx)
             iwp = Max(0,Min(imax-1,iwp))
             dr  = galat(iwp+1) - dya
             dl  = dya - galat(iwp)
          END IF
          nn0=999999.99
          nn1=999999.99
          IF (ttlon(i) >= 369.0 .OR. ttlon(i) <= -1.0) THEN
             nprt = nprt + 1
             IF (nprt >= nn0 .AND. nprt <= nn1) THEN
                WRITE(unit=99,fmt=*) "nprt, whigri(i,j) :", nprt, whigri(i,j)
                WRITE(unit=99,fmt="('i,j, iwp, jwp :')")
                WRITE(unit=99,fmt=*) i,j, iwp, jwp
                WRITE(unit=99,fmt="('dl, dr, dd, du :')")
                WRITE(unit=99,fmt=*) dl, dr, dd, du
                IF (whigri(i,j)) THEN
                   WRITE(unit=99,fmt="('dxw, dyw :')")
                   WRITE(unit=99,fmt=*) dxw, dyw
                ELSE
                   WRITE(unit=99,fmt="('dxa, dya :')")
                   WRITE(unit=99,fmt=*) dxa, dya
                END IF
             END IF
          END IF

          ! Interpolation a partir des 4 voisins iwp/iwp+1,jwp/jwp+1 .
          !--Point (i,j). A point is considered to be a land point
          !--if the nearest data point is a land point.
          unsdtx = 1.0/(dl + dr)
          rl     = dl*unsdtx
          rr     = dr*unsdtx
          unsdty = 1.0/(dd + du)
          rd     = dd*unsdty
          ru     = du*unsdty
          if (jwp.lt.1) then
             wdata(iwp,jwp,n1,k) = spv
          endif
          if ((iwp+1.eq.122).and.(jwp.gt.28)) then
              wdata(iwp,jwp,n1,k) = spv
              wdata(iwp+1,jwp,n1,k) = spv
              wdata(iwp+1,jwp+1,n1,k) = spv
              wdata(iwp,jwp+1,n1,k) = spv
          endif
 
          IF (wdata(iwp,jwp,n1,k) == spv) THEN
             IF (rl <= separ) THEN
                valdw = spv
             ELSE
                valdw = wdata(iwp+1,jwp,n1,k)
                valxd = valx(iwp+1,jwp)
                valyd = valy(iwp+1,jwp)
                valzd = valz(iwp+1,jwp)
             END IF
          ELSE IF (wdata(iwp+1,jwp,n1,k) == spv) THEN
             IF (rr <= separ) THEN
                valdw = spv
             ELSE
                valdw = wdata(iwp,jwp,n1,k)
                valxd = valx(iwp,jwp)
                valyd = valy(iwp,jwp)
                valzd = valz(iwp,jwp)
             END IF
          ELSE
             valdw = rr*wdata(iwp,jwp,n1,k)+rl*wdata(iwp+1,jwp,n1,k)
             valxd = rr*valx(iwp,jwp) + rl*valx(iwp+1,jwp)
             valyd = rr*valy(iwp,jwp) + rl*valy(iwp+1,jwp)
             valzd = rr*valz(iwp,jwp) + rl*valz(iwp+1,jwp)
          END IF

          IF (wdata(iwp,jwp+1,n1,k) == spv) THEN
             IF (rl <= separ) THEN
                valup = spv
             ELSE
                valup = wdata(iwp+1,jwp+1,n1,k)
                valxu = valx(iwp+1,jwp+1)
                valyu = valy(iwp+1,jwp+1)
                valzu = valz(iwp+1,jwp+1)
             END IF
          ELSE IF (wdata(iwp+1,jwp+1,n1,k) == spv) THEN
             IF (rr <= separ) THEN
                valup = spv
             ELSE
                valup = wdata(iwp,jwp+1,n1,k)
                valxu = valx(iwp,jwp+1)
                valyu = valy(iwp,jwp+1)
                valzu = valz(iwp,jwp+1)
             END IF
          ELSE
             valup = rr*wdata(iwp,jwp+1,n1,k)+rl*wdata(iwp+1,jwp+1,n1,k)
             valxu = rr*valx(iwp,jwp+1) + rl*valx(iwp+1,jwp+1)
             valyu = rr*valy(iwp,jwp+1) + rl*valy(iwp+1,jwp+1)
             valzu = rr*valz(iwp,jwp+1) + rl*valz(iwp+1,jwp+1)
          END IF

          IF (valdw == spv) THEN
             IF (rd <= separ) THEN
                valg  = spvw
                wdatai(i,j,n1,k)  = spvw
                wdatai(i,j,n2,k)  = spvw
             ELSE IF (valup == spv) THEN
                valg  = spvw
                wdatai(i,j,n1,k)  = spvw
                wdatai(i,j,n2,k)  = spvw
             ELSE
                valg = valup
                wdatai(i,j,n1,k) = -sxt(i)*valxu + cxt(i)*valyu
                wdatai(i,j,n2,k) =  cyt(j)*valzu - syt(j)*(cxt(i)*valxu + sxt(i)*valyu)
             END IF
          ELSE IF (valup == spv) THEN
             IF (ru <= separ) THEN
                valg  = spvw
                wdatai(i,j,n1,k)  = spvw
                wdatai(i,j,n2,k)  = spvw
             ELSE
                valg = valdw
                wdatai(i,j,n1,k) = -sxt(i)*valxd + cxt(i)*valyd
                wdatai(i,j,n2,k) =  cyt(j)*valzd - syt(j)*(cxt(i)*valxd + sxt(i)*valyd)
             END IF
          ELSE
             valg  = rd*valup + ru*valdw
             vvx = rd*valxu + ru*valxd
             vvy = rd*valyu + ru*valyd
             vvz = rd*valzu + ru*valzd
             wdatai(i,j,n1,k) = -sxt(i)*vvx + cxt(i)*vvy
             wdatai(i,j,n2,k) =  cyt(j)*vvz - syt(j)*(cxt(i)*vvx + sxt(i)*vvy)
          END IF
       END DO
    END DO

    RETURN
  END SUBROUTINE mercatv


! SUBROUTINE outputInNetCDF(ititle,descr,data)
!   USE Fields
!   INTEGER,           INTENT(in) :: ititle(:)
!   CHARACTER(len=60), INTENT(in) :: descr(:)
!   REAL(kind=4),      INTENT(in) :: data(:,:,:,:)
!
!   TYPE(Coordinate)     :: lon, lat
!   TYPE(DataFile)       :: file
!   TYPE(Error)          :: err
!   TYPE(Field2D)        :: f
!   TYPE(Grid2D)         :: g
!   TYPE(TimeCoordinate) :: time
!   INTEGER              :: i,j
!   REAL, POINTER        :: field_values(:,:)
!   CHARACTER(len=128)   :: message
!
!   CALL initialize(err)
!   CALL initialize(file,name="cresum.nc",err=err,overwrite=.TRUE.,readwrite=.TRUE.)
!   IF (isSet(err)) THEN
!      CALL fatalError("cresuint: Failed to open NetCDF file cresum.nc")
!   END IF
!
!   CALL initialize(lon,name="lon",description="Longitude",unit="degrees_east", &
!        & values=(/ (      (360.0/Size(data,1))* i,     i=LBound(data,1)-1,UBound(data,1)-1 ) /))
!   CALL initialize(lat,name="lat",description="Latitude",unit="degrees_north", &
!        & values=(/ (-90.0+(180.0/Size(data,2))*(i+0.5),i=LBound(data,2)-1,UBound(data,2)-1 ) /))
!   CALL initialize(g,length=lon,width=lat)
!   CALL initialize(time,name="time",description="Model Time",unit="days since 0001-01-01", &
!        & values=(/ (30.0*(i-0.5),i=1,Size(data,4)) /))
!
!   DO i=LBound(data,3),UBound(data,3)
!      CALL initialize(f,name=Trim(titv(ititle(i))),description=Trim(descr(i)), &
!           & unit=Trim(units(ititle(i))),grid=g,fill_value=spvw,time=time)
!      DO j=LBound(data,4),UBound(data,4)
!         CALL setTimeIndex(f,j)
!         field_values => getValues(f)
!         field_values(:,:) = data(:,:,i,j)
!         CALL write(f,file,err=err)
!         IF (isSet(err)) THEN
!            WRITE(message,fmt="('cresuint: Could not write time step ',i3,' of field ',a)") &
!                 j,Trim(getName(f))
!            CALL fatalError(Trim(message))
!         END IF
!      END DO
!      CALL finalize(f)
!   END DO
!
!   CALL finalize(time)
!   CALL finalize(g)
!   CALL finalize(lat)
!   CALL finalize(lon)
!
!   CALL finalize(file,err=err)
!   IF (isSet(err)) THEN
!      CALL fatalError("cresuint: Failed to close NetCDF file "//Trim(getName(file)))
!   END IF
!   CALL finalize(err)
!   
!   RETURN
! END SUBROUTINE outputInNetCDF

END MODULE CLIO_PostProcess


PROGRAM PostProcess
  USE CLIO_PostProcess
  IMPLICIT NONE
  CALL readCresum
  STOP
END PROGRAM PostProcess

