MODULE CLIO_PostProcess
  IMPLICIT NONE
  !IMPLICIT DOUBLE PRECISION (a-h,o-z)
  PRIVATE

  PUBLIC :: readCresum

  ! Module Parameters
  INTEGER, PARAMETER :: nio0=1, njo0=57, mfield=6, nlev=22
  INTEGER, PARAMETER :: nbas=4
  REAL(kind=4), PARAMETER :: spv    = -1.e+32
  REAL(kind=4), PARAMETER :: zero   = 0.0
  REAL(kind=4), PARAMETER :: one    = 1.0
  !--Define original grids.
  REAL(kind=4), PARAMETER :: xi1 = 0.000, dxi = 3.00
  REAL(kind=4), PARAMETER :: yj1 =-81.000, dyj = 3.00

CONTAINS

  SUBROUTINE readCresum
    !
    ! Reads mean outputs from CLIO.
    !
    INTEGER, PARAMETER :: input = 10, output = 8

    REAL(kind=4), DIMENSION(:,:,:,:), ALLOCATABLE :: wdata
    REAL(kind=4) :: data(1:nbas,1:njo0,1:nlev)

    INTEGER :: i,j,l,k,kfield
    INTEGER :: m,month,n,nmef,nmi,nsnm,nmp,nsa,nsi,tt
    CHARACTER(len=60)  :: name
    character*30 varname3D(mfield),uname(mfield)
    character*50 lname(mfield)


    j=1

    varname3D(j)='moc'
    uname(j)='Sv'
    lname(j)='Merid_overturning_streamfunction'

!   j=j+1
!   varname3D(j)='mht'
!   uname(j)='PW'
!   lname(j)='Merid_heat_transport'
    
!   j=j+1
!   varname3D(j)='mst'
!   uname(j)='m/s'
!   lname(j)='Merid_salt_transport'

    nmef=j

    print*,'Read Ocean Model output - 2Dz (read_cres2Dz.f)'

    PRINT "('Name of the file')"
    READ(unit=*,fmt="(60A)") name

    PRINT "('First sample ?')"
    READ(unit=*,fmt=*) month
    PRINT "('Sample every n month ?')"
    READ(unit=*,fmt=*) nsnm
    PRINT "('Average over n samples?')"
    READ(unit=*,fmt=*) nsa
    PRINT "('Number of samples ?')"
    READ(unit=*,fmt=*) nmp


    ALLOCATE(wdata (0:nbas,0:njo0,1:nmef,1:nlev))
    wdata (:,:,:,:) = spv

    !--Reading of the files

    

    DO kfield=1,nmef
     tt=month
     DO nmi=1,nmp
      wdata(:,:,:,:)=0.
      DO nsi=1,nsa
             call CF_READ3D(TRIM(name)//'.nc',varname3D(kfield),tt,nbas,njo0,nlev,data)
!            do k=1,nlev ; do j=1,njo0; do i=1,1
!              write(*,*)data(k,j,i),k,j,i
!            enddo      ; enddo      ; enddo   
           wdata(LBound(data,1):UBound(data,1),LBound(data,2):UBound(data,2), &
     &    kfield,LBound(data,3):UBound(data,3)) = &
     &    (data(:,:,:)/nsa)+ &
     &     wdata(LBound(data,1):UBound(data,1),LBound(data,2):UBound(data,2), &
     &     kfield,LBound(data,3):UBound(data,3))
          tt=tt+1

        END DO
        tt=tt+(nsnm-nsa)

        do j=1,njo0 ; do i=1,nbas ; do k=1,nlev
         if(wdata(i,j,1,k).eq.0.) wdata(i,j,1,k)=spv
         if(wdata(i,j,1,k).eq.spv) wdata(i,j,kfield,k)=spv
        enddo      ; enddo        ; enddo

       IF ((nmi.eq.1).and.(kfield.eq.1) ) then

       open (unit=output, FILE=TRIM(name)//'2Dz.bin', FORM='UNFORMATTED') 
       write(unit=output)nbas,njo0,nlev,nmp,xi1,dyj,yj1,dxi,nmef,spv
       write(*,*)nbas,njo0,nlev,nmp,xi1,dyj,yj1,dxi,nmef,spv
       write(unit=output)(varname3D(n),n=1,nmef)
       write(unit=output)(lname(n),n=1,nmef)
       write(unit=output)(uname(n),n=1,nmef)

       ENDIF 

       do i=1,nbas ; do j=1,njo0 ; do k=1,nlev
          WRITE(unit=output) wdata (i,j,kfield,k)
       enddo        ; enddo        ; enddo
       
    END DO

   END DO

    CLOSE(unit=output)



    DEALLOCATE(wdata)

    write(*,*)'End of read_cres2Dz'
    write(*,*)'--------------------------------------'


    RETURN
  END SUBROUTINE readCresum



END MODULE CLIO_PostProcess


PROGRAM PostProcess
  USE CLIO_PostProcess
  IMPLICIT NONE
  CALL readCresum
  STOP
END PROGRAM PostProcess

