! 
! Original code Y. Damas-sallaz
! Modified by P. Mathiot (October 2011) : large modification about loop (read 2D file) => strong speed up
!
subroutine usage()
  write(*,'(A)') "Usage:"
  write(*,'(A)') "  ./Boxifaire (-nb) [idxlon_min] [idxlon_max] [idxlat_min] [idxlat_max] [input.nc] [output.nc]"
  write(*,'(A)') "  ./Boxifaire (-nb) [input.nc] [output.nc]  <-- Box is entire world"
  write(*,'(A)') "  -nb = with nb point in mean is stored"
  stop
end subroutine usage

include "NetcdfFonction.f90"

PROGRAM Boxifaire 
  USE NETCDF
  IMPLICIT NONE

  interface
    subroutine CopyVariableFrame(intputID, outputID, tabsuffix, tabIdxDimNeeded, replace, valundef)
      USE NETCDF
      integer :: intputID, outputID
      character(nf90_max_name), dimension(:), allocatable, optional :: tabsuffix
      integer, dimension(:), allocatable, optional :: tabIdxDimNeeded
      integer, optional :: replace
      double precision, dimension(nf90_max_var_dims), optional :: valundef
    end subroutine CopyVariableFrame
  end interface

  CHARACTER(nf90_max_name) :: filenameIn, filenameOut
 
  INTEGER :: i, j, t, nRecords, idxRec, idxLon, idxLat, outputvarID, status
  INTEGER, DIMENSION(:), allocatable :: start, ncount

  integer :: intputID, outputID, outdimid, RecordDimID, LonDimID, LatDimID
  integer :: nbDim, nbVar, getLonOrLatID, getTime
  integer :: nbvarDim, dimSize, variableType
  integer, dimension(nf90_max_var_dims) :: varDimID
  double precision, dimension(nf90_max_var_dims) :: undefMinTab, undefMaxTab, undef
  character(nf90_max_name) :: varName, dimName, parm

  double precision, dimension(:), allocatable :: valueDbl1D
  double precision, dimension(:,:), allocatable :: valueDbl2D
  double precision, dimension(:,:,:), allocatable :: valueDbl3D
  double precision, dimension(:,:), allocatable :: coslat2D
  real, dimension(:), allocatable :: Lat, Lon
  integer, dimension(:,:,:), allocatable :: nmask3D
  integer, dimension(:,:), allocatable :: nmask2D

  integer :: totaltime, totalLat, totalLon, totalOtherDim

  character(nf90_max_name), dimension(nf90_max_var_dims) :: dimNameList

  !variable pour les box
  integer :: lon_min, lon_max, lat_min, lat_max
  integer, dimension(:,:), allocatable :: deftab
  integer, dimension(:,:,:), allocatable :: deftab3D
  double precision :: radian
  character(nf90_max_name), dimension(:), allocatable :: tabsuffix
  integer, dimension(:), allocatable :: tabIdxDimNeeded
  logical :: l_nb

  !GET ARGUMENT FOR SELECT METHODE
  if(iargc().ge.2) then
    call getarg(1,parm)
    if(parm.eq."-nb") then
      l_nb=.true.
      if(iargc().eq.7) then
        call getarg(2,parm); read(parm,*) lon_min
        call getarg(3,parm); read(parm,*) lon_max
        call getarg(4,parm); read(parm,*) lat_min
        call getarg(5,parm); read(parm,*) lat_max
        call getarg(6,filenameIn)
        call getarg(7,filenameOut)
      elseif(iargc().eq.3) then
        lon_min=-1
        lon_max=-1
        lat_min=-1
        lat_max=-1
        call getarg(2,filenameIn)
        call getarg(3,filenameOut)
      else
        call usage()
      endif
    else
      l_nb=.false.
      if(iargc().eq.6) then
        call getarg(1,parm); read(parm,*) lon_min
        call getarg(2,parm); read(parm,*) lon_max
        call getarg(3,parm); read(parm,*) lat_min
        call getarg(4,parm); read(parm,*) lat_max
        call getarg(5,filenameIn)
        call getarg(6,filenameOut)
      elseif(iargc().eq.2) then
        lon_min=-1
        lon_max=-1
        lat_min=-1
        lat_max=-1
        call getarg(1,filenameIn)
        call getarg(2,filenameOut)
      else
        call usage()
      endif
    endif
  else
    call usage()
  endif
  
  radian=2*acos(0.0)/180.0

  write(*,'(A)',ADVANCE='NO') "Open input file..."
  call check(nf90_open(filenameIn, nf90_nowrite, intputID))
  write(*,*) "OK"

  write(*,'(A)',ADVANCE='NO') "   Get some value..."
  !get nbdim and nbvar
  call check(nf90_inquire(intputID, nbDim, nbVar))
  !get totaltime
  totaltime=getTime(intputID, RecordDimID)
  !get undef value
  do i=1, nbVar
    call GetUndefForVar(intputID, i, undef(i), undefMinTab(i), undefMaxTab(i))
  enddo
  !deg to radian conversion
  radian=2*acos(0.0)/180.0
  write(*,*) "OK"

  !get longitude name
  write(*,'(A)',ADVANCE='NO') "   Get Longitude dimension..."
  LonDimID=getLonOrLatID(intputID,1)
  if(LonDimID.eq.-1) then
    write(*,'(A)') "Not Found"
    stop
  else
    call check(nf90_inquire_dimension(intputID, LonDimID, name = dimName, len = dimSize))
    allocate(lon(dimSize))
    call check(nf90_get_var(intputID, LonDimID, Lon, (/ 1 /), (/ dimSize /)))
    totalLon=dimSize
    write(*,'(A,A,A)') "OK(", trim(dimName), ")"
  endif

  !get latitude name
  write(*,'(A)',ADVANCE='NO') "   Get Latitude dimension..."
  LatDimID=getLonOrLatID(intputID,0)
  if(LatDimID.eq.-1) then
    write(*,'(A)') "Not Found"
    stop
  else
    call check(nf90_inquire_dimension(intputID, LatDimID, name = dimName, len = dimSize))
    allocate(lat(dimSize))
    call check(nf90_get_var(intputID, LatDimID, Lat, (/ 1 /), (/ dimSize /)))
    totalLat=dimSize
    write(*,'(A,A,A)') "OK(", trim(dimName), ")"
  endif


  write(*,'(A)',ADVANCE='NO') "Define box..."
  allocate(deftab(size(lon),size(lat)))
  call MakeBox(size(lon), size(lat), lon_min, lon_max, lat_min, lat_max, deftab)
  write(*,*) lon_min, lon_max, lat_min, lat_max


  write(*,'(A)') "Create output file :"
  call check(nf90_create(trim(filenameOut), nf90_hdf5, outputID))   !write target file
  
  
  !define dimension, variable associated and attribut
  write(*,'(A)',ADVANCE='NO') "   Define dimensions..."
  do i=1, nbDim
    !get name and size of dimension
    call check(nf90_inquire_dimension(intputID, i, name = dimName, len = dimSize))
    dimNameList(i)=dimName

    !define dimension with separation between unlimited, normal dimension and Lon/Lat
    if(i.eq.RecordDimID) then
      call check(nf90_def_dim(outputID, dimName, NF90_UNLIMITED, outdimid))
    elseif(i.eq.LonDimID) then
      call check(nf90_def_dim(outputID, dimName, 1, outdimid))
    elseif(i.eq.LatDimID) then
      call check(nf90_def_dim(outputID, dimName, 1, outdimid))
    else
      call check(nf90_def_dim(outputID, dimName, dimSize, outdimid))
    endif
  enddo
  write(*,*) "OK"


  !define variable from the list
  write(*,'(A)',ADVANCE='NO') "   Define variables and copy attributs..."
  if(l_nb.eq. .true. ) then
    allocate(tabsuffix(1)); tabsuffix(1)="_nbBox"
    allocate(tabIdxDimNeeded(3)); tabIdxDimNeeded(1)=LatDimID; tabIdxDimNeeded(2)=LonDimID; tabIdxDimNeeded(3)=RecordDimID
    call CopyVariableFrame(intputID, outputID, tabsuffix, tabIdxDimNeeded)
  else
    call CopyVariableFrame(intputID, outputID)
  endif
  write(*,*) "OK"

  status = nf90_enddef(outputID)

  !copy dimension data


  !copy dimension data
  write(*,'(A)',ADVANCE='NO') "Caculate cos value to speed up..."
  allocate(coslat2D(totalLon, totalLat))
  do i=1, totalLat
    coslat2D(:,i)=cos(lat(i)*radian)
  enddo
  write(*,*) "OK"

  !copy value 
  outputvarID=1
  do i=1, nbVar !begin var loop
    call check(nf90_inquire_variable(intputID, i, varName, xtype = variableType, ndims = nbvarDim, dimids = varDimID))
    allocate(start(nbvarDim), ncount(nbvarDim))
    start=1
    ncount=1
    totalOtherDim=1
    idxRec=-1; idxLon=-1; idxLat=-1
    do j=1, nbvarDim
      call check(nf90_inquire_dimension(intputID, varDimID(j), DimName, nRecords))
      if(varDimID(j).eq.RecordDimID) then  
        ncount(j)=1
        idxRec=j
      elseif(varDimID(j).eq.LonDimID) then
        ncount(j)=nRecords
        idxLon=j
      elseif(varDimID(j).eq.LatDimID) then  
        ncount(j)=nRecords
        idxLat=j
      else
        ncount(j)=nRecords
        totalOtherDim=totalOtherDim*ncount(j)
      endif
    enddo
    allocate(deftab3D(totalOtherDim, totalLon, totalLat))
    do j=1, totalOtherDim
      deftab3D(j,:,:)=deftab(:,:)
    enddo

    
    !check if variable contain lon, lat and time
    if((idxLon.ne.-1).and.(idxLat.ne.-1).and.(idxRec.ne.-1)) then
      write(*,'(A,A,d,A)',ADVANCE='NO') trim(varName), " is processing (undef=~", undef(i), ")..."
      
      !allocate variable
      allocate(valueDbl2D(totalOtherDim,totaltime))
      allocate(nmask2D(totalOtherDim,totaltime))
      allocate(valueDbl3D(totalOtherDim, totalLon, totalLat))
      allocate(nmask3D(totalOtherDim, totalLon, totalLat))


      !begin time loop
      do t=1, totaltime
        !initialisation
        nmask3D=0       
        !read time slide by time slide 
        start(idxRec)=t
        call check(nf90_get_var(intputID, i, valueDbl3D, start=start, count=ncount))
        !make a mask
        WHERE(((valueDbl3D .LT. undefMinTab(i)) .OR. (valueDbl3D .GT. undefMaxTab(i))) .AND. (deftab3D==1))
             nmask3D=1
        END WHERE

        !compute the mean value over the box
        valueDbl2D(:,t)=undef(i)
        DO j=1,totalOtherDim
          nmask2D(j,t)=SUM(nmask3D(j,:,:))
          IF (nmask2D(j,t) .NE. 0) valueDbl2D(j,t)=SUM(valueDbl3D(j,:,:)*coslat2D(:,:)*nmask3D(j,:,:))/SUM(coslat2D(:,:)*nmask3D(j,:,:))
        END DO

      END DO !end time loop

      !write the mean value in netcdf file
      start(idxRec)=1 ; ncount(idxRec)=totaltime
      start(idxLon)=1 ; ncount(idxLon)=1
      start(idxLat)=1 ; ncount(idxLat)=1
      
      if( l_nb .eq. .true. ) then
          call check(nf90_put_var(outputID, outputvarID, valueDbl2D, start, ncount))
          call check(nf90_put_var(outputID, outputvarID+1, nmask2D, start, ncount))
          outputvarID=outputvarID+2
      else
          call check(nf90_put_var(outputID, i, valueDbl2D, start=start, count=ncount))
          outputvarID=outputvarID+1
      endif

      !deallocate variable
      deallocate(valueDbl2D, nmask2D)
      deallocate(valueDbl3D, nmask3D)
      
      write(*,'(A)') "OK"
    else
      write(*,'(A,A)') trim(varName), " is copied"
      if(idxLon.ne.-1) ncount(idxLon)=1
      if(idxLat.ne.-1) ncount(idxLat)=1
      if(idxRec.ne.-1) ncount(idxRec)=totaltime
      allocate(valueDbl1D(product(ncount)))
      call check(nf90_get_var(intputID, i, valueDbl1D, start, ncount))
      call check(nf90_put_var(outputID, outputvarID, valueDbl1D, start, ncount))
      deallocate(valueDbl1D)
      outputvarID=outputvarID+1
    endif
    deallocate(start,ncount)
    deallocate(deftab3D)
  enddo ! end var loop
  
  call check(nf90_close(intputID));
  PRINT *,"close file out ..."
  call check(nf90_close(outputID));
END PROGRAM Boxifaire

