program butter
  USE NETCDF
  implicit none
  
  interface
    subroutine buttap(PolR, PolI, k, n)
      integer :: n
      double precision, dimension(:), allocatable :: PolR, PolI
      double precision :: k
    end subroutine buttap
    subroutine zp2ss(a, b, c, d, PolR, n)
      double precision, dimension(:,:), allocatable :: a
      double precision, dimension(:), allocatable :: b, c, PolR
      double precision :: d
      integer :: n
    end subroutine zp2ss
    subroutine lp2lp(a, b, Wn)
      double precision, dimension(:,:), allocatable :: a
      double precision, dimension(:), allocatable :: b
      double precision :: Wn
    end subroutine lp2lp
    subroutine bilinear(a, b, c, d, fs, n)
      double precision, dimension(:,:), allocatable :: a
      double precision, dimension(:), allocatable :: b, c
      double precision :: d, fs
      integer :: n
    end subroutine bilinear
    subroutine poly(c, x, n)
      integer :: n
      double precision, dimension(:,:), allocatable :: x
      double precision, dimension(:), allocatable :: c
    end subroutine poly
    subroutine buttnum(b, den, n)
      double precision, dimension(:), allocatable :: b, den
      integer :: n
    endsubroutine buttnum
    subroutine filtfilt(y,b,a,x)
      double precision, dimension(:), allocatable :: y, b, a, x
    endsubroutine filtfilt
    subroutine filter(y,b,a,x)
      double precision, dimension(:), allocatable :: y, b, a, x
    endsubroutine filter
  end interface
  
  !variable pour le filtre
  double precision :: fs, u, pi, pi_2, Wn, k, d
  integer :: N
  double precision, dimension(:,:), allocatable :: a
  double precision, dimension(:), allocatable :: PolR, PolI, b, c, den, num, inputdata, outputdata

  !variable pour le netcdf
  integer :: nAtts, annual, isDim
  REAL, DIMENSION(:), POINTER :: pvalue
  CHARACTER(nf90_max_name) :: filenameIn, filenameOut, varfiltered, parm
  INTEGER, DIMENSION(:), ALLOCATABLE :: pdimSize, dimSizeExt
  INTEGER :: ncidIn, ncidOut, status
  INTEGER :: nbDim, nbVar, unlimDimID, nbvarDim, varfilteredID
  INTEGER, DIMENSION(nf90_max_var_dims) :: varDimID
  character(len = nf90_max_name) :: DimName, VarName, attname
  character(len = nf90_max_name), DIMENSION(:), allocatable :: DimNameList
  INTEGER :: ii, jj, kk, ll, mm, month, nRecords, outdimid, outvarid, vartype
  INTEGER, DIMENSION(:), allocatable :: start, varDimIDshorttab
  REAL(kind=4), DIMENSION(:,:,:,:), ALLOCATABLE :: var
  REAL(kind=4) :: PeriodeFiltrage
  integer :: deflate_level
  deflate_level=1
  pi=dacos(-1.D0)
  pi_2=dacos(0.D0)
  
  if((iargc().ne.5).and.(iargc().ne.6)) then
    write(*,*) "./butter [input.nc] [output.nc] [variable_name/all] [Periode] [order] (an)"
    stop
  endif

  
  !GET FILE NAME
  call getarg(1,filenameIn)   
  call getarg(2,filenameOut)
  call getarg(3,varfiltered)
  call getarg(4,parm)
  read(parm,*)PeriodeFiltrage
  call getarg(5,parm)
  read(parm,*)N
  if(iargc().eq.6) then
    call getarg(6,parm)
    if(trim(parm).eq."an") then
      annual=1
    else
      annual=0
    endif
  else
    annual=0
  endif

  !COMPUTE Butterworth coefficient
  Wn=2.D0/PeriodeFiltrage !2*1/T   si fitrage sur 10 ans --> 2*1/10=0.2
  
  fs=2.D0
  u=2.D0*fs*tan(pi*Wn/fs)
  Wn=u

  call buttap(PolR, PolI, k, N)
  call zp2ss(a, b, c, d, PolR, N)
  call lp2lp(a, b, Wn)
  call bilinear(a, b, c, d, fs, n)
  call poly(den, a, n)
  call buttnum(num, den, n)
  
  write(*,*) "Butterworth coefficient :"
  call Print_Vector("den=", N+1, den)
  call Print_Vector("num=", N+1, num)
  write(*,*)
  
  !read source file
  status = nf90_open(filenameIn, nf90_nowrite, ncidIn)
  if(status /= nf90_NoErr) then
    write(*,*) "Error during open file", NF90_STRERROR(status)
    stop
  endif
    
  !write target file
  status = nf90_create(filenameOut, nf90_hdf5, ncidOut)
  if(status /= nf90_NoErr) then
    write(*,*) "Error during writing file", NF90_STRERROR(status)
    stop
  endif
    
  !get number of dimension and the dimid unlimited
  status = nf90_inquire(ncidIn, nbDim, nbVar, unlimitedDimId = unlimDimID)
  if(status /= nf90_NoErr) then
    write(*,*) "Error during dimension inquire", NF90_STRERROR(status)
    stop
  endif
  
  !copy dimension name
  allocate(DimNameList(nbDim))
  do ii=1, nbDim
      status = nf90_inquire_dimension(ncidIn, ii, DimName, nRecords)
      DimNameList(ii)=trim(DimName)
      if(ii.eq.unlimDimID) then
        status = nf90_def_dim(ncidOut, trim(DimName), NF90_UNLIMITED, outdimid)
      else
        status = nf90_def_dim(ncidOut, trim(DimName), nRecords, outdimid)
      endif
  enddo
  
  !nbVar=4
  
  !copy variable name
  do ii=1, nbVar
      status = nf90_inquire_variable(ncidIn, ii, VarName, vartype, nbvarDim, varDimID, nAtts)
      allocate(varDimIDshorttab(nbvarDim))
      varDimIDshorttab=varDimID
      status = nf90_def_var(ncidOut, trim(VarName), vartype, varDimIDshorttab, outvarid, shuffle = .TRUE., deflate_level=deflate_level)
      deallocate(varDimIDshorttab)
      if(status /= nf90_NoErr) then
        write(*,*) "Error during variable definition -> ",trim(VarName) , NF90_STRERROR(status)
        stop
      endif
      if(trim(varfiltered).eq.trim(VarName)) then
        varfilteredID=ii
      endif
      do jj=1, nAtts
        status = nf90_inq_attname(ncidIn, ii, jj, attname);
        status = nf90_copy_att(ncidIn, ii, attname, ncidOut, ii)
      enddo
  enddo
  if(trim(varfiltered).eq."all") varfilteredID=-1
 
  !finish the configuration of the output file and starting put data
  status = nf90_enddef(ncidOut)

  !copy value    
  do ii=1, nbVar
    status = nf90_inquire_variable(ncidIn, ii, VarName, ndims = nbvarDim, dimids = varDimID)
    allocate(pdimSize(nbvarDim))
    do jj=1, nbvarDim
      status = nf90_inquire_dimension(ncidIn, varDimID(jj), DimName, nRecords)
      pdimSize(jj)=nRecords
    enddo
    allocate(pvalue(product(pdimSize)))
    allocate(start(size(pdimSize)))
    start=1
    status = nf90_get_var(ncidIn, ii, pvalue, start = start, count = pdimSize)
    if(status /= nf90_NoErr) then
      write(*,*) "error 6", NF90_STRERROR(status)
      stop
    endif
    
    isDim=0
    do jj=1,nbDim
     if((trim(DimNameList(jj)).eq.trim(VarName)).or.(trim(DimNameList(jj))//"_bounds".eq.trim(VarName))) then
       isDim=1
     endif
    enddo
    
    if(((ii.eq.varfilteredID).or.(varfilteredID.eq.-1)).and.(isDim.eq.0).and.(nbvarDim.le.4)) then
      write(*,*) "Filtering...", trim(VarName)
      if(nbvarDim.ge.1) then
        allocate(dimSizeExt(4))
        dimSizeExt=1
        dimSizeExt(1:nbvarDim)=pdimSize
        allocate(var(dimSizeExt(1),dimSizeExt(2),dimSizeExt(3),dimSizeExt(4)))
        do jj=0, dimSizeExt(1)-1
          do kk=0, dimSizeExt(2)-1
            do ll=0, dimSizeExt(3)-1
              do mm=0, dimSizeExt(4)-1
                var(jj+1,kk+1,ll+1,mm+1)=pvalue(jj+dimSizeExt(1)*kk+dimSizeExt(1)*dimSizeExt(2)*ll+mm*dimSizeExt(1)*dimSizeExt(2)*dimSizeExt(3)+1)
              enddo
            enddo
          enddo
        enddo

        if(nbvarDim.eq.1) then
          if(annual.eq.1) then
            allocate(inputdata(dimSizeExt(1)))
            allocate(outputdata(dimSizeExt(1)))
          else
            allocate(inputdata(dimSizeExt(1)/12))
            allocate(outputdata(dimSizeExt(1)/12))
          endif
          if(annual.eq.1) then
            do jj=1, dimSizeExt(1)
              inputdata(jj)=var(jj,1,1,1)
            enddo
            call filtfilt(outputdata,num,den,inputdata)
            do jj=1, dimSizeExt(1)
              var(jj,1,1,1)=outputdata(jj)
            enddo
          else
            do month=1, 12
              do jj=1, dimSizeExt(1)/12
                inputdata(jj)=var((jj-1)*12+month,1,1,1)
              enddo
              call filtfilt(outputdata,num,den,inputdata)
              do jj=1, dimSizeExt(1)/12
                var((jj-1)*12+month,1,1,1)=outputdata(jj)
              enddo
            enddo
          endif
        else if(nbvarDim.eq.2) then
          if(annual.eq.1) then
            allocate(inputdata(dimSizeExt(2)))
            allocate(outputdata(dimSizeExt(2)))
          else
            allocate(inputdata(dimSizeExt(2)/12))
            allocate(outputdata(dimSizeExt(2)/12))
          endif
          do jj=1, dimSizeExt(1)
            if(annual.eq.1) then
              do kk=1, dimSizeExt(2)
                inputdata(kk)=var(jj,kk,1,1)
              enddo
              call filtfilt(outputdata,num,den,inputdata)
              do kk=1, dimSizeExt(2)
                var(jj,kk,1,1)=outputdata(kk)
              enddo
            else
              do month=1, 12
                do kk=1, dimSizeExt(2)/12
                  inputdata(kk)=var(jj,(kk-1)*12+month,1,1)
                enddo
                call filtfilt(outputdata,num,den,inputdata)
                do kk=1, dimSizeExt(2)/12
                  var(jj,(kk-1)*12+month,1,1)=outputdata(kk)
                enddo
              enddo
            endif
          enddo
        else if(nbvarDim.eq.3) then
          if(annual.eq.1) then
            allocate(inputdata(dimSizeExt(3)))
            allocate(outputdata(dimSizeExt(3)))
          else
            allocate(inputdata(dimSizeExt(3)/12))
            allocate(outputdata(dimSizeExt(3)/12))
          endif
          do jj=1, dimSizeExt(1)
            do kk=1, dimSizeExt(2)
              if(annual.eq.1) then
                do ll=1, dimSizeExt(3)
                  inputdata(ll)=var(jj,kk,ll,1)
                enddo
                call filtfilt(outputdata,num,den,inputdata)
                do ll=1, dimSizeExt(3)
                  var(jj,kk,ll,1)=outputdata(ll)
                enddo
              else
                do month=1, 12
                  do ll=1, dimSizeExt(3)/12
                    inputdata(ll)=var(jj,kk,(ll-1)*12+month,1)
                  enddo
                  call filtfilt(outputdata,num,den,inputdata)
                  do ll=1, dimSizeExt(3)/12
                    var(jj,kk,(ll-1)*12+month,1)=outputdata(ll)
                  enddo
                enddo
              endif
            enddo
          enddo
        else if(nbvarDim.eq.4) then
          if(annual.eq.1) then
            allocate(inputdata(dimSizeExt(4)))
            allocate(outputdata(dimSizeExt(4)))
          else
            allocate(inputdata(dimSizeExt(4)/12))
            allocate(outputdata(dimSizeExt(4)/12))
          endif
          do jj=1, dimSizeExt(1)
            do kk=1, dimSizeExt(2)
              do ll=1, dimSizeExt(3)
                if(annual.eq.1) then
                  do mm=1, dimSizeExt(4)
                    inputdata(mm)=var(jj,kk,ll,mm)
                  enddo
                  call filtfilt(outputdata,num,den,inputdata)
                  do mm=1, dimSizeExt(4)
                    var(jj,kk,ll,mm)=outputdata(mm)
                  enddo
                else
                  do month=1, 12
                    do mm=1, dimSizeExt(4)/12
                      inputdata(mm)=var(jj,kk,ll,(mm-1)*12+month)
                    enddo
                    call filtfilt(outputdata,num,den,inputdata)
                    do mm=1, dimSizeExt(4)/12
                      var(jj,kk,ll,(mm-1)*12+month)=outputdata(mm)
                    enddo
                  enddo
                endif
              enddo
            enddo
          enddo
        endif
        
        do jj=0, dimSizeExt(1)-1
          do kk=0, dimSizeExt(2)-1
            do ll=0, dimSizeExt(3)-1
              do mm=0, dimSizeExt(4)-1
                pvalue(jj+dimSizeExt(1)*kk+dimSizeExt(1)*dimSizeExt(2)*ll+mm*dimSizeExt(1)*dimSizeExt(2)*dimSizeExt(3)+1)=var(jj+1,kk+1,ll+1,mm+1)
              enddo
            enddo
          enddo
        enddo
      endif
    else
      write(*,*) "Copying...", trim(VarName)
    endif
    
    status = nf90_put_var(ncidOut, ii, pvalue, start, pdimSize)
    if(status /= nf90_NoErr) then
      write(*,*) "error 6", NF90_STRERROR(status)
      stop
    endif
    deallocate(pvalue)
    if(allocated(pdimSize)) deallocate(pdimSize)
    if(allocated(start)) deallocate(start)
    if(allocated(dimSizeExt)) deallocate(dimSizeExt)
    if(allocated(outputdata)) deallocate(outputdata)
    if(allocated(inputdata)) deallocate(inputdata)
    if(allocated(var)) deallocate(var)
  enddo
  
  !close file
  status = nf90_close(ncidIn);
  if(status /= nf90_NoErr) then
    write(*,*) "error 7", NF90_STRERROR(status)
    stop
  endif
  status = nf90_close(ncidOut);
  if(status /= nf90_NoErr) then
    write(*,*) "error 7", NF90_STRERROR(status)
    stop
  endif 
  
end program butter

