subroutine buttap(PolR, PolI, k, n)
  implicit none
  integer :: n
  double precision, dimension(:), allocatable :: PolR, PolI
  double precision :: k
  
  integer :: i, idx
  double precision :: value, pi, pi_2
  
  pi=dacos(-1.D0)
  pi_2=dacos(0.D0)
  
  allocate(PolR(n))
  allocate(PolI(n))

  !p = exp(i*(pi*(1:2:n-1)/(2*n) + pi/2))
  idx=1
  do i=1, n-1, 2
    value=pi*i/(2.D0*n)+pi_2
    PolR(idx)=cos(value)
    PolI(idx)=sin(value)
    idx=idx+2
  enddo
  
  !p = [p; conj(p)]
  do i=1, n-1, 2
    PolR(i+1)=PolR(i)
    PolI(i+1)=-PolI(i)
  enddo
  
  if(mod(n,2).eq.1) then !n is ode
    PolR(n)=-1.D0
    PolI(n)=0.D0
  endif
  
  k=1
end subroutine buttap


subroutine zp2ss(a, b, c, d, PolR, n)
  implicit none
  double precision, dimension(:,:), allocatable :: a
  double precision, dimension(:), allocatable :: b, c, PolR
  double precision :: d
  integer :: n
  
  integer :: i, pidx
  
  !a
  allocate(a(n,n))
  a=0 
  if(mod(n,2).eq.0) then
    pidx=n
    do i=1, n, 2
      a(i,i)=2.D0*PolR(pidx)
      pidx=pidx-2
    enddo
    do i=1, n, 2
        a(i+1,i)=-1.D0
    enddo
  else
    pidx=n-1
    do i=2, n, 2
        a(i,i)=2.D0*PolR(pidx)
        pidx=pidx-2
    enddo
    a(1,1)=-1.D0
    do i=2, n, 2
      a(i+1,i)=-1.D0
    enddo
  endif
  do i=1, n-1, 1
    a(i,i+1)=1.D0
  enddo
  
  !b
  allocate(b(n))
  b=0.D0
  b(1)=1.D0
  
  !c
  allocate(c(n))
  c=0.D0
  c(n)=1.D0
  
  !d
  d=0.D0
end subroutine zp2ss


subroutine lp2lp(a, b, Wn)
  implicit none
  double precision, dimension(:,:), allocatable :: a
  double precision, dimension(:), allocatable :: b
  double precision :: Wn
  
  a=a*Wn;
  b=b*Wn;
end subroutine lp2lp


subroutine bilinear(a, b, c, d, fs, n)
  implicit none
  double precision, dimension(:,:), allocatable :: a
  double precision, dimension(:), allocatable :: b, c
  double precision :: d, fs
  integer :: n
  
  double precision, dimension(:,:), allocatable :: t1, t2, t2inv, ad
  double precision, dimension(:), allocatable :: IPIV, bd, cd, dd
  double precision :: t, r
  integer :: LWMAX, i, info, LWORK
  parameter ( LWMAX=1000 )
  double precision :: WORK(LWMAX)
  
  t=1.D0/fs
  r=sqrt(t)
  
  !t1 = eye(size(a)) + a*t/2.0;
  allocate(t1(n,n))
  t1=a*t/2.D0
  do i=1, n, 1
    t1(i,i)=t1(i,i)+1.D0
  enddo
  
  !t2 = eye(size(a)) - a*t/2.0;
  allocate(t2(n,n))
  t2=-a*t/2.D0
  do i=1, n, 1
    t2(i,i)=t2(i,i)+1.D0
  enddo
  
  !ad = t2\t1
  !A\B=A-1 B
  allocate(t2inv(n,n))
  call DCOPY(n*n, t2, 1, t2inv, 1) !Copie de t2 dans t2inv, via DCOPY (routine BLAS)
  allocate(IPIV(n))
  call DGETRF(n, n, t2inv, n, IPIV, info) !Decomposition LU de t2inv (==t2 pour l instant)
  if(info.ne.0) then
    write(*,*) "probleme ici"
  endif
  !Query the optimal workspace.
  LWORK=-1
  call DGETRI(n, t2inv, n, IPIV, WORK, LWORK, info)
  LWORK = MIN( LWMAX, INT(WORK(1)) )
  !inverse
  call DGETRI(n, t2inv, n, IPIV, WORK, LWORK, info)
  if(info.ne.0) then
    write(*,*) "probleme la"
  endif
  !produit
  allocate(ad(n,n))
  CALL DGEMM('N', 'N', n, n, n, 1.D0, t2inv, n, t1, n, 0.D0, ad, n)
 
  !bd = t/r*(t2\b)
  allocate(bd(n))
  CALL DGEMV('T', n, n, 1.D0, t2inv, n, b, 1, 0.D0, bd, 1)
  bd=bd*t/r

  !cd = r*c/t2
  allocate(cd(n))
  CALL DGEMV('N', n, n, 1.D0, t2inv, n, c, 1, 0.D0, cd, 1)
  !suite apres appel suivant (*)
  
  !dd = c/t2*b*t/2 + d;
  allocate(dd(n))
  CALL DGEMV('N', n, 1, 1.D0, cd, n, b, 1, 0.D0, dd, 1)
  dd=dd*t/2.D0+d
  
  cd=cd*r !(*)
  
  a=ad
  b=bd
  c=cd
  d=dd(1)
end subroutine bilinear


subroutine poly(c, x, n)
  implicit none
  integer :: n
  double precision, dimension(:,:), allocatable :: x
  double precision, dimension(:), allocatable :: c
  
  double precision, dimension(:,:), allocatable :: VL, VR
  double precision, dimension(:), allocatable :: WR, WI, cI, cR, ResuR, ResuC
  integer :: LWMAX, j
  parameter ( LWMAX=1000 )
  double precision :: WORK(LWMAX)
  integer :: INFO, LWORK
  
  !e = eig(x);  
  allocate(VL(N,N))
  allocate(VR(N,N))
  allocate(WR(N))
  allocate(WI(N))
  !Query the optimal workspace.
  LWORK=-1
  call DGEEV('Vectors', 'Vectors', n, x, n, WR, WI, VL, n, VR, n, WORK, LWORK, INFO)
  LWORK = MIN( LWMAX, INT(WORK(1)) )
  !Solve eigenproblem.
  call DGEEV('Vectors', 'Vectors', n, x, n, WR, WI, VL, n, VR, n, WORK, LWORK, INFO)
  if(INFO.gt.0) then !Check for convergence.
    write(*,*) 'The algorithm failed to compute eigenvalues.'
    stop
  endif
    
  allocate(c(n+1))
  allocate(cR(n+1))
  allocate(cI(n+1))
  allocate(ResuR(n+1))
  allocate(ResuC(n+1))
  cR=0.D0
  cR(1)=1.D0
  cI=0.D0
  do j=1, n
      !c(2:(j+1)) = c(2:(j+1)) - W(j)*c(1:j)      
      ResuR(2:(j+1))=WR(j)*cR(1:j)-WI(j)*cI(1:j)
      ResuC(2:(j+1))=WI(j)*cR(1:j) + WR(j)*cI(1:j)
      cR(2:(j+1)) = cR(2:(j+1)) - ResuR(2:(j+1))
      cI(2:(j+1)) = cI(2:(j+1)) - ResuC(2:(j+1))
  enddo
  c=cR
end subroutine poly


subroutine buttnum(b, den, n)
  implicit none
  double precision, dimension(:), allocatable :: b, den
  integer :: n
  
  double precision, dimension(:), allocatable :: r
  double precision :: w
  integer :: j
  
  allocate(r(n))
  r=-1.D0
  w=0.D0

  allocate(b(n+1))
  b=0.D0
  b(1)=1.D0
  do j=1, n
      b(2:(j+1)) = b(2:(j+1)) - r(j)*b(1:j)
  enddo

  b=b*(sum(den)/sum(b))
  
end subroutine buttnum


subroutine filtfilt(y,b,a,x)
  implicit none
  
  interface
    subroutine filter(y,b,a,x)
      double precision, dimension(:), allocatable :: y, b, a, x
    endsubroutine filter
  end interface
  
  double precision, dimension(:), allocatable :: y, b, a, x, xreduce
  integer, dimension(:), allocatable :: xhole
  
  integer :: nb, na, n, nr, nfilt, nfact, i, j, nbhole, starthole, nbbeghole, nbendhole
  double precision, dimension(:), allocatable :: yF, yTemp
  double precision :: coefinterpo

  n=size(x)
  nb=size(b)
  na=size(a)
  nfilt=max(nb,na)
  nfact=6*(nfilt-1) !length of edge transients

  !remove hole at the begining
  nbbeghole=0
  if((nbbeghole.le.n).and.(x(1).lt.-99.98D0).and.(x(1).gt.-100.D0)) then
    do while((x(nbbeghole+1).lt.-99.98D0).and.(x(nbbeghole+1).gt.-100.D0))
      nbbeghole=nbbeghole+1
    enddo
  endif
  
  !remove hole at the end
  nbendhole=0
  if(nbbeghole.ne.n) then
    if((x(n).lt.-99.98D0).and.(x(n).gt.-100.D0)) then
      do while((x(n-nbendhole).lt.-99.98D0).and.(x(n-nbendhole).gt.-100.D0))
        nbendhole=nbendhole+1
      enddo
    endif
  endif
  

  if((nbbeghole+nbendhole).le.n-2) then
    !cut not need part avec x (hole beg and end)
    allocate(xreduce(n-nbendhole-nbbeghole))
    do i=1, n-nbendhole-nbbeghole
      xreduce(i)=x(nbbeghole+i)
    enddo
    nr=size(xreduce)

    !hole traitment
    nbhole=0
    i=0
    
    allocate(xhole(nr))
    xhole=0
    do while(i.le.n-nbendhole-nbbeghole)
      i=i+1
      starthole=i
      do while((xreduce(i).lt.-99.98D0).and.(xreduce(i).gt.-100.D0))
        xhole(i)=1
        nbhole=nbhole+1
        i=i+1
      enddo
      if(nbhole.gt.0) then
        !write(*,'(A3,I4,A4,I4,A3,I4,A8)') "i= ",starthole," -> ", starthole+nbhole-1, " = ", nbhole, " hole(s)"
        !write(*,*) xreduce(starthole-1:starthole+nbhole)
        coefinterpo=(xreduce(starthole+nbhole)-xreduce(starthole-1))/(real(nbhole+1))
        do j=0, nbhole-1
          xreduce(starthole+j)=coefinterpo*(j+1)+xreduce(starthole-1)
        enddo
        !write(*,*) x(starthole-1:starthole+nbhole)
        nbhole=0
      endif
    enddo
    
    !Extrapolate beginning and end of data sequence using a "reflection
    !method".  Slopes of original and extrapolated sequences match at
    !the end points.
    !This reduces end effects.
    !y = [2*x(1)-x((nfact+1):-1:2);x;2*x(len)-x((len-1):-1:len-nfact)];
    if(nfact.gt.nr) then
      write(*,*) "the filter order is too high for the number of point to filter -> Point undef"
      y=-99.99D0
    else
      allocate(yTemp(nr+2*nfact))
      do i=1, nfact
        yTemp(i)=2*xreduce(1)-xreduce(nfact+2-i)
      enddo
      do i=1, nr
        yTemp(nfact+i)=xreduce(i)
      enddo
      do i=1, nfact
        yTemp(nfact+nr+i)=2*xreduce(nr)-xreduce(nr-i)
      enddo
      
      !filter, reverse data, filter again, and reverse data again
      call filter(yF,b,a,yTemp)
      do i=1, size(yF)
        yTemp(i)=yF(size(yF)-i+1)
      enddo
      deallocate(yF)
      call filter(yF,b,a,yTemp)
      do i=1, size(yF)
        yTemp(i)=yF(size(yF)-i+1)
      enddo

      do i=1, nbbeghole
        y(i)=x(i)
      enddo
      j=0
      do i=nbbeghole+1, n-nbbeghole
!         if(xhole(i-nbbeghole).eq.1) then
!           y(i)=x(i)
!         else
          y(i)=yTemp(nfact+1+j)
!         endif
        j=j+1
      enddo
      do i=n-nbbeghole+1, n
        y(i)=x(i)
      enddo
    endif
  else
    do i=1, n
      y(i)=x(i)
    enddo
  endif
  if(allocated(xreduce)) deallocate(xreduce)
  if(allocated(yTemp)) deallocate(yTemp)
end subroutine filtfilt


subroutine filter(y,b,a,x)
  implicit none
  double precision, dimension(:), allocatable :: y, b, a, x
  
  integer :: i, j
  allocate(y(size(x)))
  y=0.D0
  do i=1, size(x)
    do j=1, size(b)
      if(i-j+1.ge.1) then
        y(i)=y(i)+b(j)*x(i-j+1)
        !if(i.le.2) write(*,'(A2,I2,A7,I2,A6,I2,A4,F,A4,F,A3,F)') 'y(',i,') += b(',j,') * x(',i-j+1,') = ',b(j),' * ',x(i-j+1),' = ',y(i)
        !if(i.eq.10) write(*,*) 'y(',i,') += b(',j,') * x(',i-j+1,') = ',b(j),' * ',x(i-j+1),' = ',y(i)
      endif
    enddo
    do j=2, min(i,size(a))
      y(i)=y(i)-a(j)*y(i-j+1)
      !if(i.le.2) write(*,'(A2,I2,A7,I2,A6,I2,A4,F,A4,F,A3,F)') 'y(',i,') -= b(',j,') * y(',i-j+1,') = ',a(j),' * ',y(i-j+1),' = ',y(i)
      !if(i.eq.10) write(*,*) 'y(',i,') -= b(',j,') * y(',i-j+1,') = ',a(j),' * ',y(i-j+1),' = ',y(i)
    enddo
  enddo
end subroutine filter

subroutine Print_Vector(DESC, N, V)
      character*(*)    DESC
      integer          N
      double precision V(*)
      integer          j

      WRITE(*,'(A,A)',ADVANCE='NO') " ",DESC
      do j = 1, N
            WRITE(*,'(F10.5,A)',ADVANCE='NO') V(J), " "
      enddo
      WRITE(*,*)
      RETURN
end subroutine Print_Vector
