// NetCDFwavg -- NetCDF weighted average 
// is based on ncfint 1.29 developped under GPL licence by
//    Charlie Zender
//    Department of Earth System Science
//    University of California at Irvine
//    Irvine, CA 92697-3100

#include <iostream>
#include <netcdfcpp.h>

using namespace std;


int main(int argc,char **argv)
{
  if(argc<3)
  {
    cerr<<"Usage :"<<endl<<"\t./Colonnifaire [input.nc] [variable1] [variable2] ..."<<endl;
    return 1;
  }
  
  NcFile dataFile(argv[1], NcFile::ReadOnly);
  
  double* value[argc-2];
  NcVar** var= new NcVar*[argc-2];
  long sizedimOld=-1;
  int sizedim;
  for(int i=2; i<=argc-1; i++)
  {
    if (!(var[i-2] = dataFile.get_var(argv[i])))
    {
      cerr<<"Variable "<<argv[i]<<" not found !"<<endl;
      return 1;
    }else{
      int numDim=(var[i-2])->num_dims();
      int nbDimSizeSup1=0;
      int sizeDimTab[5]; for(int j=0; j<5; j++) sizeDimTab[j]=0;
      for(int j=0; j<numDim; j++)
      {
        sizeDimTab[j]=(var[i-2])->get_dim(j)->size();
        if((var[i-2])->get_dim(j)->size()>1)
        {
          sizedim=sizeDimTab[j];
          nbDimSizeSup1++;
        }
      }
      if(nbDimSizeSup1==0) 
      {
        nbDimSizeSup1=1;
        sizedim=1;
      }
      if(nbDimSizeSup1!=1)
      {
        cerr<<"only 1D (or pseudo 1D) variable are supported"<<endl<<flush;
        return 1;
      }else{
        if(sizedimOld==-1)
        {
          sizedimOld=sizedim;
        }else if(sizedim!=sizedimOld)  
        {
          cerr<<"variable have different dimension size : "<<sizedim<<"!="<<sizedimOld<<endl;
          return 1;          
        }
        value[i-2]=new double[sizedim];
        var[i-2]->get(value[i-2], sizeDimTab[0], sizeDimTab[1], sizeDimTab[2], sizeDimTab[3], sizeDimTab[4]);
      }
    }
  }
  
  for(int j=0; j<argc-2; j++) cout<<argv[j+2]<<"\t\t";
  cout<<endl;
  for(int i=0; i<sizedim; i++)
  {
    for(int j=0; j<argc-2; j++)
    {
      cout<<value[j][i]<<"\t\t";
    }
    cout<<endl;
  }
  

  return 0;
}
