subroutine usage()
  write(*,'(A)') "Usage:"
  write(*,'(A)') "  ./Correl [input.nc] [varname] [input.nc] [varname] [output.nc]"
  stop
end subroutine usage

include "NetcdfFonction.f90"

PROGRAM Correl
! calculates the correlation between temperature from the dataset HadCRUT 
! (averaged over some space) 
! and sea level pressure from the dataset hadslp2
! hadslp2 was rewritten in netcdf-format by read-int.f90

  USE NETCDF
  IMPLICIT NONE

  !Definition of variable to use these previous function
  CHARACTER(nf90_max_name) :: filenameIn1, filenameIn2, filenameOut, VarName1, VarName2, dimName, name, attName
  integer :: inputID, inputID1, inputID2, outputID, RecordDimID, RecordDimID1, RecordDimID2, outvarID, corrID
  integer :: totaltime, getTime, nbVar, nbVar1, nbVar2, getLonOrLatID, VarToCorID1, VarToCorID2
  double precision, dimension(nf90_max_var_dims) :: undefMinTab1, undefMaxTab1, undef1
  double precision, dimension(nf90_max_var_dims) :: undefMinTab2, undefMaxTab2, undef2
  double precision :: undef
  integer :: LonDimID, LatDimID, LonDimID1, LatDimID1, LonDimID2, LatDimID2, reffile
  integer :: dimSize, nbAtt, nbvarDim, variableType, variableType1, variableType2, idxRec1, idxRec2
  integer, dimension(:), allocatable :: varDimIDshort
  integer, dimension(nf90_max_var_dims) :: varDimID
  INTEGER, DIMENSION(:), allocatable :: start1, start2

  INTEGER, DIMENSION(:), POINTER :: pdimSize1, pdimSize2

  INTEGER :: nlon1, nlat1, nlon2, nlat2, i, j, k, t, LatIdx, product1, product2
  real :: radian
  
  double precision, dimension(:), allocatable :: valueDbl1D1, valueDbl1D2, corrST1D, valueCorrectGrid
  double precision, dimension(:), allocatable :: moyenneDbl1, moyenneDbl2, value, value1, value2, lat
  integer, dimension(:), allocatable :: nbPoint
  integer :: deflate_level
  deflate_level=1

  !GET ARGUMENT FOR SELECT METHODE
  if(iargc().eq.5) then
    call getarg(1,filenameIn1)
    call getarg(2,VarName1)
    call getarg(3,filenameIn2)
    call getarg(4,VarName2)
    call getarg(5,filenameOut)
  else
    call usage()
  endif


  write(*,'(A)',ADVANCE='NO') "Open input file..."
  call check(nf90_open(filenameIn1, nf90_nowrite, inputID1))
  call check(nf90_open(filenameIn2, nf90_nowrite, inputID2))
  write(*,*) "OK"



  write(*,'(A)',ADVANCE='NO') "   Get some value..."
  !get nbdim and nbvar
  !call check(nf90_inquire(inputID, nbDim, nbVar))
  !get totaltime
  totaltime=getTime(inputID1, RecordDimID1)
  if(getTime(inputID2, RecordDimID2).ne.totaltime) then
    write(*,'(A)') "   Error : 2 files have different record dimensions size..."
    stop
  endif
  !get longitude name
  LonDimID1=getLonOrLatID(inputID1,1)
  if(LonDimID1.eq.-1) then
    nlon1=1
  else
    call check(nf90_inquire_dimension(inputID1, LonDimID1, name = dimName, len = dimSize))
    nlon1=dimSize
  endif
  LonDimID2=getLonOrLatID(inputID2,1)
  if(LonDimID2.eq.-1) then
    nlon2=1
  else
    call check(nf90_inquire_dimension(inputID2, LonDimID2, name = dimName, len = dimSize))
    nlon2=dimSize
  endif
  !get latitude name
  LatDimID1=getLonOrLatID(inputID1,0)
  if(LatDimID1.eq.-1) then
    nlat1=1
  else
    call check(nf90_inquire_dimension(inputID1, LatDimID1, name = dimName, len = dimSize))
    nlat1=dimSize
  endif
  LatDimID2=getLonOrLatID(inputID2,0)
  if(LatDimID2.eq.-1) then
    nlat2=1
  else
    call check(nf90_inquire_dimension(inputID2, LatDimID2, name = dimName, len = dimSize))
    nlat2=dimSize
  endif
  !get undef value
  call check(nf90_inquire(inputID1, nVariables=nbVar1))
  do i=1, nbVar1
    call GetUndefForVar(inputID1, i, undef1(i), undefMinTab1(i), undefMaxTab1(i))
  enddo
  call check(nf90_inquire(inputID2, nVariables=nbVar2))
  do i=1, nbVar2
    call GetUndefForVar(inputID2, i, undef2(i), undefMinTab2(i), undefMaxTab2(i))
  enddo
  radian=2*acos(0.0)/180.0
  if((nlat1.gt.1).and.(nlon1.gt.1)) then
    inputID=inputID1
    LatDimID=LatDimID1
    LonDimID=LonDimID1
    nbVar=nbVar1
    RecordDimID=RecordDimID1
    reffile=1
  else
    inputID=inputID2
    LatDimID=LatDimID2
    LonDimID=LonDimID2
    nbVar=nbVar2
    RecordDimID=RecordDimID2
    reffile=2
  endif
  write(*,*) reffile, "OK"


  write(*,'(A)') "Create output file :"
  call check(nf90_create(trim(filenameOut), nf90_hdf5, outputID))   !write target file
  
  !define dimension, variable associated and attribut
  write(*,'(A)',ADVANCE='NO') "   Define dimensions..."
  call CopyDimensionFrame(inputID, outputID)
  write(*,*) "OK"


  !define variable from the list
  write(*,'(A)',ADVANCE='NO') "   Define variables and attributs..."
  do i=1, nbVar
    call check(nf90_inquire_variable(inputID, i, name, xtype=variableType, nAtts=nbAtt, ndims=nbvarDim, dimids=varDimID))
    if(nf90_inq_dimid(inputID, name, outvarID).eq.nf90_noerr) then
      allocate(varDimIDshort(nbvarDim)); varDimIDshort=varDimID(1:nbvarDim)
      call check(nf90_def_var(outputID, name, variableType, varDimIDshort, outvarID, shuffle = .TRUE., deflate_level=deflate_level))
      do j=1, nbAtt
        call check(nf90_inq_attname(inputID, i, j, attName))
        call check(nf90_copy_att(inputID, i, attName, outputID, outvarID))
      enddo
      deallocate(varDimIDshort)
    endif
  enddo
  allocate(varDimIDshort(1)); varDimIDshort(1)=RecordDimID
  call check(nf90_inquire_dimension(inputID1, LatDimID1, name = dimName))
  call check(nf90_inq_varid(inputID1, dimName, outvarID))
  allocate(lat(nlat1))
  call check(nf90_get_var(inputID1, outvarID, lat, (/1/), (/nlat1/)))
  call check(nf90_def_var(outputID, "Correl", NF90_DOUBLE, varDimIDshort, corrID, shuffle = .TRUE., deflate_level=deflate_level))
  
  deallocate(varDimIDshort)
  write(*,*) "OK"


  !define variable from the list
  write(*,'(A)',ADVANCE='NO') "Copy dimensions data in output file..."
  VarToCorID2=-1
  do i=1, nbVar2
    call check(nf90_inquire_variable(inputID2, i, name, xtype=variableType, nAtts=nbAtt, ndims=nbvarDim, dimids=varDimID))
    if(name.eq.VarName2) then
      VarToCorID2=i
      variableType2=variableType
    endif
  enddo
  VarToCorID1=-1
  do i=1, nbVar1
    call check(nf90_inquire_variable(inputID1, i, name))
    if(name.eq.VarName1) then
      VarToCorID1=i
      variableType1=variableType
    endif
  enddo
  if(reffile.eq.1) then
    undef=undef1(VarToCorID1)
  else
    undef=undef2(VarToCorID2)
  endif
  call check(nf90_put_att(outputID, corrID, "_FillValue", undef))
  call check(nf90_put_att(outputID, corrID, "missing_value", undef))
  

  !finish the configuration of the output file and starting put data
  call check(nf90_enddef(outputID))

  j=1
  do i=1, nbVar
    call check(nf90_inquire_variable(inputID, i, name, xtype=variableType, nAtts=nbAtt, ndims=nbvarDim, dimids=varDimID))
    if(nf90_inq_dimid(inputID, name, outvarID).eq.nf90_noerr) then
      call check(nf90_inquire_dimension(inputID, outvarID, len = dimSize))
      call CopyVariableOfDimensionValue(inputID, outputID, i, j, variableType, 1, dimSize)
      j=j+1
    endif
  enddo
  if((VarToCorID1.eq.-1).or.(VarToCorID2.eq.-1)) then
    if(VarToCorID1.eq.-1) write(*,*) "Error ", trim(VarName1), " not found in the 1st file"
    if(VarToCorID2.eq.-1) write(*,*) "Error ", trim(VarName2), " not found in the 2st file"
    stop
  else
    write(*,*) "OK"
  endif

  !prepare variable information
  call check(nf90_inquire_variable(inputID1, VarToCorID1, xtype=variableType1, ndims=nbvarDim, dimids=varDimID))
  allocate(pdimSize1(nbvarDim))
  allocate(start1(nbvarDim)); start1=1
  do j=1, nbvarDim
    call check(nf90_inquire_dimension(inputID1, varDimID(j), DimName, DimSize))
    if(varDimID(j).eq.RecordDimID1) then
      pdimSize1(j)=1
      idxRec1=j
    else
      pdimSize1(j)=DimSize
    endif
  enddo
  call check(nf90_inquire_variable(inputID2, VarToCorID2, xtype=variableType2, ndims=nbvarDim, dimids=varDimID))
  allocate(pdimSize2(nbvarDim))
  allocate(start2(nbvarDim)); start2=1
  do j=1, nbvarDim
    call check(nf90_inquire_dimension(inputID2, varDimID(j), DimName, DimSize))
    if(varDimID(j).eq.RecordDimID2) then
      pdimSize2(j)=1
      idxRec2=j
    else
      pdimSize2(j)=DimSize
    endif
  enddo


  allocate(moyenneDbl1(totaltime))
  allocate(moyenneDbl2(totaltime))
  allocate(nbPoint(totaltime))
  allocate(valueDbl1D1(product(pdimSize1)))
  allocate(valueDbl1D2(product(pdimSize2)))
  allocate(valueCorrectGrid(product(pdimSize1)))
  call check(nf90_inquire_variable(inputID1, VarToCorID1, ndims=nbvarDim, dimids=varDimID))
  LatIdx=-1
  do i=1, nbvarDim
    if(varDimID(i).eq.LatDimID1) LatIdx=i
  enddo
  product1=product(pdimSize1)
  if(LatIdx.gt.1) then
    j=1; k=1
    product2=product(pdimSize1(1:LatIdx-1))    
    do while(j.le.product1)
      do i=1, product2
        valueCorrectGrid(j)=cos(lat(k)*radian)
        j=j+1
      enddo
      k=k+1
      if(k.gt.nlat1) k=1
    enddo
  else
    j=1; k=1
    do j=1,product1
      valueCorrectGrid(j)=cos(lat(k)*radian)
      k=k+1
      if(k.gt.nlat1) k=1
    enddo
  endif
  nbPoint=0; moyenneDbl1=0.; moyenneDbl2=0.

  do t=1, totaltime
    start1(idxRec1)=t; start2(idxRec2)=t;
    call check(nf90_get_var(inputID1, VarToCorID1, valueDbl1D1, start1, pdimSize1))
    call check(nf90_get_var(inputID2, VarToCorID2, valueDbl1D2, start2, pdimSize2))
    if(product(pdimSize1).eq.1) then !recopie les valeurs de f(t) dans f(x,y,t) pour le cas corre de f(t) avec g(x,y,t)
      valueDbl1D1=valueDbl1D1(1)
    elseif(product(pdimSize2).eq.1) then
      valueDbl1D2=valueDbl1D2(1)
    endif
    do j=1, size(valueDbl1D1)
      if(((valueDbl1D1(j).lt.undefMinTab1(VarToCorID1)).or.(valueDbl1D1(j).gt.undefMaxTab1(VarToCorID1))).and.((valueDbl1D2(j).lt.undefMinTab2(VarToCorID2)).or.(valueDbl1D2(j).gt.undefMaxTab2(VarToCorID2)))) then
        moyenneDbl1(t)=moyenneDbl1(t)+valueDbl1D1(j)*valueCorrectGrid(j)
        moyenneDbl2(t)=moyenneDbl2(t)+valueDbl1D2(j)*valueCorrectGrid(j)
        nbPoint(t)=nbPoint(t)+1
      endif
    enddo
  enddo
  do j=1, size(moyenneDbl1)
    if(nbPoint(j).ne.0.) then
      moyenneDbl1(j)=moyenneDbl1(j)/dble(nbPoint(j))
      moyenneDbl2(j)=moyenneDbl2(j)/dble(nbPoint(j))
    else
      moyenneDbl1(j)=0.
      moyenneDbl2(j)=0.
    endif
  enddo


  write(*,'(A)',ADVANCE='NO') "Calculation of the expectation..."
  allocate(value(totaltime))
  allocate(value1(totaltime))
  allocate(value2(totaltime))
  nbPoint=0; value=0.; value1=0.; value2=0.
  do t=1, totaltime
    start1(idxRec1)=t
    call check(nf90_get_var(inputID1, VarToCorID1, valueDbl1D1, start1, pdimSize1))

    start2(idxRec2)=t
    call check(nf90_get_var(inputID2, VarToCorID2, valueDbl1D2, start2, pdimSize2))

    if(product(pdimSize1).eq.1) then !recopie les valeurs de f(t) dans f(x,y,t) pour le cas corre de f(t) avec g(x,y,t)
      valueDbl1D1=valueDbl1D1(1)
    elseif(product(pdimSize2).eq.1) then
      valueDbl1D2=valueDbl1D2(1)
    endif
    do j=1, size(valueDbl1D2)
      if(((valueDbl1D1(j).lt.undefMinTab1(VarToCorID1)).or.(valueDbl1D1(j).gt.undefMaxTab1(VarToCorID1))).and.((valueDbl1D2(j).lt.undefMinTab2(VarToCorID2)).or.(valueDbl1D2(j).gt.undefMaxTab2(VarToCorID2)))) then
        value1(t)=value1(t)+(valueDbl1D1(j)*valueCorrectGrid(j)-moyenneDbl1(t))**2
        value2(t)=value2(t)+(valueDbl1D2(j)*valueCorrectGrid(j)-moyenneDbl2(t))**2
        value(t)=value(t)+(valueDbl1D1(j)*valueCorrectGrid(j)-moyenneDbl1(t))*(valueDbl1D2(j)*valueCorrectGrid(j)-moyenneDbl2(t))
        nbPoint(t)=nbPoint(t)+1
      endif
    enddo
  enddo
  do j=1, size(value)
    if(nbPoint(j).ne.0) then
      value(j)=value(j)/nbPoint(j)
      value1(j)=value1(j)/nbPoint(j)
      value2(j)=value2(j)/nbPoint(j)
    else
      value(j)=0.
      value1(j)=0.
      value2(j)=0.
    endif
  enddo
  write(*,'(A)') "OK"



  write(*,'(A)',ADVANCE='NO') "Calculation of the correlation..."
  allocate(corrST1D(size(value)))
  do j=1, size(value)
    if(value(j).eq.0) then
      corrST1D(j)=undef
    else
      corrST1D(j)=value(j)/(sqrt(value2(j))*sqrt(value1(j)))
    endif
  enddo
  start2=1
  deallocate(start1); allocate(start1(1)); start1=1
  deallocate(pdimSize1); allocate(pdimSize1(1)); pdimSize1=totaltime
  call check(nf90_put_var(outputID, corrID, corrST1D, start2, pdimSize1))
  write(*,'(A)') "OK"


  call check(nf90_close(outputID))

END 
