#include <iostream>
#include <vector>
#include <string>
#include <fstream>
#include <netcdfcpp.h>
#include <stdlib.h>
using namespace std;

bool firstwarnchangename=true;

class Variable
{
  public:
    Variable(string nameIn);
    long nbvalue;
    string name;
    string nameNetCDF;
    vector<float> value;
    float *tabvalue;
    string FormatName(string nameIn);
    void addValue(float val);
    void addName(string nameIn);
    float *gettab();
};

Variable::Variable(string nameIn)
{
  addName(nameIn);
  nbvalue=0;
  tabvalue=NULL;
}

string Variable::FormatName(string nameIn)
{
  string nameOut=nameIn;
  
  //remove end space : ex "|Variable 1|  " -> "|Variable 1|"
  long PosLastSpace=nameOut.find_last_not_of(" ");
  nameOut=name.substr(0,PosLastSpace+1);

  //replace space by _ : ex "|Variable 1|" -> "|Variable_1|"
  unsigned long Pos=nameOut.find(" ",0);
  while(Pos!=string::npos)
  {
    nameOut.replace(Pos, 1,"_");
    Pos=nameOut.find(" ",0);
  }

  //replace pipe by _ : ex "|Variable_1|" -> "IVariable_1I"
  Pos=nameOut.find("|",0);
  while(Pos!=string::npos)
  {
    nameOut.replace(Pos, 1,"I");
    Pos=nameOut.find("|",0);
  }
  
  //replace pipe by _ : ex "|Variable_1|" -> "IVariable_1I"
  Pos=nameOut.find("-",0);
  while(Pos!=string::npos)
  {
    nameOut.replace(Pos, 1,"m");
    Pos=nameOut.find("-",0);
  }
  
  return nameOut;
}

void Variable::addValue(float val)
{
  value.push_back(val);
  nbvalue++;
}

void Variable::addName(string nameIn)
{
  name=nameIn;
  nameNetCDF=FormatName(nameIn);
  if(firstwarnchangename && name.find(nameNetCDF,0)==string::npos)
  {
    cout<<"Warning : variable(s) name(s) is change due to illegal character"<<endl;
    firstwarnchangename=false;
  }
}

float *Variable::gettab()
{
  if(tabvalue==NULL)
  {
    tabvalue = new float[nbvalue];
    for(int i=0; i<nbvalue; i++)
      tabvalue[i]=value[i];
  }
  return tabvalue;
}

int main(int argc, char *argv[])
{
  //Check execution argument
  if(argc<3)
  {
    cout<<"Usage :"<<endl;
    cout<<"\tEvoluConverter [Evolufile] ... [Evolufile] [NetCDFfile]"<<endl;
    return 1;
  }
    
  //Container variable declaration
  vector<Variable> variable;
  
  string header;
  for(int numfile=0; numfile<argc-2; numfile++)
  {
    //open Evolu file
    ifstream file(argv[numfile+1]);
    if (!file)
    {
      cerr<<argv[numfile+1]<<" openning error !!"<<endl;
      return 1;
    }/*else{
      cout<<"Convert "<<argv[numfile+1]<<endl;
    }*/
    
    //ignore head of file
    string line;
    for(int i=0; i<4; i++)getline(file, line);
    
    //read variable liste 
    if(numfile==0)
    { //and initializate the container
      for(unsigned int i=0; i<line.length(); i+=5)
      {
	variable.push_back(line.substr(i,5));
      }
      header=line;
    }else{ //and check with the container
      if(header!=line)
      {
	cerr<<"Warning : header of "<<argv[numfile+1]<<" is different"<<endl;
	exit(1);
      }
    }
    
    //read variable value
    unsigned long PosName, PosValStart, PosValEnd;
    while(getline(file, line))
    {   
      for(unsigned int i=0; i<variable.size(); i++)
      {
	PosName=line.find(variable[i].name);
	if(PosName!=string::npos)
	{
	  PosValStart=line.find_first_not_of(" ", PosName+5);
	  PosValEnd=line.find(" ", PosValStart);

	  variable[i].addValue(atof((line.substr(PosValStart,PosValEnd-PosValStart)).c_str()));
	}
      }
    }
    
    file.close();
  }
  
  long size=variable[0].nbvalue;
  for(unsigned int i=1; i<variable.size(); i++)
  {
    if(size!=variable[i].nbvalue) cerr<<"Warning : "<<variable[i].name<<" is different size ("<<size<<"->"<<variable[i].nbvalue<<")"<<endl;
  }
  
  
  //Open output file
  int outid=nccreate(argv[argc-1], NC_CLOBBER);
  
  //attribut
  string title="EvoluCat2NetCDF file";
  nc_put_att_text (outid, NC_GLOBAL, "title", title.length(), title.c_str());

  
  //Define dimensions in output file
  ncdimdef(outid, "time", NC_UNLIMITED);
  ncdimdef(outid, "longitude", 1);
  ncdimdef(outid, "latitude", 1);
  
 
  //Define variables in output file
  int dimtab[MAX_NC_DIMS];
  string AttValue;
  
  dimtab[0]=0;
  ncvardef(outid, "time", NC_INT, 1, dimtab);
  AttValue = "years"; ncattput(outid, 0, "units", NC_CHAR, AttValue.length(), AttValue.c_str());
  AttValue = "time"; ncattput(outid, 0, "long_name", NC_CHAR, AttValue.length(), AttValue.c_str());
  AttValue = "time"; ncattput(outid, 0, "standard_name", NC_CHAR, AttValue.length(), AttValue.c_str());
  AttValue = "01-JAN-0001 00:00:00"; ncattput(outid, 0, "time_origin", NC_CHAR, AttValue.length(), AttValue.c_str());
  dimtab[0]=1;
  ncvardef(outid, "longitude", NC_INT, 1, dimtab);
  AttValue = "longitude"; ncattput(outid, 1, "long_name", NC_CHAR, AttValue.length(), AttValue.c_str());
  AttValue = "longitude"; ncattput(outid, 1, "standard_name", NC_CHAR, AttValue.length(), AttValue.c_str());
  AttValue = "X"; ncattput(outid, 1, "axis", NC_CHAR, AttValue.length(), AttValue.c_str());
  dimtab[0]=2;
  ncvardef(outid, "latitude", NC_INT, 1, dimtab);
  AttValue = "latitude"; ncattput(outid, 2, "long_name", NC_CHAR, AttValue.length(), AttValue.c_str());
  AttValue = "latitude"; ncattput(outid, 2, "standard_name", NC_CHAR, AttValue.length(), AttValue.c_str());
  AttValue = "Y"; ncattput(outid, 2, "axis", NC_CHAR, AttValue.length(), AttValue.c_str());
  dimtab[0]=0;
  for(unsigned int i=0; i<variable.size(); i++)
  {
    ncvardef(outid, variable[i].nameNetCDF.c_str(), NC_FLOAT, 1, dimtab);
    ncattput(outid, i+3, "long_name", NC_CHAR, variable[i].name.length(), variable[i].name.c_str());
    ncattput(outid, i+3, "standard_name", NC_CHAR, variable[i].nameNetCDF.length(), variable[i].nameNetCDF.c_str());
  }

  
  
  //Turn off default filling behavior to enhance efficiency
  ncsetfill(outid, NC_NOFILL);
  
  //Take output file out of define mode
  ncendef(outid);
  
  //create dummy time variable
  int timevalue[variable[0].nbvalue];
  long timestart[1]={0};
  long timecount[1]={variable[0].nbvalue};
  for(int i=0; i<variable[0].nbvalue; i++) timevalue[i]=i;
  ncvarput(outid, 0, timestart, timecount, (void*)timevalue);
  
  //Write value
  for(unsigned int i=0; i<variable.size(); i++)
  {
    long start[1]={0};
    long count[1]={variable[i].nbvalue};
    ncvarput(outid, i+3, start, count, (void*)variable[i].gettab());
  }
  ncclose(outid);
    
//   cout<<"NetCDF file "<<argv[argc-1]<<" is writen"<<endl;
    
  return 0;
}
