subroutine usage()
  write(*,'(A)') "Usage:"
  write(*,'(A)') "  ./Iceifaire [threshold] [idxlon_min] [idxlon_max] [idxlat_min] [idxlat_max] [input.nc] [output.nc]"
  write(*,'(A)') "  ./Iceifaire [threshold] [input.nc] [output.nc]  <-- Box is entire world"
  stop
end subroutine usage

include "NetcdfFonction.f90"

PROGRAM Iceifaire 
  USE NETCDF
  IMPLICIT NONE

  CHARACTER(nf90_max_name) :: filenameIn, filenameOut
  INTEGER, DIMENSION(:), allocatable :: pdimSize
 
  INTEGER :: i, j, t, nRecords, idxRec, idxLon, idxLat, status, productionPdimSize
  INTEGER, DIMENSION(:), allocatable :: start
  

  integer :: inputID, outputID, outdimid, RecordDimID, LonDimID, LatDimID, LonVarID, LatVarID
  integer :: nbDim, nbVar, nbAtt, getLonOrLatID, getTime
  integer :: nbvarDim, dimSize, varID, IceExtID, IceExtNbID, IceAreaID
  integer :: variableType, outvarid
  integer, dimension(nf90_max_var_dims) :: varDimID
  double precision, dimension(nf90_max_var_dims) :: undefMinTab, undefMaxTab, undef
  character(nf90_max_name) :: varName, dimName, parm, AttName
  double precision, dimension(:), allocatable :: valueDbl

  double precision, dimension(:), allocatable :: valueDbl1D
  real, dimension(:), allocatable :: Lat, Lon
  double precision, dimension(:), allocatable :: surface
  double precision, dimension(:), allocatable :: IceExtDbl, IceAreaDbl
  double precision, dimension(:), allocatable :: sumDbl
  integer, dimension(:), allocatable :: nbPoint

  integer :: totaltime, totalLat, totalLon, iLon, iLat

  character(nf90_max_name), dimension(nf90_max_var_dims) :: dimNameList

  !variable pour les box
  integer :: lon_min, lon_max, lat_min, lat_max
  integer, dimension(:,:), allocatable :: deftab
  double precision :: radian, Pi, seuil
  integer, dimension(:), allocatable :: varDimIDshort


  !GET ARGUMENT FOR SELECT METHODE
  if(iargc().gt.2) then
    if(iargc().eq.7) then
      call getarg(1,parm); read(parm,*) seuil
      call getarg(2,parm); read(parm,*) lon_min
      call getarg(3,parm); read(parm,*) lon_max
      call getarg(4,parm); read(parm,*) lat_min
      call getarg(5,parm); read(parm,*) lat_max
      call getarg(6,filenameIn)
      call getarg(7,filenameOut)
    elseif(iargc().eq.3) then
      lon_min=-1
      lon_max=-1
      lat_min=-1
      lat_max=-1
      call getarg(1,parm); read(parm,*) seuil
      call getarg(2,filenameIn)
      call getarg(3,filenameOut)
    else
      call usage()
    endif
  else
    call usage()
  endif
  
  Pi=2*acos(0.0)
  radian=Pi/180.0


  write(*,'(A)',ADVANCE='NO') "Open input file..."
  call check(nf90_open(filenameIn, nf90_nowrite, inputID))
  write(*,*) "OK"


  write(*,'(A)',ADVANCE='NO') "   Get some value..."
  !get nbdim and nbvar
  call check(nf90_inquire(inputID, nbDim, nbVar))
  !get totaltime
  totaltime=getTime(inputID, RecordDimID)
  !get undef value
  do i=1, nbVar
    call GetUndefForVar(inputID, i, undef(i), undefMinTab(i), undefMaxTab(i))
  enddo
  !deg to radian conversion
  radian=2*acos(0.0)/180.0
  write(*,*) "OK"



  !get longitude name
  write(*,'(A)',ADVANCE='NO') "   Get Longitude dimension..."
  LonDimID=getLonOrLatID(inputID,1)
  if(LonDimID.eq.-1) then
    write(*,'(A)') "Not Found"
    stop
  else
    call check(nf90_inquire_dimension(inputID, LonDimID, name = dimName, len = dimSize))
    call check(nf90_inq_varid(inputID, dimName, LonVarID))
    allocate(lon(dimSize))
    call check(nf90_get_var(inputID, LonVarID, Lon, (/ 1 /), (/ dimSize /)))
    totalLon=dimSize
    write(*,'(A,A,A)') "OK(", trim(dimName), ")"
  endif

  !get latitude name
  write(*,'(A)',ADVANCE='NO') "   Get Latitude dimension..."
  LatDimID=getLonOrLatID(inputID,0)
  if(LatDimID.eq.-1) then
    write(*,'(A)') "Not Found"
    stop
  else
    call check(nf90_inquire_dimension(inputID, LatDimID, name = dimName, len = dimSize))
    call check(nf90_inq_varid(inputID, dimName, LatVarID))
    allocate(lat(dimSize))
    call check(nf90_get_var(inputID, LatVarID, Lat, (/ 1 /), (/ dimSize /)))
    totalLat=dimSize
    write(*,'(A,A,A)') "OK(", trim(dimName), ")"
  endif


  write(*,'(A)',ADVANCE='NO') "Define box..."
  allocate(deftab(size(lon),size(lat)))
  call MakeBox(size(lon), size(lat), lon_min, lon_max, lat_min, lat_max, deftab)
  write(*,*) lon_min, lon_max, lat_min, lat_max


  write(*,'(A)') "Create output file :"
  call check(nf90_create(trim(filenameOut), nf90_hdf5, outputID))   !write target file
  
  
  !define dimension, variable associated and attribut
  write(*,'(A)',ADVANCE='NO') "   Define dimensions..."
  do i=1, nbDim
    !get name and size of dimension
    call check(nf90_inquire_dimension(inputID, i, name = dimName, len = dimSize))
    dimNameList(i)=dimName

    !define dimension with separation between unlimited, normal dimension and Lon/Lat
    if(i.eq.RecordDimID) then
      call check(nf90_def_dim(outputID, dimName, NF90_UNLIMITED, outdimid))
    elseif(i.eq.LonDimID) then
      call check(nf90_def_dim(outputID, dimName, 1, outdimid))
    elseif(i.eq.LatDimID) then
      call check(nf90_def_dim(outputID, dimName, 1, outdimid))
    else
      call check(nf90_def_dim(outputID, dimName, dimSize, outdimid))
    endif
  enddo
  write(*,*) "OK"


  !define variable from the list
  write(*,'(A)',ADVANCE='NO') "   Define variables and copy attributs..."
  do i=1, nbVar
    call check(nf90_inquire_variable(inputID, i, dimName, xtype=variableType, nAtts=nbAtt, ndims=nbvarDim, dimids=varDimID))
    if(nf90_inq_dimid(inputID, dimName, outvarID).eq.nf90_noerr) then
      allocate(varDimIDshort(nbvarDim)); varDimIDshort=varDimID(1:nbvarDim)
      call check(nf90_def_var(outputID, dimName, variableType, varDimIDshort, outvarID, shuffle = .TRUE., deflate_level=1))
      do j=1, nbAtt
        call check(nf90_inq_attname(inputID, i, j, attName))
        call check(nf90_copy_att(inputID, i, attName, outputID, outvarID))
      enddo
      deallocate(varDimIDshort)
    endif
  enddo
  allocate(varDimIDshort(1)); varDimIDshort(1)=RecordDimID
  call check(nf90_def_var(outputID, "IceExt", NF90_DOUBLE, varDimIDshort, IceExtID, shuffle = .TRUE., deflate_level=1))
  call check(nf90_put_att(outputID, IceExtID, "long_name", "Sea Ice Extention"))
  call check(nf90_put_att(outputID, IceExtID, "units", "10^6km2"))
  call check(nf90_def_var(outputID, "IceExt_nb", NF90_INT, varDimIDshort, IceExtNbID, shuffle = .TRUE., deflate_level=1))
  call check(nf90_put_att(outputID, IceExtNbID, "long_name", "Nb grid point in Sea Ice Extention"))
  call check(nf90_put_att(outputID, IceExtNbID, "units", "number"))
  call check(nf90_def_var(outputID, "IceArea", NF90_DOUBLE, varDimIDshort, IceAreaID, shuffle = .TRUE., deflate_level=1))
  call check(nf90_put_att(outputID, IceAreaID, "long_name", "Sea Ice Aera"))
  call check(nf90_put_att(outputID, IceAreaID, "units", "Mm2"))
  deallocate(varDimIDshort)
  write(*,*) "OK"


  !finish the configuration of the output file and starting put data
  status = nf90_enddef(outputID)


  !copy dimension data
  write(*,'(A)',ADVANCE='NO') "Copy dimensions data in output file..."
  do i=1, nbDim
    call check(nf90_inquire_dimension(inputID, i, name = dimName, len = dimSize))
    if(nf90_inq_varid(inputID, dimName, varID).eq.nf90_noerr) then
      if(nf90_inq_varid(outputID, dimName, outvarid).eq.nf90_noerr) then
        call check(nf90_inquire_variable(inputID, varID, xtype = variableType, nAtts =nbAtt))
        if(i.eq.LonDimID) then
          dimSize=1
        elseif(i.eq.LatDimID) then
          dimSize=1
        endif
        allocate(valueDbl(dimSize))
        call check(nf90_get_var(inputID, varID, valueDbl, (/ 1 /), (/ dimSize /)))
        call check(nf90_put_var(outputID, outvarid, valueDbl, (/ 1 /), (/ dimSize /)))
        deallocate(valueDbl)
      endif
    endif
  enddo
  write(*,*) "OK"

  !copy dimension data
  write(*,'(A)',ADVANCE='NO') "Calculate cos value to speed up..."
  allocate(surface(size(lat)))
  do iLat=1, totalLat
!    surface(iLat)=2*Pi/real(totalLon)*(cos((90.-lat(iLat)-1.5)*radian)-cos((90.-lat(iLat)+1.5)*radian))*6.371*6.371
     surface(iLat)=3.0*radian*(cos(lat(iLat)*radian))*3.0*radian*6.371*6.371
  enddo
  write(*,*) "OK"

  !copy value 
  if(nf90_inq_varid(inputID, "albq", i).ne.nf90_noerr) then
    write(*,*) "albq variable not found !!"
    stop
  endif
  call check(nf90_inquire_variable(inputID, i, xtype = variableType, ndims = nbvarDim, dimids = varDimID))
  allocate(pdimSize(nbvarDim))
  allocate(start(nbvarDim))
  start=1
  idxRec=-1; idxLon=-1; idxLat=-1
  do j=1, nbvarDim
    call check(nf90_inquire_dimension(inputID, varDimID(j), DimName, nRecords))
    if(varDimID(j).eq.RecordDimID) then  
      pdimSize(j)=nRecords!1
      idxRec=j
    elseif(varDimID(j).eq.LonDimID) then  
      pdimSize(j)=1
      idxLon=j
    elseif(varDimID(j).eq.LatDimID) then  
      pdimSize(j)=1
      idxLat=j
    else
      pdimSize(j)=nRecords
    endif
  enddo

  !check if variable contain lon, lat and time
  if((idxLon.ne.-1).and.(idxLat.ne.-1).and.(idxRec.ne.-1)) then
    write(6,'(A,d,A)',ADVANCE='NO') "Processing (undef=~", undef(i), ")..."

    allocate(valueDbl1D(product(pdimSize)))
    allocate(nbPoint(product(pdimSize)))
    allocate(sumDbl(product(pdimSize)))
    allocate(IceExtDbl(product(pdimSize)))
    allocate(IceAreaDbl(product(pdimSize)))
    productionPdimSize=product(pdimSize)

    do t=1, totaltime
      write(6,'(A)',ADVANCE='NO') '.'
      close(6)
      open(6)
      nbPoint=0
      sumDbl=0.
      IceExtDbl=0.
      IceAreaDbl=0.
      start(idxRec)=t
      pdimSize(idxRec)=1
      do iLon=1, totalLon
        do iLat=1, totalLat
          if(deftab(iLon,iLat).eq.1) then
            start(idxLon)=iLon
            start(idxLat)=iLat
            call check(nf90_get_var(inputID, i, valueDbl1D, start, pdimSize))
            do j=1, productionPdimSize
              if((valueDbl1D(j).lt.undefMinTab(i)).or.(valueDbl1D(j).gt.undefMaxTab(i))) then
                if(valueDbl1D(j).ge.seuil) then
                  IceExtDbl(j)=IceExtDbl(j)+surface(iLat)
                  nbPoint(j)=nbPoint(j)+1
                endif
                IceAreaDbl(j)=IceAreaDbl(j)+valueDbl1D(j)*surface(iLat)
              endif
            enddo
          endif
        enddo
      enddo
      call check(nf90_put_var(outputID, IceExtID, IceExtDbl, (/t/), (/1/)))
      call check(nf90_put_var(outputID, IceExtNbID, nbPoint, (/t/), (/1/)))
      call check(nf90_put_var(outputID, IceAreaID, IceAreaDbl, (/t/), (/1/)))
    enddo

    deallocate(valueDbl1D)
    deallocate(nbPoint)
    deallocate(sumDbl)
    deallocate(IceExtDbl)
    write(*,'(A)') "OK"
  else
    write(*,'(A,A)') "albq is skipped"
  endif
  if(allocated(start)) deallocate(start)
  if(allocated(pdimSize)) deallocate(pdimSize)
  
  
  !close file
  call check(nf90_close(inputID));
  call check(nf90_close(outputID));

END PROGRAM Iceifaire

