subroutine usage()
  write(*,'(A)') "Usage:"
  write(*,'(A)') "  ./IceifaireClio [threshold] [lon_min(deg)] [lon_max(deg)] [lat_min(deg)] [lat_max(deg)] [input.nc] [output.nc]"
  write(*,'(A)') "  ./IceifaireClio [threshold] [input.nc] [output.nc]  <-- Box is entire world"
  stop
end subroutine usage

include "NetcdfFonction.f90"

PROGRAM Iceifaire 
  USE NETCDF
  IMPLICIT NONE

  CHARACTER(nf90_max_name) :: filenameIn, filenameOut
 
  INTEGER :: i, j, t, status

  integer :: inputID, outputID, RecordDimID, LonDimID, LatDimID, DimID
  integer :: nbDim, nbVar, nbAtt, getTime
  integer :: dimSize, IceExtID, IceExtNbID, IceAreaID
  integer :: variableType, albqID, areaID
  double precision, dimension(nf90_max_var_dims) :: undefMinTab, undefMaxTab, undef
  character(nf90_max_name) :: dimName, parm, AttName
  double precision, dimension(:), allocatable :: valueDbl

  double precision, dimension(:), allocatable :: valueDbl1D
  double precision, dimension(120,65) :: Lat, Lon
  double precision, dimension(120,65) :: surface
  double precision, dimension(:), allocatable :: IceExtDbl, IceAreaDbl
  integer, dimension(:), allocatable :: nbPoint

  integer :: totaltime, iLon, iLat

  !variable pour les box
  double precision :: lon_minDbl, lon_maxDbl, lat_minDbl, lat_maxDbl
  integer, dimension(120,65) :: deftab
  double precision :: radian, Pi, seuil


  !GET ARGUMENT FOR SELECT METHODE
  if(iargc().gt.2) then
    if(iargc().eq.7) then
      call getarg(1,parm); read(parm,*) seuil
      call getarg(2,parm); read(parm,*) lon_minDbl
      call getarg(3,parm); read(parm,*) lon_maxDbl
      call getarg(4,parm); read(parm,*) lat_minDbl
      call getarg(5,parm); read(parm,*) lat_maxDbl
      call getarg(6,filenameIn)
      call getarg(7,filenameOut)
    elseif(iargc().eq.3) then
      lon_minDbl=-1.0
      lon_maxDbl=-1.0
      lat_minDbl=-1.0
      lat_maxDbl=-1.0
      call getarg(1,parm); read(parm,*) seuil
      call getarg(2,filenameIn)
      call getarg(3,filenameOut)
    else
      call usage()
    endif
  else
    call usage()
  endif
  
  Pi=2*acos(0.0)
  radian=Pi/180.0


  write(*,'(A)',ADVANCE='NO') "Open input file..."
  call check(nf90_open(filenameIn, nf90_nowrite, inputID))
  write(*,*) "OK"


  write(*,'(A)',ADVANCE='NO') "   Get some value..."
  !get nbdim and nbvar
  call check(nf90_inquire(inputID, nbDim, nbVar))
  !get totaltime
  totaltime=getTime(inputID, RecordDimID)
  !get undef value
  do i=1, nbVar
    call GetUndefForVar(inputID, i, undef(i), undefMinTab(i), undefMaxTab(i))
  enddo
  !deg to radian conversion
  radian=2*acos(0.0)/180.0
  write(*,*) "OK"

  !get longitude name
  write(*,'(A)',ADVANCE='NO') "Get Longitude matrix..."
  call check(nf90_inq_varid(inputID, "tlon", LonDimID))
  call check(nf90_get_var(inputID, LonDimID, Lon, (/1,1/), (/120,65/)))
  !write(*,*) "tlon", Lon
  write(*,'(A,A,A)') "OK(", trim(dimName), ")"

  !get latitude name
  write(*,'(A)',ADVANCE='NO') "Get Latitude matrix..."
  call check(nf90_inq_varid(inputID, "tlat", LatDimID))
  call check(nf90_get_var(inputID, LatDimID, Lat, (/1,1/), (/120,65/)))
  !write(*,*) "tlat", Lat
  write(*,'(A,A,A)') "OK(", trim(dimName), ")"

  write(*,'(A)',ADVANCE='NO') "Define box..."
  call MakeBoxClio(lon, lat, lon_minDbl, lon_maxDbl, lat_minDbl, lat_maxDbl, deftab)
  write(*,*) lon_minDbl, lon_maxDbl, lat_minDbl, lat_maxDbl

  write(*,'(A)') "Create output file :"
  call check(nf90_create(trim(filenameOut), nf90_hdf5, outputID))   !write target file
  
  
  !define dimension, variable associated and attribut
  write(*,'(A)',ADVANCE='NO') "Define dimensions..."

  call check(nf90_def_dim(outputID, "lon", 1, DimID))
  call check(nf90_def_var(outputID, "lon", NF90_DOUBLE, (/DimID/), DimID))
  call check(nf90_put_att(outputID, DimID, "long_name", "longitude"))
  call check(nf90_put_att(outputID, DimID, "units", "degrees_east"))
  call check(nf90_put_att(outputID, DimID, "axis", "X"))
  call check(nf90_put_var(outputID, DimID, (/1/), (/1/), (/1/)))

  call check(nf90_def_dim(outputID, "lat", 1, DimID))
  call check(nf90_def_var(outputID, "lat", NF90_DOUBLE, (/DimID/), DimID))
  call check(nf90_put_att(outputID, DimID, "long_name", "latitude"))
  call check(nf90_put_att(outputID, DimID, "units", "degrees_north"))
  call check(nf90_put_att(outputID, DimID, "axis", "Y"))
  call check(nf90_put_var(outputID, DimID, (/1/), (/1/), (/1/)))

  call check(nf90_inquire_dimension(inputID, RecordDimID, name = dimName, len = dimSize))
  call check(nf90_def_dim(outputID, dimName, NF90_UNLIMITED, DimID))
  call check(nf90_inq_varid(inputID, "time", RecordDimID))
  call check(nf90_inquire_variable(inputID, RecordDimID, xtype=variableType, nAtts=nbAtt))
  call check(nf90_def_var(outputID, "time", variableType, (/DimID/), DimID))
  do j=1, nbAtt
    call check(nf90_inq_attname(inputID, RecordDimID, j, attName))
    call check(nf90_copy_att(inputID, RecordDimID, attName, outputID, DimID))
  enddo
  allocate(valueDbl(totaltime))
  call check(nf90_get_var(inputID, RecordDimID, valueDbl, (/1/), (/totaltime/)))
  call check(nf90_put_var(outputID, DimID, valueDbl, (/1/), (/totaltime/)))
  deallocate(valueDbl)
  write(*,*) "OK"


  !define variable from the list
  write(*,'(A)',ADVANCE='NO') "Define variables and copy attributs..."
  call check(nf90_def_var(outputID, "IceExt", NF90_DOUBLE, (/3/), IceExtID, shuffle = .TRUE., deflate_level=1))
  call check(nf90_put_att(outputID, IceExtID, "long_name", "Sea Ice Extention"))
  call check(nf90_put_att(outputID, IceExtID, "units", "Mm2"))
  call check(nf90_def_var(outputID, "IceExt_nb", NF90_INT, (/3/), IceExtNbID, shuffle = .TRUE., deflate_level=1))
  call check(nf90_put_att(outputID, IceExtNbID, "long_name", "Nb grid point in Sea Ice Extention"))
  call check(nf90_put_att(outputID, IceExtNbID, "units", "number"))
  call check(nf90_def_var(outputID, "IceArea", NF90_DOUBLE, (/3/), IceAreaID, shuffle = .TRUE., deflate_level=1))
  call check(nf90_put_att(outputID, IceAreaID, "long_name", "Sea Ice Aera"))
  call check(nf90_put_att(outputID, IceAreaID, "units", "Mm2"))
  write(*,*) "OK"


  !finish the configuration of the output file and starting put data
  status = nf90_enddef(outputID)


!   !copy dimension data
!   write(*,'(A)',ADVANCE='NO') "Copy dimensions data in output file..."
!   do i=1, nbDim
!     call check(nf90_inquire_dimension(inputID, i, name = dimName, len = dimSize))
!     if(nf90_inq_varid(inputID, dimName, varID).eq.nf90_noerr) then
!       if(nf90_inq_varid(outputID, dimName, outvarid).eq.nf90_noerr) then
!         call check(nf90_inquire_variable(inputID, varID, xtype = variableType, nAtts =nbAtt))
!         if(i.eq.LonDimID) then
!           dimSize=1
!         elseif(i.eq.LatDimID) then
!           dimSize=1
!         endif
!         allocate(valueDbl(dimSize))
!         call check(nf90_get_var(inputID, varID, valueDbl, (/ 1 /), (/ dimSize /)))
!         call check(nf90_put_var(outputID, outvarid, valueDbl, (/ 1 /), (/ dimSize /)))
!         deallocate(valueDbl)
!       endif
!     endif
!   enddo
!   write(*,*) "OK"

  !copy dimension data
  !get longitude name
  write(*,'(A)',ADVANCE='NO') "Load area grid matrix..."
  if(nf90_inq_varid(inputID, "area", areaID).ne.nf90_noerr) then
    write(*,*) "area variable not found !!"
    stop
  endif
  call check(nf90_get_var(inputID, areaID, surface, (/1,1/), (/120,65/)))
  surface=surface/(1E6*1E6)
  write(*,*) "OK"

  !check usefull variable
  if(nf90_inq_varid(inputID, "albq", albqID).ne.nf90_noerr) then
    write(*,*) "albq variable not found !!"
    stop
  endif
 
  !Compute sea ice extention
  write(6,'(A,d,A)',ADVANCE='NO') "Processing (undef=~", undef(albqID), ")..."
  allocate(valueDbl1D(1))
  allocate(nbPoint(1))
  allocate(IceExtDbl(1))
  allocate(IceAreaDbl(1))
  do t=1, totaltime
    write(6,'(A)',ADVANCE='NO') '.'
    close(6)
    open(6)
    nbPoint=0
    IceExtDbl=0.
    IceAreaDbl=0.
    do iLon=1, 120
      do iLat=1, 65
        if(deftab(iLon,iLat).eq.1) then
          if(surface(iLon,iLat).gt.0) then
          call check(nf90_get_var(inputID, albqID, valueDbl1D, (/iLon,iLat,t/), (/1,1,1/)))
            if((valueDbl1D(1).lt.undefMinTab(albqID)).or.(valueDbl1D(1).gt.undefMaxTab(albqID))) then
              if(valueDbl1D(1).gt.seuil) then
                IceExtDbl(1)=IceExtDbl(1)+surface(iLon,iLat)
                nbPoint(1)=nbPoint(1)+1
              endif
            IceAreaDbl(1)=IceAreaDbl(1)+valueDbl1D(1)*surface(iLon,iLat)
            endif
          endif
        endif
      enddo
    enddo
    !write(*,*) IceExtDbl(1)
    call check(nf90_put_var(outputID, IceExtID, IceExtDbl, (/t/), (/1/)))
    call check(nf90_put_var(outputID, IceExtNbID, nbPoint, (/t/), (/1/)))
    call check(nf90_put_var(outputID, IceAreaID, IceAreaDbl, (/t/), (/1/)))
  enddo

  deallocate(valueDbl1D)
  deallocate(nbPoint)
  deallocate(IceExtDbl)
  write(*,'(A)') "OK"
  
  !close file
  call check(nf90_close(inputID));
  call check(nf90_close(outputID));

END PROGRAM Iceifaire

