subroutine usage()
  write(*,'(A)') "Usage:"
  write(*,'(A)') "  ./MoyenneGlissante (-undef/-u) [input.nc] [output.nc] [length] (window)"
  write(*,'(A)') "  -undef = if point missing in door, value = undef"
  write(*,'(A)') "  window : 1(default) -> Rectangular window"
  write(*,'(A)') "           5          -> Lanczos window"
  write(*,'(A)') "           6          -> Blackman window"
  write(*,'(A)') "           2          -> Hann window"
  write(*,'(A)') "           3          -> Hamming window"
  write(*,'(A)') "           4          -> Cosine window"
  write(*,'(A)') "  more information on http://en.wikipedia.org/wiki/Window_function"
  stop
end subroutine usage

function sinc(x)
  IMPLICIT  NONE
  double precision :: sinc
  double precision :: x, pi
  pi=acos(0.D0)*2.D0
  if(x.ne.0) then
    sinc=sin(pi*x)/(pi*x)
  else
    sinc=1.
  endif
end function sinc

include "NetcdfFonction.f90"

PROGRAM MoyenneGlissante 
  USE NETCDF
  IMPLICIT NONE
  
  interface
    subroutine CopyVariableFrame(intputID, outputID, tabsuffix, tabIdxDimNeeded, replace, valundef)
      USE NETCDF
      integer :: intputID, outputID
      character(nf90_max_name), dimension(:), allocatable, optional :: tabsuffix
      integer, dimension(:), allocatable, optional :: tabIdxDimNeeded
      integer, optional :: replace
      double precision, dimension(nf90_max_var_dims), optional :: valundef
    end subroutine CopyVariableFrame
  end interface
  
  CHARACTER(nf90_max_name) :: filenameIn, filenameOut
  INTEGER, DIMENSION(:), allocatable :: pdimSize, MaxdimSize
  INTEGER :: i, j, s, k, w, nRecords, idxRec, status
  INTEGER, DIMENSION(:), allocatable :: start
  double precision, dimension(nf90_max_var_dims) :: undefMinTab, undefMaxTab, undef
  integer :: intputID, outputID, RecordDimID, RecordVarID, outvarid
  integer :: nbDim, nbVar, isdim, nbvarDim, dimSize, varID
  integer, dimension(nf90_max_var_dims) :: varDimID
  character(nf90_max_name) :: varName, dimName, parm
  double precision, dimension(:), allocatable :: valueDbl, window
  double precision, dimension(:,:,:,:,:), allocatable :: valueDbl5D, valueDbl5Dext
  integer :: totaltime, withundef, getTime
  integer :: length, demilength
  double precision :: pi, sinc

  pi=acos(0.D0)*2.D0

  !GET ARGUMENT FOR SELECT METHODE
  withundef=0
  if(iargc().ge.3) then
    do i=1, 2
      call getarg(i,parm)
      if((parm.eq."-undef").or.(parm.eq."-u")) then
        withundef=1
      endif
    enddo
  else
    call usage()
  endif
  if(withundef.eq.0) then
    if(iargc().eq.3) then
      call getarg(1,filenameIn)
      call getarg(2,filenameOut)
      call getarg(3,parm); read(parm,*) length
      w=1
    elseif(iargc().eq.4) then
      call getarg(1,filenameIn)
      call getarg(2,filenameOut)
      call getarg(3,parm); read(parm,*) length
      call getarg(4,parm); read(parm,*) w
    else
      call usage()
    endif
  else
    if(iargc().eq.4) then
      call getarg(2,filenameIn)
      call getarg(3,filenameOut)
      call getarg(4,parm); read(parm,*) length
      w=1
    elseif(iargc().eq.5) then
      call getarg(2,filenameIn)
      call getarg(3,filenameOut)
      call getarg(4,parm); read(parm,*) length
      call getarg(5,parm); read(parm,*) w
    else
      call usage()
    endif
  endif
  if(mod(length,2).eq.0) then
    write(*,*) length, " is not odd"
    call usage()
  endif
  demilength=int(length/2.)


  write(*,'(A)',ADVANCE='NO') "Create window for moving average..."
  allocate(window(-demilength:demilength))
  if(w.eq.1) then
    window=1.
    write(*,'(A)',ADVANCE='NO')  "Rectangular..."
  elseif(w.eq.2) then
    do i=0, length-1
      window(i-demilength)=0.5*(1.-cos(2.*pi*i/real(length-1)))
    enddo
    write(*,'(A)',ADVANCE='NO')  "Hann..."
  elseif(w.eq.3) then
    do i=0, length-1
      window(i-demilength)=0.54-0.46*cos(2.*pi*i/real(length-1))
    enddo
    write(*,'(A)',ADVANCE='NO')  "Hamming..."
  elseif(w.eq.4) then
    do i=0, length-1
      window(i-demilength)=sin(pi*i/real(length-1))
    enddo
    write(*,'(A)',ADVANCE='NO')  "Cosine..."
  elseif(w.eq.5) then
    do i=0, length-1
      window(i-demilength)=sinc(dble((2.*i/real(length-1))-1.))
    enddo
    write(*,'(A)',ADVANCE='NO')  "Lanczos..."
  elseif(w.eq.6) then
    do i=0, length-1
      window(i-demilength)=(1.-0.16)*0.5-0.5*cos(2.*pi*i/real(length-1))+0.16*0.5*cos(4*pi*i/real(length-1))
    enddo
    write(*,'(A)',ADVANCE='NO')  "Blackman..."
  else
    write(*,*) "ERROR"
    call usage()
  endif
  window=window/sum(window)
  write(*,*) "OK"
!   do i=-demilength, demilength
!     write(*,*) window(i)
!   enddo

  write(*,'(A)',ADVANCE='NO') "Open input file..."
  call check(nf90_open(filenameIn, nf90_nowrite, intputID))
  write(*,*) "OK"


  write(*,'(A)',ADVANCE='NO') "   Get some value..."
  !get nbdim and nbvar
  call check(nf90_inquire(intputID, nbDim, nbVar))
  !get totaltime
  totaltime=getTime(intputID, RecordDimID)
  !get undef value
  do i=1, nbVar
    call GetUndefForVar(intputID, i, undef(i), undefMinTab(i), undefMaxTab(i))
  enddo
  write(*,*) "OK"
  

  write(*,'(A)') "Create output file :"
  call check(nf90_create(trim(filenameOut), nf90_hdf5, outputID))   !write target file
  
  !define dimension, variable associated and attribut
  write(*,'(A)',ADVANCE='NO') "   Define dimensions..."
  call CopyDimensionFrame(intputID, outputID)
  write(*,*) "OK"


  !define variable from the list
  write(*,'(A)',ADVANCE='NO') "   Define variables and copy attributs..."
  call CopyVariableFrame(intputID, outputID)
  write(*,*) "OK"


  !finish the configuration of the output file and starting put data
  status = nf90_enddef(outputID)


  !copy dimension data
  write(*,'(A)',ADVANCE='NO') "Copy dimensions data in output file..."
  do i=1, nbDim
    call check(nf90_inquire_dimension(intputID, i, name = dimName, len = dimSize))
    if(nf90_inq_varid(intputID, dimName, varID).eq.nf90_noerr) then
      varName=dimName;
    elseif(nf90_inq_varid(intputID, trim(dimName)//"_bounds", varID).eq.nf90_noerr) then
      varName=trim(dimName)//"_bounds";
    else
      varName="-1"
    endif
    if(varName.ne."-1") then
      call check(nf90_inq_varid(outputID, varName, outvarid))
      call check(nf90_inquire_variable(outputID, outvarid, varName, ndims = nbvarDim, dimids = varDimID))
      allocate(pdimSize(nbvarDim))
      allocate(start(nbvarDim))
      start=1
      do j=1, nbvarDim
        call check(nf90_inquire_dimension(intputID, varDimID(j), DimName, nRecords))
        pdimSize(j)=nRecords
      enddo
      if(i.eq.RecordDimID) RecordVarID=varID
      allocate(valueDbl(product(pdimSize)))
      call check(nf90_get_var(intputID, varID, valueDbl, start, pdimSize))
      call check(nf90_put_var(outputID, outvarid, valueDbl, start, pdimSize))
      deallocate(valueDbl)
      deallocate(start)
      deallocate(pdimSize)
    endif
  enddo
  write(*,*) "OK"

  !copy value
  do i=1, nbVar
    call check(nf90_inquire_variable(intputID, i, varName,  ndims = nbvarDim, dimids = varDimID))

    if(nf90_inq_dimid(intputID, varName, varID).eq.nf90_noerr) then
      isdim=1
    elseif(index(varName,"_bounds").ne.0) then
      isdim=1
    else
      isdim=0
    endif

    if(isdim.eq.0) then
      write(*,'(A,A,d,A)',ADVANCE='NO') trim(varName), " is processing (undef=~", undef(i), ")..."
      allocate(pdimSize(nbvarDim))
      allocate(MaxdimSize(5))
      allocate(start(nbvarDim))
      MaxdimSize=1
      start=1
      idxRec=-1;
      do j=1, nbvarDim
        call check(nf90_inquire_dimension(intputID, varDimID(j), DimName, nRecords))
        MaxdimSize(j)=nRecords
        if(varDimID(j).eq.RecordDimID) then  
          pdimSize(j)=nRecords
          idxRec=j
        else
          pdimSize(j)=nRecords
        endif
      enddo

      allocate(valueDbl5D(MaxdimSize(1),MaxdimSize(2),MaxdimSize(3),MaxdimSize(4), MaxdimSize(5)))
      call check(nf90_get_var(intputID, i, valueDbl5D, start, pdimSize))      

      if(idxRec.eq.1) then
        allocate(valueDbl5Dext(MaxdimSize(1)+demilength*2,MaxdimSize(2),MaxdimSize(3),MaxdimSize(4), MaxdimSize(5)))
        valueDbl5Dext(1+demilength:MaxdimSize(1)+demilength,:,:,:,:)=valueDbl5D(:,:,:,:,:)
        do k=1, demilength
          valueDbl5Dext(k,:,:,:,:)=2*valueDbl5D(1,:,:,:,:)-valueDbl5D(demilength-k+1,:,:,:,:)
          valueDbl5Dext(MaxdimSize(1)+demilength+k,:,:,:,:)=2*valueDbl5D(MaxdimSize(1),:,:,:,:)-valueDbl5D(MaxdimSize(1)-k,:,:,:,:)
        enddo
      elseif(idxRec.eq.2) then
        allocate(valueDbl5Dext(MaxdimSize(1),1+MaxdimSize(2)+demilength*2,MaxdimSize(3),MaxdimSize(4), MaxdimSize(5)))
        valueDbl5Dext(:,1+demilength:MaxdimSize(2)+demilength,:,:,:)=valueDbl5D(:,:,:,:,:)
        do k=1, demilength
          valueDbl5Dext(:,k,:,:,:)=2*valueDbl5D(:,1,:,:,:)-valueDbl5D(:,demilength-k+1,:,:,:)
          valueDbl5Dext(:,MaxdimSize(2)+demilength+k,:,:,:)=2*valueDbl5D(:,MaxdimSize(2),:,:,:)-valueDbl5D(:,MaxdimSize(2)-k,:,:,:)
        enddo
      elseif(idxRec.eq.3) then
        allocate(valueDbl5Dext(MaxdimSize(1),MaxdimSize(2),1+MaxdimSize(3)+demilength*2,MaxdimSize(4), MaxdimSize(5)))
        valueDbl5Dext(:,:,1+demilength:MaxdimSize(3)+demilength,:,:)=valueDbl5D(:,:,:,:,:)
        do k=1, demilength
          valueDbl5Dext(:,:,k,:,:)=2*valueDbl5D(:,:,1,:,:)-valueDbl5D(:,:,demilength-k+1,:,:)
          valueDbl5Dext(:,:,MaxdimSize(3)+demilength+k,:,:)=2*valueDbl5D(:,:,MaxdimSize(3),:,:)-valueDbl5D(:,:,MaxdimSize(3)-k,:,:)
        enddo
      elseif(idxRec.eq.4) then
        allocate(valueDbl5Dext(MaxdimSize(1),MaxdimSize(2),MaxdimSize(3),1+MaxdimSize(4)+demilength*2, MaxdimSize(5)))
        valueDbl5Dext(:,:,:,1+demilength:MaxdimSize(4)+demilength,:)=valueDbl5D(:,:,:,:,:)
        do k=1, demilength
          valueDbl5Dext(:,:,:,k,:)=2*valueDbl5D(:,:,:,1,:)-valueDbl5D(:,:,:,demilength-k+1,:)
          valueDbl5Dext(:,:,:,MaxdimSize(4)+demilength+k,:)=2*valueDbl5D(:,:,:,MaxdimSize(4),:)-valueDbl5D(:,:,:,MaxdimSize(4)-k,:)
        enddo
      elseif(idxRec.eq.5) then
        allocate(valueDbl5Dext(MaxdimSize(1),MaxdimSize(2),MaxdimSize(3),MaxdimSize(4), 1+MaxdimSize(5)+demilength*2))
        valueDbl5Dext(:,:,:,:,1+demilength:MaxdimSize(5)+demilength)=valueDbl5D(:,:,:,:,:)
        do k=1, demilength
          valueDbl5Dext(:,:,:,:,k)=2*valueDbl5D(:,:,:,:,1)-valueDbl5D(:,:,:,:,demilength-k+1)
          valueDbl5Dext(:,:,:,:,MaxdimSize(5)+demilength+k)=2*valueDbl5D(:,:,:,:,MaxdimSize(5))-valueDbl5D(:,:,:,:,MaxdimSize(5)-k)
        enddo
      endif

      valueDbl5D=0


      if(idxRec.eq.1) then
        do k=1, MaxdimSize(idxRec)
          do s=-demilength, demilength
            valueDbl5D(k,:,:,:,:)=valueDbl5D(k,:,:,:,:)+window(s)*valueDbl5Dext(k+demilength+s,:,:,:,:)
          enddo
        enddo
      elseif(idxRec.eq.2) then
        do k=1, MaxdimSize(idxRec)
          do s=-demilength, demilength
            valueDbl5D(:,k,:,:,:)=valueDbl5D(:,k,:,:,:)+window(s)*valueDbl5Dext(:,k+demilength+s,:,:,:)
          enddo
        enddo
      elseif(idxRec.eq.3) then
        do k=1, MaxdimSize(idxRec)
!           if(k.lt.150) then 
!             valueDbl5D(:,:,k,:,:)=1.0
!           else
!             valueDbl5D(:,:,k,:,:)=-1.0
!           endif
          do s=-demilength, demilength
            valueDbl5D(:,:,k,:,:)=valueDbl5D(:,:,k,:,:)+window(s)*valueDbl5Dext(:,:,k+demilength+s,:,:)
          enddo
        enddo
      elseif(idxRec.eq.4) then
        do k=1, MaxdimSize(idxRec)
          do s=-demilength, demilength
            valueDbl5D(:,:,:,k,:)=valueDbl5D(:,:,:,k,:)+window(s)*valueDbl5Dext(:,:,:,k+demilength+s,:)
          enddo
        enddo
      elseif(idxRec.eq.5) then
        do k=1, MaxdimSize(idxRec)
          do s=-demilength, demilength
            valueDbl5D(:,:,:,:,k)=valueDbl5D(:,:,:,:,k)+window(s)*valueDbl5Dext(:,:,:,:,k+demilength+s)
          enddo
        enddo
      endif

      call check(nf90_put_var(outputID, i, valueDbl5D, start, pdimSize))

      write(*,'(A)') "OK"
      deallocate(valueDbl5D)
      deallocate(valueDbl5Dext)
      deallocate(pdimSize)
      deallocate(MaxdimSize)
      deallocate(start)
    endif
  enddo
  
  !close file
  call check(nf90_close(intputID));
  call check(nf90_close(outputID));
END PROGRAM MoyenneGlissante



