subroutine usage()
  write(*,'(A)') "Usage:"
  write(*,'(A)') "  ./MoyenneTempo (-nb/-n) (-undef/-u) [input.nc] [output.nc] (mod)"
  write(*,'(A)') "  -nb = with nb point in mean is stored"
  write(*,'(A)') "  -undef = if point missing in mean, mean value = undef"
  write(*,'(A)') "  mod :"
  write(*,'(A)') "    Default 1"
  write(*,'(A)') "    1 = JFM(1-3), AMJ(4-6), JAS(7-9), OND(10-12)"
  write(*,'(A)') "    2 = DJF(12-2), MAM(3-5), JJA(6-8), SON(9-11)"
  write(*,'(A)') "    3 = ONDJFM(10-3), AMJJAS(4-9)"
  write(*,'(A)') "    4 = Annual(1-12)"
  write(*,'(A)') "    5 = Mensuelle"
  stop
end subroutine usage

include "NetcdfFonction.f90"

PROGRAM MoyenneTempo 
  USE NETCDF
  IMPLICIT NONE
  
  interface
    subroutine month2SaisonAvg_Dbl(undef, undefMin, undefMax, inputtab, outputtab, outputNb, mode, undefmode)
      double precision :: undef, undefMin, undefMax
      double precision, DIMENSION(:,:), ALLOCATABLE :: inputtab
      double precision, DIMENSION(:,:), ALLOCATABLE :: outputtab
      integer, DIMENSION(:,:), ALLOCATABLE :: outputNb
      integer :: mode, undefmode
    end subroutine month2SaisonAvg_Dbl
    subroutine month2SaisonAvg_Flt(undef, undefMin, undefMax, inputtab, outputtab, outputNb, mode, undefmode)
      double precision :: undef, undefMin, undefMax
      REAL, DIMENSION(:,:), ALLOCATABLE :: inputtab
      REAL, DIMENSION(:,:), ALLOCATABLE :: outputtab
      integer, DIMENSION(:,:), ALLOCATABLE :: outputNb
      integer :: mode, undefmode
    end subroutine month2SaisonAvg_Flt
    subroutine month2SaisonAvg_Int(undef, undefMin, undefMax, inputtab, outputtab, outputNb, mode, undefmode)
      double precision :: undef, undefMin, undefMax
      integer, DIMENSION(:,:), ALLOCATABLE :: inputtab
      integer, DIMENSION(:,:), ALLOCATABLE :: outputtab
      integer, DIMENSION(:,:), ALLOCATABLE :: outputNb
      integer :: mode, undefmode
    end subroutine month2SaisonAvg_Int
    subroutine CopyVariableFrame(intputID, outputID, tabsuffix, tabIdxDimNeeded, replace, valundef)
      USE NETCDF
      integer :: intputID, outputID
      character(nf90_max_name), dimension(:), allocatable, optional :: tabsuffix
      integer, dimension(:), allocatable, optional :: tabIdxDimNeeded
      integer, optional :: replace
      double precision, dimension(nf90_max_var_dims), optional :: valundef
    end subroutine CopyVariableFrame
    subroutine Get12monthsFromT_Dbl(intputID, varId, t, undef, outputtab, start, pdimSize, idxRec, tmax)
      USE NETCDF
      integer :: intputID, t, idxRec, varId, tmax
      double precision :: undef
      double precision, DIMENSION(:,:), ALLOCATABLE :: outputtab
      integer, DIMENSION(:), ALLOCATABLE :: start, pdimSize
    end subroutine Get12monthsFromT_Dbl
    subroutine Get12monthsFromT_Flt(intputID, varId, t, undef, outputtab, start, pdimSize, idxRec, tmax)
      USE NETCDF
      integer :: intputID, t, idxRec, varId, tmax
      double precision :: undef
      real, DIMENSION(:,:), ALLOCATABLE :: outputtab
      integer, DIMENSION(:), ALLOCATABLE :: start, pdimSize
    end subroutine Get12monthsFromT_Flt
    subroutine Get12monthsFromT_Int(intputID, varId, t, undef, outputtab, start, pdimSize, idxRec, tmax)
      USE NETCDF
      integer :: intputID, t, idxRec, varId, tmax
      double precision :: undef
      integer, DIMENSION(:,:), ALLOCATABLE :: outputtab
      integer, DIMENSION(:), ALLOCATABLE :: start, pdimSize
    end subroutine Get12monthsFromT_Int
  end interface
  
  CHARACTER(nf90_max_name) :: filenameIn, filenameOut
  INTEGER, DIMENSION(:), allocatable :: pdimSize
 
  INTEGER :: i, j, s, t, nRecords, idxRec, idxProcessVar, status
  INTEGER, DIMENSION(:), allocatable :: start

  double precision, dimension(nf90_max_var_dims) :: undefMinTab, undefMaxTab, undef

  integer :: intputID, outputID, RecordDimID, RecordVarID
  integer :: nbDim, nbVar, nbAtt
  integer :: nbvarDim, dimSize, varID
  integer :: variableType, outvarid
  integer, dimension(nf90_max_var_dims) :: varDimID
  character(nf90_max_name) :: varName, dimName, parm
  integer, dimension(:), allocatable :: valueInt
  real, dimension(:), allocatable :: valueFlt
  double precision, dimension(:), allocatable :: valueDbl

  integer, dimension(:,:), allocatable :: valueInt2D, saisInt2D
  real, dimension(:,:), allocatable :: valueFlt2D, saisFlt2D
  double precision, dimension(:,:), allocatable :: valueDbl2D, saisDbl2D

  integer :: totaltime, maxtime, withnb, withundef, getTime

  character(nf90_max_name), dimension(:), allocatable :: tabsuffix
  integer, dimension(:), allocatable :: tabIdxDimNeeded
  integer, dimension(:,:), allocatable :: outputNb
  integer :: mode

  !GET ARGUMENT FOR SELECT METHODE
  withundef=0
  withnb=0
  if(iargc().ge.2) then
    do i=1, 2
      call getarg(i,parm)
      if((parm.eq."-nb").or.(parm.eq."-n")) then
        withnb=1
      endif
      if((parm.eq."-undef").or.(parm.eq."-u")) then
        withundef=1
      endif
    enddo
  else
    call usage()
  endif
  if((withnb.eq.0).and.(withundef.eq.0)) then
    if(iargc().eq.2) then
      call getarg(1,filenameIn)
      call getarg(2,filenameOut)
      mode=1
    elseif(iargc().eq.3) then
      call getarg(1,filenameIn)
      call getarg(2,filenameOut)
      call getarg(3,parm); read(parm,*) mode
    else
      call usage()
    endif
  elseif((withnb.eq.1).xor.(withundef.eq.1)) then
    if(iargc().eq.3) then
      call getarg(2,filenameIn)
      call getarg(3,filenameOut)
      mode=1
    elseif(iargc().eq.4) then
      call getarg(2,filenameIn)
      call getarg(3,filenameOut)
      call getarg(4,parm); read(parm,*) mode
    else
      call usage()
    endif
  elseif((withnb.eq.1).and.(withundef.eq.1)) then
    if(iargc().eq.4) then
      call getarg(3,filenameIn)
      call getarg(4,filenameOut)
      mode=1
    elseif(iargc().eq.5) then
      call getarg(3,filenameIn)
      call getarg(4,filenameOut)
      call getarg(5,parm); read(parm,*) mode
    else
      call usage()
    endif
  endif
  if((mode.gt.5).or.(mode.lt.1)) then
    call usage()
  endif

  write(*,'(A)',ADVANCE='NO') "Open input file..."
  call check(nf90_open(filenameIn, nf90_nowrite, intputID))
  write(*,*) "OK"



  write(*,'(A)',ADVANCE='NO') "   Get some value..."
  !get nbdim and nbvar
  call check(nf90_inquire(intputID, nbDim, nbVar))
  !get totaltime
  totaltime=getTime(intputID, RecordDimID)
  !get undef value
  do i=1, nbVar
    call GetUndefForVar(intputID, i, undef(i), undefMinTab(i), undefMaxTab(i))
  enddo
  write(*,*) "OK"
  

  write(*,'(A)') "Create output file :"
  call check(nf90_create(trim(filenameOut), nf90_hdf5, outputID))   !write target file
  
  !define dimension, variable associated and attribut
  write(*,'(A)',ADVANCE='NO') "   Define dimensions..."
  call CopyDimensionFrame(intputID, outputID)
  write(*,*) "OK"


  !define variable from the list
  write(*,'(A)',ADVANCE='NO') "   Define variables and copy attributs..."
  if(mode.eq.3) then
    if(withnb.eq.0) then
      allocate(tabsuffix(2)); tabsuffix(1)="_ONDJFM"; tabsuffix(2)="_AMJJAS"
      allocate(tabIdxDimNeeded(1)); tabIdxDimNeeded(1)=RecordDimID;
      call CopyVariableFrame(intputID, outputID, tabsuffix, tabIdxDimNeeded, 1)
    else
      allocate(tabsuffix(4))
      tabsuffix(1)="_ONDJFM"; tabsuffix(2)="_ONDJFMnb"; tabsuffix(3)="_AMJJAS"; tabsuffix(4)="_AMJJASnb"
      allocate(tabIdxDimNeeded(1)); tabIdxDimNeeded(1)=RecordDimID;
      call CopyVariableFrame(intputID, outputID, tabsuffix, tabIdxDimNeeded, 1)
    endif
  elseif(mode.eq.4) then
    if(withnb.eq.0) then
      call CopyVariableFrame(intputID, outputID)
    else
      allocate(tabsuffix(1)); tabsuffix(1)="_NbMoy"
      allocate(tabIdxDimNeeded(1)); tabIdxDimNeeded(1)=RecordDimID;
      call CopyVariableFrame(intputID, outputID, tabsuffix, tabIdxDimNeeded, 0)
    endif
  elseif(mode.eq.5) then
    if(withnb.eq.0) then
      allocate(tabsuffix(12)); tabsuffix(1)="_jan"; tabsuffix(2)="_feb"; tabsuffix(3)="_mar"; tabsuffix(4)="_apr" 
      tabsuffix(5)="_may"; tabsuffix(6)="_jun"; tabsuffix(7)="_jul" ; tabsuffix(8)="_aug"; tabsuffix(9)="_sep" ; tabsuffix(10)="_oct"
      tabsuffix(11)="_nov"; tabsuffix(12)="_dec"
      allocate(tabIdxDimNeeded(1)); tabIdxDimNeeded(1)=RecordDimID;
      call CopyVariableFrame(intputID, outputID, tabsuffix, tabIdxDimNeeded, 1)
    else
      allocate(tabsuffix(24))
      tabsuffix(1)="_jan"; tabsuffix(3)="_feb"; tabsuffix(5)="_mar"; tabsuffix(7)="_apr" 
      tabsuffix(9)="_may"; tabsuffix(11)="_jun"; tabsuffix(13)="_jul" ; tabsuffix(15)="_aug"; tabsuffix(17)="_sep" ; tabsuffix(19)="_oct"
      tabsuffix(21)="_nov"; tabsuffix(23)="_dec"
      tabsuffix(2)="_janNb"; tabsuffix(4)="_febNb"; tabsuffix(6)="_marNb"; tabsuffix(8)="_aprNb" 
      tabsuffix(10)="_mayNb"; tabsuffix(12)="_junNb"; tabsuffix(14)="_julNb" ; tabsuffix(16)="_augNb"; tabsuffix(18)="_sepNb" ; tabsuffix(20)="_octNb"
      tabsuffix(22)="_novNb"; tabsuffix(24)="_decNb"
      allocate(tabIdxDimNeeded(1)); tabIdxDimNeeded(1)=RecordDimID;
      call CopyVariableFrame(intputID, outputID, tabsuffix, tabIdxDimNeeded, 1)
    endif
  elseif(mode.eq.1) then
    if(withnb.eq.0) then
      allocate(tabsuffix(4)); tabsuffix(1)="_JFM"; tabsuffix(2)="_AMJ"; tabsuffix(3)="_JAS"; tabsuffix(4)="_OND"
      allocate(tabIdxDimNeeded(1)); tabIdxDimNeeded(1)=RecordDimID;
      call CopyVariableFrame(intputID, outputID, tabsuffix, tabIdxDimNeeded, 1)
    else
      allocate(tabsuffix(8))
      tabsuffix(1)="_JFM"; tabsuffix(2)="_JFMnb"; tabsuffix(3)="_AMJ"; tabsuffix(4)="_AMJnb"
      tabsuffix(5)="_JAS"; tabsuffix(6)="_JASnb"; tabsuffix(7)="_OND"; tabsuffix(8)="_ONDnb"
      allocate(tabIdxDimNeeded(1)); tabIdxDimNeeded(1)=RecordDimID;
      call CopyVariableFrame(intputID, outputID, tabsuffix, tabIdxDimNeeded, 1)
    endif
  else
    if(withnb.eq.0) then
      allocate(tabsuffix(4)); tabsuffix(1)="_DJF"; tabsuffix(2)="_MAM"; tabsuffix(3)="_JJA"; tabsuffix(4)="_SON"
      allocate(tabIdxDimNeeded(1)); tabIdxDimNeeded(1)=RecordDimID;
      call CopyVariableFrame(intputID, outputID, tabsuffix, tabIdxDimNeeded, 1, undef)
    else
      allocate(tabsuffix(8))
      tabsuffix(1)="_DJF"; tabsuffix(2)="_DJFnb"; tabsuffix(3)="_MAM"; tabsuffix(4)="_MAMnb"
      tabsuffix(5)="_JJA"; tabsuffix(6)="_JJANb"; tabsuffix(7)="_SON"; tabsuffix(8)="_SONnb"
      allocate(tabIdxDimNeeded(1)); tabIdxDimNeeded(1)=RecordDimID;
      call CopyVariableFrame(intputID, outputID, tabsuffix, tabIdxDimNeeded, 1)
    endif
  endif
  write(*,*) "OK"


  !finish the configuration of the output file and starting put data
  status = nf90_enddef(outputID)


  !copy dimension data
  write(*,'(A)',ADVANCE='NO') "Copy dimensions data in output file..."
  do i=1, nbDim
    call check(nf90_inquire_dimension(intputID, i, name = dimName, len = dimSize))
    if(nf90_inq_varid(intputID, dimName, varID).eq.nf90_noerr) then
      if(i.eq.RecordDimID) RecordVarID=varID
      if((i.ne.RecordDimID).and.(nf90_inq_varid(outputID, dimName, outvarid).eq.nf90_noerr)) then
        call check(nf90_inquire_variable(intputID, varID, xtype = variableType, nAtts =nbAtt))
        if(variableType.eq.NF90_INT) then
          allocate(valueInt(dimSize))
          call check(nf90_get_var(intputID, varID, valueInt, (/ 1 /), (/ dimSize /)))
          call check(nf90_put_var(outputID, outvarid, valueInt, (/ 1 /), (/ dimSize /)))
          deallocate(valueInt)
        elseif(variableType.eq.NF90_FLOAT) then
          allocate(valueFlt(dimSize))
          call check(nf90_get_var(intputID, varID, valueFlt, (/ 1 /), (/ dimSize /)))
          call check(nf90_put_var(outputID, outvarid, valueFlt, (/ 1 /), (/ dimSize /)))
          deallocate(valueFlt)
        elseif(variableType.eq.NF90_DOUBLE) then
          allocate(valueDbl(dimSize))
          call check(nf90_get_var(intputID, varID, valueDbl, (/ 1 /), (/ dimSize /)))
          call check(nf90_put_var(outputID, outvarid, valueDbl, (/ 1 /), (/ dimSize /)))
          deallocate(valueDbl)
        endif
      endif
    endif
  enddo
  write(*,*) "OK"

  !copy value
  maxtime=totaltime
  idxProcessVar=1
  do i=1, nbVar
    call check(nf90_inquire_variable(intputID, i, varName, xtype = variableType, ndims = nbvarDim, dimids = varDimID))
    allocate(pdimSize(nbvarDim))
    allocate(start(nbvarDim))
    start=1
    idxRec=-1;
    do j=1, nbvarDim
      call check(nf90_inquire_dimension(intputID, varDimID(j), DimName, nRecords))
      if(varDimID(j).eq.RecordDimID) then  
        pdimSize(j)=1
        idxRec=j
      else
        pdimSize(j)=nRecords
      endif
    enddo

    !check if variable contain time
!(i.ne.RecordVarID).and.
    if(idxRec.ne.-1) then
      write(*,'(A,A,d,A)',ADVANCE='NO') trim(varName), " is processing (undef=~", undef(i), ")..."

      if(mode.eq.3) then
        allocate(outputNb(product(pdimSize),2))
      elseif(mode.eq.4) then
        allocate(outputNb(product(pdimSize),1))
      elseif(mode.eq.5) then
        allocate(outputNb(product(pdimSize),12))
      else
        allocate(outputNb(product(pdimSize),4))
      endif

      if(variableType.eq.NF90_INT) then

        allocate(valueInt2D(product(pdimSize),12))
        if(mode.eq.3) then
          allocate(saisInt2D(product(pdimSize),2))
        elseif(mode.eq.4) then
          allocate(saisInt2D(product(pdimSize),1))
        elseif(mode.eq.5) then
          allocate(saisInt2D(product(pdimSize),12))
        else
          allocate(saisInt2D(product(pdimSize),4))
        endif
        do t=1, maxtime, 12
          if((mode.eq.1).or.(mode.eq.4).or.(mode.eq.5)) then
            call Get12monthsFromT_Int(intputID, i, t, undef(i), valueInt2D, start, pdimSize, idxRec, totaltime)
          elseif(mode.eq.2) then
            call Get12monthsFromT_Int(intputID, i, t-1, undef(i), valueInt2D, start, pdimSize, idxRec, totaltime)
          elseif(mode.eq.3) then
            call Get12monthsFromT_Int(intputID, i, t-9, undef(i), valueInt2D, start, pdimSize, idxRec, totaltime)
          endif
          call month2SaisonAvg_Int(undef(i), undefMinTab(i), undefMaxTab(i), valueInt2D, saisInt2D, outputNb, mode, withundef)
          start(idxRec)=(t-1)/12+1
          if(i.eq.RecordVarID) then
            call check(nf90_put_var(outputID, idxProcessVar, saisInt2D(:,1), start, pdimSize))
          elseif(mode.eq.3) then
            do s=1, 2
              if((s.eq.1).and.(t.eq.1)) saisInt2D(:,s)=undef(i)
              call check(nf90_put_var(outputID, idxProcessVar+(s-1)*(withnb+1), saisInt2D(:,s), start, pdimSize))
              if(withnb.eq.1) call check(nf90_put_var(outputID, idxProcessVar+(s-1)*2+1, outputNb(:,s), start, pdimSize))
            enddo
          elseif(mode.eq.4) then
            call check(nf90_put_var(outputID, idxProcessVar, saisInt2D(:,1), start, pdimSize))
            if(withnb.eq.1) call check(nf90_put_var(outputID, idxProcessVar+1, outputNb(:,1), start, pdimSize))
          elseif((mode.eq.1).or.(mode.eq.2)) then
            do s=1, 4
              if((mode.eq.2).and.(s.eq.1).and.(t.eq.1)) saisInt2D(:,s)=undef(i)
              call check(nf90_put_var(outputID, idxProcessVar+(s-1)*(withnb+1), saisInt2D(:,s), start, pdimSize))
              if(withnb.eq.1) call check(nf90_put_var(outputID, idxProcessVar+(s-1)*2+1, outputNb(:,s), start, pdimSize))
            enddo
          else
            do s=1, 12
              call check(nf90_put_var(outputID, idxProcessVar+(s-1)*(withnb+1), saisInt2D(:,s), start, pdimSize))
              if(withnb.eq.1) call check(nf90_put_var(outputID, idxProcessVar+(s-1)*2+1, outputNb(:,s), start, pdimSize))
            enddo
          endif
        enddo
        if(i.eq.RecordVarID) then
          idxProcessVar=idxProcessVar+1
        elseif(mode.eq.3) then
          idxProcessVar=idxProcessVar+2*(withnb+1)
        elseif(mode.eq.4) then
          idxProcessVar=idxProcessVar+withnb+1
        elseif((mode.eq.1).or.(mode.eq.2)) then
          idxProcessVar=idxProcessVar+4*(withnb+1)
        else
          idxProcessVar=idxProcessVar+12*(withnb+1)
        endif
        deallocate(valueInt2D)
        deallocate(saisInt2D)

      elseif(variableType.eq.NF90_FLOAT) then

        allocate(valueFlt2D(product(pdimSize),12))
        if(mode.eq.3) then
          allocate(saisFlt2D(product(pdimSize),2))
        elseif(mode.eq.4) then
          allocate(saisFlt2D(product(pdimSize),1))
        elseif(mode.eq.5) then
          allocate(saisFlt2D(product(pdimSize),12))
        else
          allocate(saisFlt2D(product(pdimSize),4))
        endif
        do t=1, maxtime, 12
          if((mode.eq.1).or.(mode.eq.4).or.(mode.eq.5)) then
            call Get12monthsFromT_Flt(intputID, i, t, undef(i), valueFlt2D, start, pdimSize, idxRec, totaltime)
          elseif(mode.eq.2) then
            call Get12monthsFromT_Flt(intputID, i, t-1, undef(i), valueFlt2D, start, pdimSize, idxRec, totaltime)
          elseif(mode.eq.3) then
            call Get12monthsFromT_Flt(intputID, i, t-3, undef(i), valueFlt2D, start, pdimSize, idxRec, totaltime)
          endif
          call month2SaisonAvg_Flt(undef(i), undefMinTab(i), undefMaxTab(i), valueFlt2D, saisFlt2D, outputNb, mode, withundef)
          start(idxRec)=(t-1)/12+1
          if(i.eq.RecordVarID) then
            call check(nf90_put_var(outputID, idxProcessVar, saisFlt2D(:,1), start, pdimSize))
          elseif(mode.eq.3) then
            do s=1, 2
              if((s.eq.1).and.(t.eq.1)) saisFlt2D(:,s)=undef(i)
              call check(nf90_put_var(outputID, idxProcessVar+(s-1)*(withnb+1), saisFlt2D(:,s), start, pdimSize))
              if(withnb.eq.1) call check(nf90_put_var(outputID, idxProcessVar+(s-1)*2+1, outputNb(:,s), start, pdimSize))
            enddo
          elseif(mode.eq.4) then
            call check(nf90_put_var(outputID, idxProcessVar, saisFlt2D(:,1), start, pdimSize))
            if(withnb.eq.1) call check(nf90_put_var(outputID, idxProcessVar+1, outputNb(:,1), start, pdimSize))
          elseif((mode.eq.1).or.(mode.eq.2)) then
            do s=1, 4
              if((mode.eq.2).and.(s.eq.1).and.(t.eq.1)) saisFlt2D(:,s)=undef(i)
              call check(nf90_put_var(outputID, idxProcessVar+(s-1)*(withnb+1), saisFlt2D(:,s), start, pdimSize))
              if(withnb.eq.1) call check(nf90_put_var(outputID, idxProcessVar+(s-1)*2+1, outputNb(:,s), start, pdimSize))
            enddo
          else
            do s=1, 12
              call check(nf90_put_var(outputID, idxProcessVar+(s-1)*(withnb+1), saisFlt2D(:,s), start, pdimSize))
              if(withnb.eq.1) call check(nf90_put_var(outputID, idxProcessVar+(s-1)*2+1, outputNb(:,s), start, pdimSize))
            enddo
          endif
        enddo
        if(i.eq.RecordVarID) then
          idxProcessVar=idxProcessVar+1
        elseif(mode.eq.3) then
          idxProcessVar=idxProcessVar+2*(withnb+1)
        elseif(mode.eq.4) then
          idxProcessVar=idxProcessVar+withnb+1
        elseif((mode.eq.1).or.(mode.eq.2)) then
          idxProcessVar=idxProcessVar+4*(withnb+1)
        else
          idxProcessVar=idxProcessVar+12*(withnb+1)
        endif
        deallocate(valueFlt2D)
        deallocate(saisFlt2D)

      elseif(variableType.eq.NF90_DOUBLE) then
               
        allocate(valueDbl2D(product(pdimSize),12))
        if(mode.eq.3) then
          allocate(saisDbl2D(product(pdimSize),2))
        elseif(mode.eq.4) then
          allocate(saisDbl2D(product(pdimSize),1))
        elseif(mode.eq.5) then
          allocate(saisDbl2D(product(pdimSize),12))
        else
          allocate(saisDbl2D(product(pdimSize),4))
        endif
        do t=1, maxtime, 12
          if((mode.eq.1).or.(mode.eq.4).or.(mode.eq.5)) then
            call Get12monthsFromT_Dbl(intputID, i, t, undef(i), valueDbl2D, start, pdimSize, idxRec, totaltime)
          elseif(mode.eq.2) then
            call Get12monthsFromT_Dbl(intputID, i, t-1, undef(i), valueDbl2D, start, pdimSize, idxRec, totaltime)
          elseif(mode.eq.3) then
            call Get12monthsFromT_Dbl(intputID, i, t-9, undef(i), valueDbl2D, start, pdimSize, idxRec, totaltime)
          endif
          call month2SaisonAvg_Dbl(undef(i), undefMinTab(i), undefMaxTab(i), valueDbl2D, saisDbl2D, outputNb, mode, withundef)
          start(idxRec)=(t-1)/12+1
          if(i.eq.RecordVarID) then
            call check(nf90_put_var(outputID, idxProcessVar, saisDbl2D(:,1), start, pdimSize))
          elseif(mode.eq.3) then
            do s=1, 2
              if((s.eq.1).and.(t.eq.1)) saisDbl2D(:,s)=undef(i)
              call check(nf90_put_var(outputID, idxProcessVar+(s-1)*(withnb+1), saisDbl2D(:,s), start, pdimSize))
              if(withnb.eq.1) call check(nf90_put_var(outputID, idxProcessVar+(s-1)*2+1, outputNb(:,s), start, pdimSize))
            enddo
          elseif(mode.eq.4) then
            call check(nf90_put_var(outputID, idxProcessVar, saisDbl2D(:,1), start, pdimSize))
            if(withnb.eq.1) call check(nf90_put_var(outputID, idxProcessVar+1, outputNb(:,1), start, pdimSize))
          elseif((mode.eq.1).or.(mode.eq.2)) then
            do s=1, 4
              if((mode.eq.2).and.(s.eq.1).and.(t.eq.1)) saisDbl2D(:,s)=undef(i)
              call check(nf90_put_var(outputID, idxProcessVar+(s-1)*(withnb+1), saisDbl2D(:,s), start, pdimSize))
              if(withnb.eq.1) call check(nf90_put_var(outputID, idxProcessVar+(s-1)*2+1, outputNb(:,s), start, pdimSize))
            enddo
          else
            do s=1, 12
              call check(nf90_put_var(outputID, idxProcessVar+(s-1)*(withnb+1), saisDbl2D(:,s), start, pdimSize))
              if(withnb.eq.1) call check(nf90_put_var(outputID, idxProcessVar+(s-1)*2+1, outputNb(:,s), start, pdimSize))
            enddo
          endif
        enddo
        if(i.eq.RecordVarID) then
          idxProcessVar=idxProcessVar+1
        elseif(mode.eq.3) then
          idxProcessVar=idxProcessVar+2*(withnb+1)
        elseif(mode.eq.4) then
          idxProcessVar=idxProcessVar+withnb+1
        elseif((mode.eq.1).or.(mode.eq.2)) then
          idxProcessVar=idxProcessVar+4*(withnb+1)
        else
          idxProcessVar=idxProcessVar+12*(withnb+1)
        endif
        deallocate(valueDbl2D)
        deallocate(saisDbl2D)
 
      endif
      write(*,'(A)') "OK"
    else
      write(*,'(A,A)') trim(varName), " is skipped"
      idxProcessVar=idxProcessVar+1
    endif
    if(allocated(outputNb)) deallocate(outputNb)
    if(allocated(start)) deallocate(start)
    if(allocated(pdimSize)) deallocate(pdimSize)
  enddo
  
  !close file
  call check(nf90_close(intputID));
  call check(nf90_close(outputID));
END PROGRAM MoyenneTempo

subroutine month2SaisonAvg_Dbl(undef, undefMin, undefMax, inputtab, outputtab, outputNb, mode, undefmode)
  IMPLICIT NONE

  double precision :: undef, undefMin, undefMax
  double precision, DIMENSION(:,:), ALLOCATABLE :: inputtab
  double precision, DIMENSION(:,:), ALLOCATABLE :: outputtab
  integer, DIMENSION(:,:), ALLOCATABLE :: outputNb
  integer :: mode, undefmode
  INTEGER :: i, m, s
  double precision, DIMENSION(:,:), ALLOCATABLE :: outputtabTemp
  integer, DIMENSION(:,:), ALLOCATABLE :: outputNbTemp
  !write(*,*) "undefmode=", undefmode
  allocate(outputtabTemp(size(outputtab,1),12))
  allocate(outputNbTemp(size(outputNb,1),12))
  outputtab=0.
  outputNb=0
  outputtabTemp=0.
  outputNbTemp=0
  do i=1,size(inputtab,1)
    do m=1, 12
      if((inputtab(i,m).lt.undefMin).or.(inputtab(i,m).gt.undefMax)) then
	outputtabTemp(i,m)=outputtabTemp(i,m)+inputtab(i,m)
        outputNbTemp(i,m)=outputNbTemp(i,m)+1
      endif
    enddo
    if((mode.eq.1).or.(mode.eq.2)) then
      outputNb(i,1)=outputNbTemp(i,1)+outputNbTemp(i,2)+outputNbTemp(i,3)
      outputNb(i,2)=outputNbTemp(i,4)+outputNbTemp(i,5)+outputNbTemp(i,6)
      outputNb(i,3)=outputNbTemp(i,7)+outputNbTemp(i,8)+outputNbTemp(i,9)
      outputNb(i,4)=outputNbTemp(i,10)+outputNbTemp(i,11)+outputNbTemp(i,12)
      if((undefmode.eq.1).and.(outputNb(i,1).lt.3)) then
        outputtab(i,1)=undef
      elseif((undefmode.eq.0).and.(outputNb(i,1).eq.0)) then
        outputtab(i,1)=undef
      else
	outputtab(i,1)=(outputtabTemp(i,1)+outputtabTemp(i,2)+outputtabTemp(i,3))/dble(outputNb(i,1))
      endif
      if((undefmode.eq.1).and.(outputNb(i,2).lt.3)) then
        outputtab(i,2)=undef
      elseif((undefmode.eq.0).and.(outputNb(i,2).eq.0)) then
        outputtab(i,2)=undef
      else
	outputtab(i,2)=(outputtabTemp(i,4)+outputtabTemp(i,5)+outputtabTemp(i,6))/dble(outputNb(i,2))
      endif
      if((undefmode.eq.1).and.(outputNb(i,3).lt.3)) then
        outputtab(i,3)=undef
      elseif((undefmode.eq.0).and.(outputNb(i,3).eq.0)) then
        outputtab(i,3)=undef
      else
	outputtab(i,3)=(outputtabTemp(i,7)+outputtabTemp(i,8)+outputtabTemp(i,9))/dble(outputNb(i,3))
      endif
      if((undefmode.eq.1).and.(outputNb(i,4).lt.3)) then
        outputtab(i,4)=undef
      elseif((undefmode.eq.0).and.(outputNb(i,4).eq.0)) then
        outputtab(i,4)=undef
      else
	outputtab(i,4)=(outputtabTemp(i,10)+outputtabTemp(i,11)+outputtabTemp(i,12))/dble(outputNb(i,4))
      endif
    elseif(mode.eq.3) then
      outputNb(i,1)=outputNbTemp(i,1)+outputNbTemp(i,2)+outputNbTemp(i,3)+outputNbTemp(i,4)+outputNbTemp(i,5)+outputNbTemp(i,6)
      outputNb(i,2)=outputNbTemp(i,7)+outputNbTemp(i,8)+outputNbTemp(i,9)+outputNbTemp(i,10)+outputNbTemp(i,11)+outputNbTemp(i,12)
      if((undefmode.eq.1).and.(outputNb(i,1).lt.6)) then
        outputtab(i,1)=undef
      elseif((undefmode.eq.0).and.(outputNb(i,1).eq.0)) then
        outputtab(i,1)=undef
      else
	outputtab(i,1)=(outputtabTemp(i,1)+outputtabTemp(i,2)+outputtabTemp(i,3)+outputtabTemp(i,4)+outputtabTemp(i,5)+outputtabTemp(i,6))/dble(outputNb(i,1))
      endif
      if((undefmode.eq.1).and.(outputNb(i,2).lt.6)) then
        outputtab(i,2)=undef
      elseif((undefmode.eq.0).and.(outputNb(i,2).eq.0)) then
        outputtab(i,2)=undef
      else
        outputtab(i,2)=(outputtabTemp(i,7)+outputtabTemp(i,8)+outputtabTemp(i,9)+outputtabTemp(i,10)+outputtabTemp(i,11)+outputtabTemp(i,12))/dble(outputNb(i,2))
      endif
    elseif(mode.eq.4) then
      outputNb(i,1)=outputNbTemp(i,1)+outputNbTemp(i,2)+outputNbTemp(i,3)+outputNbTemp(i,4)+outputNbTemp(i,5)+outputNbTemp(i,6)+outputNbTemp(i,7)+outputNbTemp(i,8)+outputNbTemp(i,9)+outputNbTemp(i,10)+outputNbTemp(i,11)+outputNbTemp(i,12)
      if((undefmode.eq.1).and.(outputNb(i,1).lt.12)) then
        outputtab(i,1)=undef
      elseif((undefmode.eq.0).and.(outputNb(i,1).eq.0)) then
        outputtab(i,1)=undef
      else
        outputtab(i,1)=(outputtabTemp(i,1)+outputtabTemp(i,2)+outputtabTemp(i,3)+outputtabTemp(i,4)+outputtabTemp(i,5)+outputtabTemp(i,6)+outputtabTemp(i,7)+outputtabTemp(i,8)+outputtabTemp(i,9)+outputtabTemp(i,10)+outputtabTemp(i,11)+outputtabTemp(i,12))/dble(outputNb(i,1))
      endif
    else
      outputNb(i,:)=outputNbTemp(i,:)
      do s=1, 12
        if((undefmode.eq.1).and.(outputNb(i,s).lt.3)) then
          outputtab(i,s)=undef
        elseif((undefmode.eq.0).and.(outputNb(i,s).eq.0)) then
          outputtab(i,s)=undef
        else
          outputtab(i,s)=outputtabTemp(i,s)/dble(outputNb(i,s))
        endif
      enddo
    endif
  enddo
END SUBROUTINE  month2SaisonAvg_Dbl

subroutine month2SaisonAvg_Flt(undef, undefMin, undefMax, inputtab, outputtab, outputNb, mode, undefmode)
  IMPLICIT NONE

  interface
    subroutine month2SaisonAvg_Dbl(undef, undefMin, undefMax, inputtab, outputtab, outputNb, mode, undefmode)
      double precision :: undef, undefMin, undefMax
      double precision, DIMENSION(:,:), ALLOCATABLE :: inputtab
      double precision, DIMENSION(:,:), ALLOCATABLE :: outputtab
      integer, DIMENSION(:,:), ALLOCATABLE :: outputNb
      integer :: mode, undefmode
    end subroutine month2SaisonAvg_Dbl
  end interface

  double precision :: undef, undefMin, undefMax
  REAL, DIMENSION(:,:), ALLOCATABLE :: inputtab
  REAL, DIMENSION(:,:), ALLOCATABLE :: outputtab
  integer, DIMENSION(:,:), ALLOCATABLE :: outputNb
  integer :: mode, undefmode

  double precision, DIMENSION(:,:), ALLOCATABLE :: inputtabDbl
  double precision, DIMENSION(:,:), ALLOCATABLE :: outputtabDbl
  INTEGER :: i, m

  allocate(inputtabDbl(size(inputtab,1),size(inputtab,2)))
  allocate(outputtabDbl(size(outputtab,1),size(outputtab,2)))
  do i=1, size(inputtab,1)
    do m=1, 12
      inputtabDbl(i,m)=dble(inputtab(i,m))
    enddo
  enddo
  call month2SaisonAvg_Dbl(undef, undefMin, undefMax, inputtabDbl, outputtabDbl, outputNb, mode, undefmode)
  outputtab=real(outputtabDbl)
END SUBROUTINE  month2SaisonAvg_Flt

subroutine month2SaisonAvg_Int(undef, undefMin, undefMax, inputtab, outputtab, outputNb, mode, undefmode)
  IMPLICIT NONE

  interface
    subroutine month2SaisonAvg_Dbl(undef, undefMin, undefMax, inputtab, outputtab, outputNb, mode, undefmode)
      double precision :: undef, undefMin, undefMax
      double precision, DIMENSION(:,:), ALLOCATABLE :: inputtab
      double precision, DIMENSION(:,:), ALLOCATABLE :: outputtab
      integer, DIMENSION(:,:), ALLOCATABLE :: outputNb
      integer :: mode, undefmode
    end subroutine month2SaisonAvg_Dbl
  end interface

  double precision :: undef, undefMin, undefMax
  integer, DIMENSION(:,:), ALLOCATABLE :: inputtab
  integer, DIMENSION(:,:), ALLOCATABLE :: outputtab
  integer, DIMENSION(:,:), ALLOCATABLE :: outputNb
  integer :: mode, undefmode

  double precision, DIMENSION(:,:), ALLOCATABLE :: inputtabDbl
  double precision, DIMENSION(:,:), ALLOCATABLE :: outputtabDbl
  INTEGER :: i, m

  allocate(inputtabDbl(size(inputtab,1),size(inputtab,2)))
  allocate(outputtabDbl(size(outputtab,1),size(outputtab,2)))
  do i=1, size(inputtab,1)
    do m=1, 12
      inputtabDbl(i,m)=dble(inputtab(i,m))
    enddo
  enddo
  call month2SaisonAvg_Dbl(undef, undefMin, undefMax, inputtabDbl, outputtabDbl, outputNb, mode, undefmode)
  outputtab=nint(outputtabDbl)
END SUBROUTINE  month2SaisonAvg_Int


subroutine Get12monthsFromT_Dbl(intputID, varId, t, undef, outputtab, start, pdimSize, idxRec, tmax)
  USE NETCDF
  IMPLICIT NONE
  integer :: intputID, t, idxRec, varId, tmax
  double precision :: undef
  double precision, DIMENSION(:,:), ALLOCATABLE :: outputtab
  integer, DIMENSION(:), ALLOCATABLE :: start, pdimSize
  INTEGER :: i

  i=0
  do while((i.lt.12).and.((t+i).lt.1))
    i=i+1
    outputtab(:,i)=undef
  enddo
  do while((i.lt.12).and.((t+i).le.tmax))
    i=i+1
    start(idxRec)=t+i-1
    call check(nf90_get_var(intputID, varId, outputtab(:,i), start, pdimSize))
  enddo
  do while(i.lt.12)
    i=i+1
    outputtab(:,i)=undef
  enddo
END SUBROUTINE Get12monthsFromT_Dbl

subroutine Get12monthsFromT_Flt(intputID, varId, t, undef, outputtab, start, pdimSize, idxRec, tmax)
  USE NETCDF
  IMPLICIT NONE
  integer :: intputID, t, idxRec, varId, tmax
  double precision :: undef
  real, DIMENSION(:,:), ALLOCATABLE :: outputtab
  integer, DIMENSION(:), ALLOCATABLE :: start, pdimSize
  INTEGER :: i

  i=0
  do while((i.lt.12).and.((t+i).lt.1))
    i=i+1
    outputtab(:,i)=undef
  enddo
  do while((i.lt.12).and.((t+i).le.tmax))
    i=i+1
    start(idxRec)=t+i-1
    call check(nf90_get_var(intputID, varId, outputtab(:,i), start, pdimSize))
  enddo
  do while(i.lt.12)
    i=i+1
    outputtab(:,i)=undef
  enddo
END SUBROUTINE Get12monthsFromT_Flt

subroutine Get12monthsFromT_Int(intputID, varId, t, undef, outputtab, start, pdimSize, idxRec, tmax)
  USE NETCDF
  IMPLICIT NONE
  integer :: intputID, t, idxRec, varId, tmax
  double precision :: undef
  integer, DIMENSION(:,:), ALLOCATABLE :: outputtab
  integer, DIMENSION(:), ALLOCATABLE :: start, pdimSize
  INTEGER :: i

  i=0
  do while((i.lt.12).and.((t+i).lt.1))
    i=i+1
    outputtab(:,i)=undef
  enddo
  do while((i.lt.12).and.((t+i).le.tmax))
    i=i+1
    start(idxRec)=t+i-1
    call check(nf90_get_var(intputID, varId, outputtab(:,i), start, pdimSize))
  enddo
  do while(i.lt.12)
    i=i+1
    outputtab(:,i)=undef
  enddo
END SUBROUTINE Get12monthsFromT_Int

