subroutine usage()
  write(6,'(A)') "Usage:"
  write(6,'(A)') "  ./NetCDFwavg [input.nc] [weight] [input.nc] [weight] ... [output.nc]"
  stop
end subroutine usage

include "NetcdfFonction.f90"

PROGRAM NetCDFwavg 
  USE NETCDF
  IMPLICIT NONE
  
  interface
    subroutine CopyVariableFrame(intputID, outputID, tabsuffix, tabIdxDimNeeded, replace, valundef)
      USE NETCDF
      integer :: intputID, outputID
      character(nf90_max_name), dimension(:), allocatable, optional :: tabsuffix
      integer, dimension(:), allocatable, optional :: tabIdxDimNeeded
      integer, optional :: replace
      double precision, dimension(nf90_max_var_dims), optional :: valundef
    end subroutine CopyVariableFrame
  end interface
  
  CHARACTER(nf90_max_name) :: filenameOut
  INTEGER, DIMENSION(:), allocatable :: pdimSize
 
  INTEGER :: i, j, k,t, nRecords, idxRec, status
  INTEGER, DIMENSION(:), allocatable :: start

  double precision, dimension(nf90_max_var_dims) :: undefMinTab, undefMaxTab, undef

  integer :: outputID, RecordDimID, RecordVarID, iargc
  integer :: nbDim, nbVar, nbAtt
  integer :: nbvarDim, dimSize, varID
  integer :: variableType, outvarid
  integer, dimension(nf90_max_var_dims) :: varDimID
  character(nf90_max_name) :: varName, dimName, parm
  double precision, dimension(:), allocatable :: value
  double precision, dimension(:), allocatable :: moyenne

  integer :: totaltime, maxtime, getTime, step


  integer, dimension(:), allocatable :: intputID;
  double precision, dimension(:), allocatable :: weight;
  character(nf90_max_name), dimension(:), allocatable :: filenameIn
  integer :: nbfile
  double precision :: totalweight

  !GET ARGUMENT FOR SELECT METHODE
  if((mod(iargc(),2).ne.1).or.iargc().lt.3) then
    call usage()
  else
    nbfile=(iargc()-1)/2
    allocate(weight(nbfile))
    allocate(filenameIn(nbfile))
    j=1
    do i=1, nbfile*2, 2
      call getarg(i,filenameIn(j))
      call getarg(i+1,parm); read(parm,*) weight(j)
      j=j+1
    enddo
    call getarg(iargc(),filenameOut)
  endif

  write(6,'(A)',ADVANCE='NO') "Normalize weight..."
  totalweight=0
  do i=1, nbfile
    totalweight=totalweight+weight(i)
  enddo
  weight=weight/totalweight
  write(6,*) "OK"

  write(6,'(A)') "Open input file..."
  allocate(intputID(nbfile))
  do i=1, nbfile
    write(6,'(A,A,A,F4.2,A)') "   ", trim(filenameIn(i)), " (", weight(i), ")"
    call check(nf90_open(filenameIn(i), nf90_nowrite, intputID(i)))
  enddo



  write(6,'(A)',ADVANCE='NO') "   Get some value..."
  !get nbdim and nbvar
  call check(nf90_inquire(intputID(1), nbDim, nbVar))
  !get totaltime
  totaltime=getTime(intputID(1), RecordDimID)
  !get undef value
  do i=1, nbVar
    call GetUndefForVar(intputID(1), i, undef(i), undefMinTab(i), undefMaxTab(i))
  enddo
  write(6,*) "OK"
  

  write(6,'(A)') "Create output file :"
  call check(nf90_create(trim(filenameOut), nf90_hdf5, outputID))   !write target file
  
  !define dimension, variable associated and attribut
  write(6,'(A)',ADVANCE='NO') "   Define dimensions..."
  call CopyDimensionFrame(intputID(1), outputID)
  write(6,*) "OK"


  !define variable from the list
  write(6,'(A)',ADVANCE='NO') "   Define variables and copy attributs..."
  call CopyVariableFrame(intputID(1), outputID)
  write(6,*) "OK"


  !finish the configuration of the output file and starting put data
  status = nf90_enddef(outputID)


  !copy dimension data
  write(6,'(A)',ADVANCE='NO') "Copy dimensions data in output file..."
  do i=1, nbDim
    call check(nf90_inquire_dimension(intputID(1), i, name = dimName, len = dimSize))
    if(nf90_inq_varid(intputID(1), dimName, varID).eq.nf90_noerr) then
      if(i.eq.RecordDimID) RecordVarID=varID
      if((i.ne.RecordDimID).and.(nf90_inq_varid(outputID, dimName, outvarid).eq.nf90_noerr)) then
        call check(nf90_inquire_variable(intputID(1), varID, xtype = variableType, nAtts =nbAtt))
        allocate(value(dimSize))
        call check(nf90_get_var(intputID(1), varID, value, (/ 1 /), (/ dimSize /)))
        call check(nf90_put_var(outputID, outvarid, value, (/ 1 /), (/ dimSize /)))
        deallocate(value)
      endif
    endif
  enddo
  write(6,*) "OK"

  !copy value
  do i=1, nbVar
    call check(nf90_inquire_variable(intputID(1), i, varName, xtype = variableType, ndims = nbvarDim, dimids = varDimID))
    allocate(pdimSize(nbvarDim))
    allocate(start(nbvarDim))
    start=1
    idxRec=-1;
    step=1
    do j=1, nbvarDim
      call check(nf90_inquire_dimension(intputID(1), varDimID(j), DimName, nRecords))
      if(varDimID(j).eq.RecordDimID) then  
        pdimSize(j)=1
        idxRec=j
      else
        pdimSize(j)=nRecords
      endif
    enddo


    if(idxRec.eq.-1) then
      maxtime=1
    else
      maxtime=totaltime
      if(maxtime.lt.200) then
        step=maxtime
      elseif((mod(maxtime,10).eq.0).and.(maxtime.lt.2000)) then
        step=maxtime/10
      endif
      pdimSize(idxRec)=step
    endif
    allocate(value(product(pdimSize)))
    allocate(moyenne(product(pdimSize)))

    write(6,'(A,A,d,A)',ADVANCE='NO') trim(varName), " is processing (undef=~", undef(i), ")..."
    close(6); open(6) !remplace flush qui marche pas sur cette merde de fortran

    do t=1, maxtime, step

      if(t.eq.maxtime*2/3) then
        write(6,'(A)',ADVANCE='NO') " 75%"; close(6); open(6)
      elseif(t.eq.maxtime/2) then
        write(6,'(A)',ADVANCE='NO') " 50%"; close(6); open(6)
      elseif(t.eq.maxtime/3) then
        write(6,'(A)',ADVANCE='NO') " 25%"; close(6); open(6)
      endif
      
      if(idxRec.ne.-1) then
        start(idxRec)=t
      endif
      moyenne=0.D0
      do j=1, nbfile
        call check(nf90_get_var(intputID(j), i, value, start, pdimSize))
        do k=1, size(value)
          if((moyenne(k).ne.undef(i)).and.(value(k).lt.undefMinTab(i)).or.(value(k).gt.undefMaxTab(i))) then
            !moyenne(k)=moyenne(k)+value(k)
            moyenne(k)=moyenne(k)+value(k)*weight(j) !(PM)
          else
            moyenne(k)=undef(i)
          endif
        enddo
      enddo
      !WEIGHT ALREADY NORMALIZED in l71 you don't need this bloc (PM)
      !do k=1, size(value)
      !  if(moyenne(k).ne.undef(i)) then
      !    moyenne(k)=moyenne(k)/(dble(nbfile))
      !  endif
      !enddo (PM)
      call check(nf90_put_var(outputID, i, moyenne, start, pdimSize))
    enddo

    deallocate(value)
    deallocate(moyenne)
    deallocate(pdimSize)
    deallocate(start)
    write(6,*) " 100%"
  enddo
  
  !close file
!   call check(nf90_close(intputID));
  call check(nf90_close(outputID));
END PROGRAM NetCDFwavg



