subroutine check(status)
    USE NETCDF
    IMPLICIT NONE
    
    INTEGER, INTENT (IN) :: status
    if(status /= nf90_noerr) then 
      write(*,*)"Error : ", trim(nf90_strerror(status))
      stop 
    end if
end subroutine check 

function getTime(intputID, RecordDimID)
  USE NETCDF
  IMPLICIT NONE
  integer :: intputID, RecordDimID
  integer :: getTime
  integer :: nbDim, j
  character(nf90_max_name) :: dimName, dimNameToFind

  call check(nf90_inquire(intputID, nbDim, unlimitedDimId = RecordDimID))
  if(RecordDimID.eq.-1) then !il n'y a plus de dimensions unlimited (genre butté par cdo)
    write(*,'(A)') "Warning no unlimited dimensions can be found !!"
    dimName=""; dimNameToFind="time"
    write(*,'(A,A,A)',ADVANCE='NO') "      Try to find '",trim(dimNameToFind),"'..."
    j=0
    do while((j.lt.nbDim).and.(dimName.ne.dimNameToFind))
      j=j+1
      call check(nf90_inquire_dimension(intputID, j, name = dimName))
    enddo
    if((j.eq.nbDim).and.(dimName.ne.dimNameToFind)) then
      write(*,'(A)') "Not Found"
      stop
    else
      write(*,'(A)') "OK"
      RecordDimID=j
    endif
  endif
  call check(nf90_inquire_dimension(intputID, RecordDimID, len = getTime))
end function getTime

function getLonOrLatID(intputID, Lon)
  USE NETCDF
  IMPLICIT NONE
  integer :: intputID, getLonOrLatID, Lon
  integer :: i, try, nbDim
  character(nf90_max_name) :: dimName
  character(nf90_max_name), dimension(8) :: LonName

  call check(nf90_inquire(intputID, nbDim))
  if(Lon.eq.1) then
    LonName(1)="lon"; LonName(2)="Lon"; LonName(3)="LON"; LonName(4)="longitude"
    LonName(5)="Longitude"; LonName(6)="LONGITUDE"; LonName(7)="x"; LonName(8)="X";  
  else
    LonName(1)="lat"; LonName(2)="Lat"; LonName(3)="LAT"; LonName(4)="latitude"
    LonName(5)="Latitude"; LonName(6)="LATITUDE"; LonName(7)="y"; LonName(8)="Y";
  endif
  try=1
  do while((try.ne.-1).and.(try.le.8))
    i=0
    dimName=""
    do while((i.lt.nbDim).and.(dimName.ne.LonName(try)))
      i=i+1
      dimName=""
      call check(nf90_inquire_dimension(intputID, i, name = dimName))
    enddo
    if(dimName.eq.LonName(try)) then
      getLonOrLatID=i
      try=-1
    else
      try=try+1
    endif
  enddo
  if(try.gt.8) then
    getLonOrLatID=-1
  endif
end function getLonOrLatID

function getEnsID(intputID)
  USE NETCDF
  IMPLICIT NONE
  integer :: intputID, getEnsID
  integer :: i, try, nbDim
  character(nf90_max_name) :: dimName
  character(nf90_max_name), dimension(9) :: EnsName

  call check(nf90_inquire(intputID, nbDim))
  EnsName(1)="ens"; EnsName(2)="Ens"; EnsName(3)="ENS"; EnsName(4)="e"
  EnsName(5)="E"; EnsName(6)="Ensemble"; EnsName(7)="ensemble"; EnsName(8)="Ensembles"
  EnsName(9)="ensembles";
  try=1
  do while((try.ne.-1).and.(try.le.9))
    i=0
    do while((i.lt.nbDim).and.(dimName.ne.EnsName(try)))
      i=i+1
      call check(nf90_inquire_dimension(intputID, i, name = dimName))
    enddo
    if(dimName.eq.EnsName(try)) then
      getEnsID=i
      try=-1
    else
      try=try+1
    endif
  enddo
  if(try.gt.8) then
    getEnsID=-1
  endif
end function getEnsID

function getWeightID(intputID)
  USE NETCDF
  IMPLICIT NONE
  integer :: intputID, getWeightID
  integer :: i, try, nbVar
  character(nf90_max_name) :: varName
  character(nf90_max_name), dimension(5) :: WgtName

  call check(nf90_inquire(intputID, nVariables=nbVar))
  WgtName(1)="weight"; WgtName(2)="Weight"; WgtName(3)="Weight"; WgtName(4)="w"; WgtName(5)="W"
  try=1
  do while((try.ne.-1).and.(try.le.5))
    i=0
    do while((i.lt.nbVar).and.(varName.ne.WgtName(try)))
      i=i+1
      call check(nf90_inquire_variable(intputID, i, name=varName))
    enddo
    if(varName.eq.WgtName(try)) then
      getWeightID=i
      try=-1
    else
      try=try+1
    endif
  enddo
  if(try.gt.4) then
    getWeightID=-1
  endif
end function getWeightID

subroutine MakeBoxClio(tlon, tlat, lon_min, lon_max, lat_min, lat_max, Mask)
  double precision :: lon_min, lon_max, lat_min, lat_max
  integer, dimension(120,65) :: Mask
  double precision, dimension(120,65) :: tlon, tlat
  integer :: iLon, iLat, isundef

  if(lon_min.gt.lon_max) lon_max=lon_max+360.

  if((lat_min.eq.-1.).and.(lon_min.eq.-1.)) then
    Mask=1
  else
    Mask=0
    DO iLon=1,120
      DO iLat=1,65
        isundef=1
        if((lat_min.le.tlat(iLon,iLat)).and.(tlat(iLon,iLat).le.lat_max)) then
          if((lon_min.le.tlon(iLon,iLat)).and.(tlon(iLon,iLat).le.lon_max)) then
            isundef=0
          elseif((lon_min.le.(tlon(iLon,iLat)+360.)).and.((tlon(iLon,iLat)+360.).le.lon_max)) then
            isundef=0
          elseif(((lon_min+360).le.tlon(iLon,iLat)).and.(tlon(iLon,iLat).le.(lon_max+360))) then
            isundef=0
          endif
          !write(*,*) lon_min, lon_max, tlon(iLon,iLat), isundef
        endif
        if(isundef.eq.0) Mask(iLon,iLat)=1
      ENDDO
    ENDDO
  endif

  
end subroutine MakeBoxClio


subroutine MakeBox(sizeLon, sizeLat, lon_min, lon_max, lat_min, lat_max, Mask)
  integer :: sizeLon, sizeLat, lon_min, lon_max, lat_min, lat_max
  integer, dimension(sizeLon,sizeLat) :: Mask
  integer :: iLon, iLat, extendlon, extendlat, isundef, iLonC, iLatC

  extendlat=sizeLat
  extendlon=sizeLon
  if(lat_min.gt.lat_max) then
    lat_max=lat_max+sizeLat
    extendlat=sizeLat*2
  endif
  if(lon_min.gt.lon_max) then
    lon_max=lon_max+sizeLon
    extendlon=sizeLon*2
  endif

  Mask=0
  DO iLon=1,extendlon
    DO iLat=1,extendlat
      isundef=1
      if(((lat_min.eq.-1).or.((iLat.ge.lat_min).and.(iLat.le.lat_max))).and.    &
        &   ((lon_min.eq.-1).or.((iLon.ge.lon_min).and.(iLon.le.lon_max)))) isundef=0
      if(iLon.gt.sizeLon)then
        iLonC=iLon-sizeLon
      else
        iLonC=iLon
      endif
      if(iLat.gt.sizeLat)then
        iLatC=iLat-sizeLat
      else
        iLatC=iLat
      endif
      if(isundef.eq.0) Mask(iLonC,iLatC)=1
    ENDDO
  ENDDO
end subroutine MakeBox


subroutine CopyDimensionFrame(intputID, outputID)
  USE NETCDF
  IMPLICIT NONE
  integer :: intputID, outputID, RecordDimID
  integer :: i, getTime, nbDim, time, outdimid, dimSize
  character(nf90_max_name) :: dimName

  time=getTime(intputID, RecordDimID)
  call check(nf90_inquire(intputID, nbDim))
  do i=1, nbDim
    call check(nf90_inquire_dimension(intputID, i, name = dimName, len = dimSize))
    if(i.eq.RecordDimID) then
      call check(nf90_def_dim(outputID, dimName, NF90_UNLIMITED, outdimid))
    else
      call check(nf90_def_dim(outputID, dimName, dimSize, outdimid))
    endif
  enddo
end subroutine CopyDimensionFrame

subroutine CopyVariableFrame(intputID, outputID, tabsuffix, tabIdxDimNeeded, replace, valundef)
  USE NETCDF
  IMPLICIT NONE
  integer :: intputID, outputID 
  character(nf90_max_name), dimension(:), allocatable, optional :: tabsuffix
  integer, dimension(:), allocatable, optional :: tabIdxDimNeeded
  integer, optional :: replace
  double precision, dimension(nf90_max_var_dims), optional :: valundef
  integer :: i, j, k, nbDim, nbVar, variableType, nbAtt, nbvarDim, outvarid1, outvarid2, RecordDimID
  integer, dimension(nf90_max_var_dims) :: varDimID
  integer, dimension(:), allocatable :: varDimIDshorttab
  character(nf90_max_name) :: name, attName, recordname
  integer, dimension(:), allocatable :: mask
  integer :: replaceval, deflate_level
  real :: curValUndef
  deflate_level=1


  call check(nf90_inquire(intputID, nbDim, nbVar))
  call check(nf90_inquire(outputID, unlimitedDimId = RecordDimID))
  call check(nf90_inquire_dimension(outputID, RecordDimID, name = recordname))
  if(present(replace)) then
    replaceval=replace
  else
    replaceval=0
  endif

  do i=1, nbVar
    call check(nf90_inquire_variable(intputID, i, name, xtype=variableType, nAtts=nbAtt, ndims=nbvarDim, dimids=varDimID))
    allocate(varDimIDshorttab(nbvarDim))
    varDimIDshorttab=varDimID(1:nbvarDim)    

    if((present(tabIdxDimNeeded)).and.(present(tabsuffix))) then
      allocate(mask(size(tabIdxDimNeeded)))
      mask=0
      do j=1, nbvarDim
        do k=1, size(tabIdxDimNeeded)
          if(tabIdxDimNeeded(k).eq.varDimIDshorttab(j)) mask(k)=1
        enddo
      enddo
    endif

    if((present(tabIdxDimNeeded)).and.(present(tabsuffix)).and.(name.ne.recordname)) then
      if((replaceval.eq.1).and.(sum(mask).eq.size(tabIdxDimNeeded))) then
        call check(nf90_def_var(outputID, trim(name)//trim(tabsuffix(1)), variableType, varDimIDshorttab, outvarid1, &
        &       shuffle=.TRUE., deflate_level=deflate_level))
      else
        call check(nf90_def_var(outputID, name, variableType, varDimIDshorttab, outvarid1, shuffle=.TRUE., &
        &       deflate_level=deflate_level))
      endif
    else
      call check(nf90_def_var(outputID, name, variableType, varDimIDshorttab, outvarid1, shuffle=.TRUE., &
      &         deflate_level=deflate_level))
    endif
    do j=1, nbAtt
      call check(nf90_inq_attname(intputID, i, j, attName))
      call check(nf90_copy_att(intputID, i, attName, outputID, outvarid1))
    enddo
    if ((present(valundef)).and.(nf90_get_att(intputID, i, "missing_value", curValUndef) /= nf90_noerr)) then
      call check(nf90_put_att(outputID, outvarid1, "missing_value", real(valundef(i))))
    endif

    if((present(tabIdxDimNeeded)).and.(present(tabsuffix)).and.(name.ne.recordname)) then
      if(sum(mask).eq.size(tabIdxDimNeeded)) then
        do k=1, size(tabsuffix)-replaceval
          if(replaceval.eq.1) then
            call check(nf90_def_var(outputID, trim(name)//trim(tabsuffix(k+replaceval)), variableType, varDimIDshorttab, &
            &   outvarid2, shuffle = .TRUE., deflate_level=deflate_level))
          else
            call check(nf90_def_var(outputID, trim(name)//trim(tabsuffix(k+replaceval)), NF90_INT, varDimIDshorttab, &
            &   outvarid2, shuffle = .TRUE., deflate_level=deflate_level))
          endif
          if ((present(valundef)).and.(nf90_get_att(intputID, i, "missing_value", curValUndef) /= nf90_noerr)) then
            call check(nf90_put_att(outputID, outvarid2, "missing_value", real(valundef(i))))
          endif
        enddo
      endif
    endif
    if(allocated(mask)) deallocate(mask) 
    deallocate(varDimIDshorttab)
  enddo
end subroutine CopyVariableFrame

subroutine GetUndefForVar(intputID, varID, Undef, undefMin, undefMax)
  USE NETCDF
  IMPLICIT NONE
  integer :: intputID, varID
  double precision :: Undef, undefMin, undefMax
  integer :: status

  status=nf90_get_att(intputID, varID, "missing_value", undef)
  if(status.ne.nf90_noerr) then
    status=nf90_get_att(intputID, varID, "_FillValue", undef)
    if(status.ne.nf90_noerr) then
      undef=-99.99
      if(undef.lt.0) then
        undefMin=undef*1.01
        undefMax=undef*0.99
      else
        undefMin=undef*0.99
        undefMax=undef*1.01
      endif
    else
      if(undef.lt.0) then
        undefMin=undef*1.01
        undefMax=undef*0.99
      else
        undefMin=undef*0.99
        undefMax=undef*1.01
      endif
    endif
  else
    if(undef.lt.0) then
      undefMin=undef*1.01
      undefMax=undef*0.99
    else
      undefMin=undef*0.99
      undefMax=undef*1.01
    endif
  endif
end subroutine GetUndefForVar

subroutine CopyVariableOfDimensionValue(intputID, outputID, intputVarID, outputVarID, varType, start, nb)
  USE NETCDF
  IMPLICIT NONE
  integer :: intputID, outputID, intputVarID, outputVarID
  integer :: start, nb, varType
  integer, dimension(:), allocatable :: valueInt
  real, dimension(:), allocatable :: valueFlt
  double precision, dimension(:), allocatable :: valueDbl

  if(varType.eq.NF90_INT) then
    allocate(valueInt(nb))
    call check(nf90_get_var(intputID, intputVarID, valueInt, (/ start /), (/ nb /)))
    call check(nf90_put_var(outputID, outputVarID, valueInt, (/ start /), (/ nb /)))
    deallocate(valueInt)
  elseif(varType.eq.NF90_FLOAT) then
    allocate(valueFlt(nb))
    call check(nf90_get_var(intputID, intputVarID, valueFlt, (/ start /), (/ nb /)))
    call check(nf90_put_var(outputID, outputVarID, valueFlt, (/ start /), (/ nb /)))
    deallocate(valueFlt)
  elseif(varType.eq.NF90_DOUBLE) then
    allocate(valueDbl(nb))
    call check(nf90_get_var(intputID, intputVarID, valueDbl, (/ start /), (/ nb /)))
    call check(nf90_put_var(outputID, outputVarID, valueDbl, (/ start /), (/ nb /)))
    deallocate(valueDbl)
  endif
end subroutine CopyVariableOfDimensionValue
 
