PROGRAM SelectLevel
  USE NETCDF
  implicit none
  


  
  !variable pour l'ouverture ecriture du netcdf
  character(nf90_max_name) :: inputfile, outputfile, dimtoselect, argument
  integer :: level, dimtoselectID
  integer :: intputID, outputID, outID
  integer :: nbDim, nbVar, nbGlobAtt, unlimDimID, idxUnlimDim, outidxUnlimDim, idxSelect
  integer :: recordvar, recordvarsize
  character(nf90_max_name) :: dimname, varname, attname
  integer :: dimsize
  integer :: vartype, nbAtt, nbDimInVar
  integer, dimension(nf90_max_var_dims) :: dimidTab
  integer, dimension(:), allocatable :: reducedimidTab, startTab, countTab, outstartTab, outcountTab, dimsizeTab


  !variable pour faire le lien entre input et output
  integer, dimension(nf90_max_var_dims) :: InputdimToOutputdimTab
  integer :: nbskip

  !variable pour les donnees
  integer, dimension(:), allocatable :: valueI
  real, dimension(:), allocatable :: valueF

  !variable d'index qui servent à tout
  integer :: i,j,k,step
  integer :: deflate_level
  deflate_level=1

  !!!!!!!!!!!!!!!!!!!!!!!!!
  !!!! GET PARAMETER   !!!!
  !!!!!!!!!!!!!!!!!!!!!!!!!
  call getarg(1,inputfile)
  call getarg(2,outputfile)
  call getarg(3,dimtoselect)
  call getarg(4,argument)
  read (argument,'(I)') level
  
  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  !!!! OPEN INPUT/OUTPUT FILE !!!!
  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  call check(nf90_open(inputfile, nf90_nowrite, intputID))
  call check(nf90_create(trim(outputfile), nf90_hdf5, outputID))

  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  !!!! GET INPUT INFORMATION !!!!
  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  call check(nf90_inquire(intputID, nbDim, nbVar, nbGlobAtt, unlimDimID))
  write(*,*) trim(inputfile)
  write(*,*) nbDim, " dim"
  write(*,*) nbVar, " var"
  write(*,*) nbGlobAtt, " global att"

  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  !!!! COPY DIMENSION STRUCTURE !!!!
  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  write(*,*) "Dim:"
  allocate(dimsizeTab(nbDim))
  k=1
  do i=1, nbDim
    call check(nf90_inquire_dimension(intputID, i, dimname, dimsize))
    write(*,*) i, trim(dimname)
    if(dimname.eq.dimtoselect) then
      !call check(nf90_def_dim(outputID, dimname, 1, outID))
      dimsizeTab(i)=1
      dimtoselectID=i
      InputdimToOutputdimTab(i)=-1
      idxSelect=i
    else if(i.eq.unlimDimID) then
      call check(nf90_def_dim(outputID, dimname, NF90_UNLIMITED, outID))
      dimsizeTab(i)=dimsize
      InputdimToOutputdimTab(i)=k
      k=k+1
    else
      call check(nf90_def_dim(outputID, dimname, dimsize, outID))
      dimsizeTab(i)=dimsize
      InputdimToOutputdimTab(i)=k
      k=k+1
    endif
  enddo
  write(*,*) InputdimToOutputdimTab(1:7)
  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  !!!! COPY VARIABLE STRUCTURE !!!!
  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  write(*,*) "Var:"
  nbskip=0
  do i=1, nbVar
    call check(nf90_inquire_variable(intputID, i, varname, vartype, nbDimInVar, dimidTab, nbAtt))
    write(*,*) i, trim(varname)

    if(varname.eq.dimtoselect) then
      write(*,*) "            skip"
      nbskip=nbskip+1
    else
      !cherche et enleve la dimension que l'on veut virer
      j=1
      do while((j.le.nbDimInVar).and.(dimidTab(j).ne.idxSelect))
        j=j+1
      enddo
      if(j.le.nbDimInVar) then !Select est trouve
        allocate(reducedimidTab(nbDimInVar-1))
        reducedimidTab(1:j-1)=InputdimToOutputdimTab(dimidTab(1:j-1))
        reducedimidTab(j:nbDimInVar-1)=InputdimToOutputdimTab(dimidTab(j+1:nbDimInVar))
      else
        allocate(reducedimidTab(nbDimInVar))
        reducedimidTab(1:nbDimInVar)=InputdimToOutputdimTab(dimidTab(1:nbDimInVar))
      endif
      

      call check(nf90_def_var(outputID, varname, vartype, reducedimidTab, outID, shuffle = .TRUE., deflate_level=deflate_level))
      deallocate(reducedimidTab)

      do j=1, nbAtt
        call check(nf90_inq_attname(intputID, i, j, attname))
        write(*,*) "            copy attibute ", trim(attname)
        call check(nf90_copy_att(intputID, i, attname, outputID, i-nbskip))
      enddo
    endif
  enddo

  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  !!!! COPY GLOBAL ATTRIBUTE !!!!
  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  write(*,*) "Global Attribute:"
  do i=1, nbGlobAtt
    call check(nf90_inq_attname(intputID, NF90_GLOBAL, i, attname))
    write(*,*) "            copy attibute ", trim(attname)
    call check(nf90_copy_att(intputID, NF90_GLOBAL, attname, outputID, NF90_GLOBAL))
  enddo

  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  !!!! END OF COPY STRUCTURE !!!!
  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  call check(nf90_enddef(outputID))


  !!!!!!!!!!!!!!!!!!!
  !!!! COPY DATA !!!!
  !!!!!!!!!!!!!!!!!!!
  write(*,*) "Var:"
  step=100
  nbskip=0
  do i=1, nbVar
    call check(nf90_inquire_variable(intputID, i, varname, vartype, nbDimInVar, dimidTab))
    write(*,*) i, trim(varname)
    
    if(varname.eq.dimtoselect) then
      write(*,*) "            skip"
      nbskip=nbskip+1
    else
      !cherche et enleve la dimension que l'on veut virer
      j=1
      do while((j.le.nbDimInVar).and.(dimidTab(j).ne.idxSelect))
        j=j+1
      enddo
      allocate(startTab(nbDimInVar))
      allocate(countTab(nbDimInVar))
      if(j.le.nbDimInVar) then !Select est trouve
        allocate(outstartTab(nbDimInVar-1))
        allocate(outcountTab(nbDimInVar-1))
      else
        allocate(outstartTab(nbDimInVar))
        allocate(outcountTab(nbDimInVar))
      endif

      
      idxUnlimDim=-1
      outidxUnlimDim=-1
      k=1
      do j=1, nbDimInVar
        if(dimidTab(j).eq.unlimDimID) then
          startTab(j)=1
          countTab(j)=1
          idxUnlimDim=j
          outidxUnlimDim=k
          recordvarsize=dimsizeTab(dimidTab(j))
          k=k+1
        else if(dimidTab(j).eq.dimtoselectID) then
          startTab(j)=level
          countTab(j)=1
        else
          startTab(j)=1
          countTab(j)=dimsizeTab(dimidTab(j))
          outstartTab(k)=1
          outcountTab(k)=dimsizeTab(dimidTab(j))
          k=k+1
        endif
      enddo
      
      if(idxUnlimDim.ne.-1) then
        write(*,*) "            copy by part"
        recordvar=1
        if((vartype.eq.3).or.(vartype.eq.4)) then
          allocate(valueI(product(countTab)*step))
        else if((vartype.eq.5).or.(vartype.eq.6)) then
          allocate(valueF(product(countTab)*step))
        else
          write(*,*) "Type not supported"
          stop
        endif

        do while(recordvar.le.recordvarsize)
          startTab(idxUnlimDim)=recordvar
          countTab(idxUnlimDim)=min(recordvarsize+1,recordvar+step)-recordvar
          outstartTab(outidxUnlimDim)=startTab(idxUnlimDim)
          outcountTab(outidxUnlimDim)=countTab(idxUnlimDim)
          write(*,*) "            #",startTab(idxUnlimDim), '->', startTab(idxUnlimDim)+countTab(idxUnlimDim)-1
          !write(*,*) startTab
          !write(*,*) countTab
          !write(*,*) outstartTab
          !write(*,*) outcountTab
          if((vartype.eq.3).or.(vartype.eq.4)) then
            call check(nf90_get_var(intputID, i, valueI, start = startTab, count = countTab))
            call check(nf90_put_var(outputID, i-nbskip, valueI, start = outstartTab, count = outcountTab))
          else
            call check(nf90_get_var(intputID, i, valueF, start = startTab, count = countTab))
            call check(nf90_put_var(outputID, i-nbskip, valueF, start = outstartTab, count = outcountTab))
          endif
          recordvar=recordvar+step
        enddo
      else
        write(*,*) "            copy in once"
        if((vartype.eq.3).or.(vartype.eq.4)) then
          allocate(valueI(product(countTab)))
        else if((vartype.eq.5).or.(vartype.eq.6)) then
          allocate(valueF(product(countTab)))
        else
          write(*,*) "Type not supported"
          stop
        endif
        if((vartype.eq.3).or.(vartype.eq.4)) then
          call check(nf90_get_var(intputID, i, valueI, start = startTab, count = countTab))
          call check(nf90_put_var(outputID, i-nbskip, valueI, start = outstartTab, count = outcountTab))
        else
          call check(nf90_get_var(intputID, i, valueF, start = startTab, count = countTab))
          call check(nf90_put_var(outputID, i-nbskip, valueF, start = outstartTab, count = outcountTab))
        endif
      endif
    endif

    if(allocated(valueI)) deallocate(valueI)
    if(allocated(valueF)) deallocate(valueF)
    if(allocated(startTab))deallocate(startTab)
    if(allocated(outstartTab))deallocate(outstartTab)
    if(allocated(countTab))deallocate(countTab)
    if(allocated(outcountTab))deallocate(outcountTab)
  enddo

  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  !!!! CLOSE INPUT/OUTPUT FILE !!!!
  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  call check(nf90_close(intputID))
  call check(nf90_close(outputID))


END PROGRAM SelectLevel

subroutine check(status)
    USE NETCDF
    IMPLICIT NONE
    
    INTEGER, INTENT (IN) :: status
    if(status /= nf90_noerr) then 
      write(*,*)"Error : ", trim(nf90_strerror(status))
      stop 
    end if
end subroutine check 
