subroutine usage()
  write(*,'(A)') "Usage:"
  write(*,'(A)') "  ./VarianceEns [input.nc] [output.nc]"
  stop
end subroutine usage

include "NetcdfFonction.f90"

PROGRAM VarianceEns
  USE NETCDF
  IMPLICIT NONE

  CHARACTER(nf90_max_name) :: filenameIn, filenameOut
 
  INTEGER :: i, j, k, t, ens, nRecords, idxRec, idxEns, idxProcessVar, status
  INTEGER, DIMENSION(:), allocatable :: start, startShort, pdimSize, pdimSizeShort
  

  integer :: intputID, outputID, outdimid, RecordDimID, EnsDimID, WgtVarID
  integer :: nbDim, nbVar, nbAtt, getEnsID, getTime, getWeightID
  integer :: nbvarDim, dimSize, varID
  integer :: variableType, outvarid
  integer, dimension(nf90_max_var_dims) :: varDimID
  double precision, dimension(nf90_max_var_dims) :: undefMinTab, undefMaxTab, undef
  character(nf90_max_name) :: varName, dimName, RecordName, EnsName, WgtName, name, attName
  double precision, dimension(:,:), allocatable :: valueDbl2D
  double precision, dimension(:), allocatable :: valueDbl, moyenneDbl, stdevDbl, sumInt
  integer, dimension(:), allocatable :: weight, varDimIDshorttab, ok
  integer :: totaltime, totalEns
  character(nf90_max_name), dimension(nf90_max_var_dims) :: dimNameList
  integer :: deflate_level
  deflate_level=1

  !GET ARGUMENT FOR SELECT METHODE
  if(iargc().eq.2) then
    call getarg(1,filenameIn)
    call getarg(2,filenameOut)
  else
    call usage()
  endif


  write(*,'(A)',ADVANCE='NO') "Open input file..."
  call check(nf90_open(filenameIn, nf90_nowrite, intputID))
  write(*,*) "OK"


  write(*,'(A)',ADVANCE='NO') "   Get some value..."
  !get nbdim and nbvar
  call check(nf90_inquire(intputID, nbDim, nbVar))
  !get totaltime
  totaltime=getTime(intputID, RecordDimID)
  !get undef value
  do i=1, nbVar
    call GetUndefForVar(intputID, i, undef(i), undefMinTab(i), undefMaxTab(i))
  enddo
  write(*,*) "OK"



  !get ensemble dim name
  write(*,'(A)',ADVANCE='NO') "   Get Ensemble dimension..."
  EnsDimID=getEnsID(intputID)
  if(EnsDimID.eq.-1) then
    write(*,'(A)') "Not Found"
    stop
  else
    call check(nf90_inquire_dimension(intputID, EnsDimID, name = dimName, len = dimSize))
    totalEns=dimSize
    write(*,'(A,A,A)') "OK(", trim(dimName), ")"
  endif

  !get weight dim name
  write(*,'(A)',ADVANCE='NO') "   Get weight dimension..."
  WgtVarID=getWeightID(intputID)
  if(WgtVarID.eq.-1) then
    write(*,'(A)') "Not Found"
    stop
  else
    allocate(weight(totalEns))
    call check(nf90_inquire_variable(intputID, WgtVarID, name = WgtName))
    write(*,'(A,A,A)') "OK(", trim(WgtName), ")"
  endif


  write(*,'(A)') "Create output file :"
  call check(nf90_create(trim(filenameOut), nf90_hdf5, outputID))   !write target file
  
  
  !define dimension, variable associated and attribut
  write(*,'(A)',ADVANCE='NO') "   Define dimensions..."
  do i=1, nbDim
    !get name and size of dimension
    call check(nf90_inquire_dimension(intputID, i, name = dimName, len = dimSize))
    dimNameList(i)=dimName

    !define dimension with separation between unlimited, normal dimension and Lon/Lat
    if(i.eq.RecordDimID) then
      call check(nf90_def_dim(outputID, dimName, NF90_UNLIMITED, outdimid))
    elseif(i.eq.EnsDimID) then
      write(*,'(A,A)',ADVANCE='NO') trim(dimName), " skipped"
    else
      call check(nf90_def_dim(outputID, dimName, dimSize, outdimid))
    endif
  enddo
  write(*,*) "OK"


  !define variable from the list
  write(*,'(A)',ADVANCE='NO') "   Define variables and copy attributs..."
  call check(nf90_inquire_dimension(intputID, RecordDimID, name = RecordName))
  call check(nf90_inquire_dimension(intputID, EnsDimID, name = EnsName))
  allocate(ok(nbVar))
  do i=1, nbVar
    call check(nf90_inquire_variable(intputID, i, name, xtype=variableType, nAtts=nbAtt, ndims=nbvarDim, dimids=varDimID))
    !write(*,*) trim(name)
    j=1
    do while((j.le.nbvarDim).and.(varDimID(j).ne.EnsDimID))
      j=j+1
    enddo
    if(j.gt.nbvarDim) then
      allocate(varDimIDshorttab(nbvarDim))
      varDimIDshorttab=varDimID(1:nbvarDim)
      ok(i)=0  
    else
      allocate(varDimIDshorttab(nbvarDim-1))
      k=1
      do j=1, nbvarDim
        if((varDimID(j).ne.EnsDimID)) then
          varDimIDshorttab(k)=varDimID(j)
          k=k+1
        endif
      enddo
      nbvarDim=nbvarDim-1
      ok(i)=1
    endif
    
    if((name.eq.EnsName).or.(name.eq.WgtName)) then
      ok(i)=0
      if(name.eq.WgtName) WgtVarID=i
   elseif(name.ne.RecordName) then
      if(ok(i).eq.1) then
        call check(nf90_def_var(outputID, trim(name)//"_stdev", NF90_DOUBLE, varDimIDshorttab, outvarid, shuffle = .TRUE., deflate_level=deflate_level))
      else
        call check(nf90_def_var(outputID, name, variableType, varDimIDshorttab, outvarid, shuffle = .TRUE., deflate_level=deflate_level))
      endif
      do j=1, nbAtt
        call check(nf90_inq_attname(intputID, i, j, attName))
        call check(nf90_copy_att(intputID, i, attName, outputID, outvarid))
      enddo
    else
      ok(i)=0
      call check(nf90_def_var(outputID, name, variableType, varDimIDshorttab, outvarid, shuffle = .TRUE., deflate_level=deflate_level))
      do j=1, nbAtt
        call check(nf90_inq_attname(intputID, i, j, attName))
        call check(nf90_copy_att(intputID, i, attName, outputID, outvarid))
      enddo
    endif

    deallocate(varDimIDshorttab)
  enddo
  write(*,*) "OK"


  !finish the configuration of the output file and starting put data
  status = nf90_enddef(outputID)


  !copy dimension data
  write(*,'(A)',ADVANCE='NO') "Copy dimensions data in output file..."
  do i=1, nbDim
    if(i.ne.EnsDimID) then
      call check(nf90_inquire_dimension(intputID, i, name = dimName, len = dimSize))
      if(nf90_inq_varid(intputID, dimName, varID).eq.nf90_noerr) then
        if(nf90_inq_varid(outputID, dimName, outvarid).eq.nf90_noerr) then
          call check(nf90_inquire_variable(intputID, varID, xtype = variableType, nAtts =nbAtt))
          allocate(valueDbl(dimSize))
          call check(nf90_get_var(intputID, varID, valueDbl, (/ 1 /), (/ dimSize /)))
          call check(nf90_put_var(outputID, outvarid, valueDbl, (/ 1 /), (/ dimSize /)))
          deallocate(valueDbl)
        endif
      endif
!       if(nf90_inq_varid(intputID, trim(dimName)//"_bounds", varID).eq.nf90_noerr) then
!         if(nf90_inq_varid(outputID, trim(dimName)//"_bounds", outvarid).eq.nf90_noerr) then
!           allocate(start(2)); allocate(pdimSize(2))
!           call check(nf90_inquire_variable(intputID, varID, ndims=nbvarDim, dimids=varDimID, nAtts=nbAtt))
!           call check(nf90_inquire_dimension(intputID, varDimID(1),len = dimSize)); start(1)=1; pdimSize(1)=dimSize
!           call check(nf90_inquire_dimension(intputID, varDimID(2),len = dimSize)); start(2)=1; pdimSize(2)=dimSize       
!           allocate(valueDbl(product(pdimSize)))
!           valueDbl=-10.0D0
!           call check(nf90_get_var(intputID, varID, valueDbl, start, pdimSize))
!           do j=1, size(valueDbl)
!             write(*,*) j,valueDbl(j)
!           enddo
!           call check(nf90_put_var(outputID, outvarid, valueDbl, start, pdimSize))
!           deallocate(valueDbl)
!           deallocate(start)
!           deallocate(pdimSize)
!         endif
!       endif
    endif
  enddo
  write(*,*) "OK"


  !copy value 
  idxProcessVar=0
  do i=1, nbVar
    call check(nf90_inquire_variable(intputID, i, varName, xtype = variableType, ndims = nbvarDim, dimids = varDimID))
    allocate(pdimSize(nbvarDim)); allocate(pdimSizeShort(nbvarDim-1))
    allocate(start(nbvarDim)); allocate(startShort(nbvarDim-1))
    start=1
    idxRec=-1; idxEns=-1
    do j=1, nbvarDim
      call check(nf90_inquire_dimension(intputID, varDimID(j), DimName, nRecords))
      if(varDimID(j).eq.RecordDimID) then  
        pdimSize(j)=1
        idxRec=j
      elseif(varDimID(j).eq.EnsDimID) then  
        pdimSize(j)=1
        idxEns=j
      else
        pdimSize(j)=nRecords
      endif
    enddo
    
    if((varName.ne."weight").and.(varName.ne.EnsName)) then

      idxProcessVar=idxProcessVar+1

      !check if variable contain lon, lat and time
      if(ok(i).eq.1) then
        write(*,'(A,A,d,A)',ADVANCE='NO') trim(varName), " is processing (undef=~", undef(i), ")..."
      
        allocate(moyenneDbl(product(pdimSize)))
        allocate(sumInt(product(pdimSize)))
        allocate(stdevDbl(product(pdimSize)))

        do t=1, totaltime

          sumInt=0
          moyenneDbl=0.
          start(idxRec)=t

          if(idxEns.eq.1) then
            pdimSizeShort(1:size(pdimSize)-1)=pdimSize(idxEns+1:size(pdimSize))
            startShort(1:size(pdimSize)-1)=start(idxEns+1:size(pdimSize))
          else
            pdimSizeShort(1:idxEns-1)=pdimSize(1:idxEns-1); pdimSizeShort(idxEns:size(pdimSize)-1)=pdimSize(idxEns+1:size(pdimSize))
            startShort(1:idxEns-1)=start(1:idxEns-1); startShort(idxEns:size(pdimSize)-1)=start(idxEns+1:size(pdimSize))
          endif

         allocate(valueDbl2D(product(pdimSize), totalEns))
          call check(nf90_get_var(intputID, WgtVarID, weight, (/1, t/), (/totalEns, 1/)))
          do ens=1, totalEns
            start(idxEns)=ens
            call check(nf90_get_var(intputID, i, valueDbl2D(:,ens), start, pdimSize))
            do j=1, product(pdimSize)
              if((valueDbl2D(j,ens).lt.undefMinTab(i)).or.(valueDbl2D(j,ens).gt.undefMaxTab(i))) then
                moyenneDbl(j)=moyenneDbl(j)+dble(valueDbl2D(j,ens)*weight(ens))
                sumInt(j)=sumInt(j)+weight(ens)
              endif
            enddo
          enddo

          do j=1, product(pdimSize)
            if(sumInt(j).ne.0.) then
              moyenneDbl(j)=moyenneDbl(j)/dble(sumInt(j))
              stdevDbl(j)=0.
              do ens=1, totalEns
                start(idxEns)=ens
                if((valueDbl2D(j,ens).lt.undefMinTab(i)).or.(valueDbl2D(j,ens).gt.undefMaxTab(i))) then
                  stdevDbl(j)=stdevDbl(j)+(dble(valueDbl2D(j,ens)-moyenneDbl(j))**2)*dble(weight(ens))
                endif
              enddo
              stdevDbl(j)=stdevDbl(j)/dble(sumInt(j))
            else
              stdevDbl(j)=undef(i)
            endif
          enddo    

          call check(nf90_put_var(outputID, idxProcessVar, stdevDbl, startShort, pdimSizeShort))
          deallocate(valueDbl2D)
        enddo
          
        deallocate(sumInt)
        deallocate(moyenneDbl)
        write(*,'(A)') "OK"

      else
        write(*,'(A,A)') trim(varName), " is skipped"
        if(idxRec.ne.-1) then
          allocate(valueDbl2D(product(pdimSize),1))
          do t=1, totaltime
            start(idxRec)=t
            call check(nf90_get_var(intputID, i, valueDbl2D(:,1), start, pdimSize))
            call check(nf90_put_var(outputID, idxProcessVar, valueDbl2D(:,1), start, pdimSize))
          enddo
          deallocate(valueDbl2D)
        endif
      endif
    else
        write(*,'(A,A)') trim(varName), " not writed"
    endif 

    if(allocated(start)) deallocate(start)
    if(allocated(pdimSize)) deallocate(pdimSize)
    if(allocated(startShort)) deallocate(startShort)
    if(allocated(pdimSizeShort)) deallocate(pdimSizeShort)
  enddo
  
  !close file
  call check(nf90_close(intputID));
  call check(nf90_close(outputID));
END PROGRAM VarianceEns

