#include <iostream>
#include <iomanip>
#include <vector>
#include <string>
#include <fstream>
#include <netcdfcpp.h>
#include <stdlib.h>
 
using namespace std;

bool firstwarnchangename=true;

class Variable
{
  public:
    Variable(string nameIn);
    long nbvalue;
    string name;
    string nameNetCDF;
    vector<float> value;
    float *tabvalue;
    string FormatName(string nameIn);
    void addValue(float val);
    void addName(string nameIn);
    float *gettab();
};

Variable::Variable(string nameIn)
{
  addName(nameIn);
  nbvalue=0;
  tabvalue=NULL;
}

string Variable::FormatName(string nameIn)
{
  string nameOut=nameIn;
  
  //remove end space : ex "|Variable 1|  " -> "|Variable 1|"
  long PosLastSpace=nameOut.find_last_not_of(" ");
  nameOut=name.substr(0,PosLastSpace+1);

  //replace space by _ : ex "|Variable 1|" -> "|Variable_1|"
  unsigned long Pos=nameOut.find(" ",0);
  while(Pos!=string::npos)
  {
    nameOut.replace(Pos, 1,"_");
    Pos=nameOut.find(" ",0);
  }

  //replace pipe by _ : ex "|Variable_1|" -> "IVariable_1I"
  Pos=nameOut.find("|",0);
  while(Pos!=string::npos)
  {
    nameOut.replace(Pos, 1,"I");
    Pos=nameOut.find("|",0);
  }
  
  //replace pipe by _ : ex "|Variable_1|" -> "IVariable_1I"
  Pos=nameOut.find("-",0);
  while(Pos!=string::npos)
  {
    nameOut.replace(Pos, 1,"m");
    Pos=nameOut.find("-",0);
  }
  
  return nameOut;
}

void Variable::addValue(float val)
{
  value.push_back(val);
  nbvalue++;
}

void Variable::addName(string nameIn)
{
  name=nameIn;
  nameNetCDF=FormatName(nameIn);
  if(firstwarnchangename && name.find(nameNetCDF,0)==string::npos)
  {
    cout<<"Warning : variable name is change due to illegal character"<<endl;
    firstwarnchangename=false;
  }
}

float *Variable::gettab()
{
  if(tabvalue==NULL)
  {
    tabvalue = new float[nbvalue];
    for(int i=0; i<nbvalue; i++)
      tabvalue[i]=value[i];
  }
  return tabvalue;
}

int main(int argc, char *argv[])
{
  //Check execution argument
  if(argc<3)
  {
    cout<<"Usage :"<<endl;
    cout<<"\tVolcCat2NetCDF [VOLC.dat] ... [VOLC.dat] [NetCDFfile]"<<endl;
    return 1;
  }
    
  //Container variable declaration
  vector<Variable> variable;
  
  string header;
  for(int numfile=0; numfile<argc-2; numfile++)
  {
    //open Evolu file
    ifstream file(argv[numfile+1]);
    if (!file)
    {
      cerr<<argv[numfile+1]<<" openning error !!"<<endl;
      return 1;
    }else{
      cout<<"Convert "<<argv[numfile+1]<<endl;
    }
    if(numfile==0)
    {
      variable.push_back(Variable("year"));
      variable.push_back(Variable("month"));
      variable.push_back(Variable("Volc1"));
      variable.push_back(Variable("Volc2"));
      variable.push_back(Variable("Volc3"));
      variable.push_back(Variable("Volc4"));
      variable.push_back(Variable("random"));
    }
    
    //read variable value
    char buf[512];
    while(file>>buf) //annee
    {
      variable[0].addValue(atoi(buf));            //year
      file>>buf; variable[1].addValue(atoi(buf)); //month
      file>>buf; variable[2].addValue(atof(buf)); //volc1
      file>>buf; variable[3].addValue(atof(buf)); //volc2
      file>>buf; variable[4].addValue(atof(buf)); //volc3
      file>>buf; variable[5].addValue(atof(buf)); //volc4
      file>>buf; variable[6].addValue(atof(buf)); //random
    }
       
    file.close();
  }
  
  
  
  //Open output file
  int outid=nccreate(argv[argc-1], NC_NETCDF4);
  
  //attribut
  string title="VolcCat2NetCDF file";
  nc_put_att_text (outid, NC_GLOBAL, "title", title.length(), title.c_str());

  
  //Define dimensions in output file
  ncdimdef(outid, "time", NC_UNLIMITED);
   
  //Define variables in output file
  int dimtab[MAX_NC_DIMS];
  string AttValue;
  stringstream ss (stringstream::in | stringstream::out);
  int varid;
  
  dimtab[0]=0;
  varid=ncvardef(outid, "time", NC_INT, 1, dimtab);
  nc_def_var_deflate(outid, varid, NC_SHUFFLE, 1/*deflate*/, 1/*deflate_level*/);
  ss<<"months since "<<int(variable[0].value[0])<<"-"<<int(variable[1].value[0])<<"-1";
  AttValue = ss.str(); ncattput(outid, 0, "units", NC_CHAR, AttValue.length(), AttValue.c_str());
  AttValue = "time"; ncattput(outid, 0, "long_name", NC_CHAR, AttValue.length(), AttValue.c_str());
  AttValue = "time"; ncattput(outid, 0, "standard_name", NC_CHAR, AttValue.length(), AttValue.c_str());
//   ss<<"01-JAN-"<<setw(4)<<setfill('0')<<int(variable[0].value[0])<<" 00:00:00";
//   AttValue=ss.str(); ncattput(outid, 0, "time_origin", NC_CHAR, AttValue.length(), AttValue.c_str());
  
  for(unsigned int i=2; i<variable.size(); i++)
  {
    varid=ncvardef(outid, variable[i].nameNetCDF.c_str(), NC_FLOAT, 1, dimtab);
    nc_def_var_deflate(outid, varid, NC_SHUFFLE, 1/*deflate*/, 1/*deflate_level*/);
    ncattput(outid, varid, "long_name", NC_CHAR, variable[i].name.length(), variable[i].name.c_str());
    ncattput(outid, varid, "standard_name", NC_CHAR, variable[i].nameNetCDF.length(), variable[i].nameNetCDF.c_str());
  }
 
  //create time variable
  int timevalue[variable[0].nbvalue];
  long timestart[1]={0};
  long timecount[1]={variable[0].nbvalue};
  for(int i=0; i<variable[0].nbvalue; i++) timevalue[i]=(variable[0].value[i]-variable[0].value[0])*12+variable[1].value[i]-variable[1].value[0];
  ncvarput(outid, 0, timestart, timecount, (void*)timevalue);
  
  //Write value
  for(unsigned int i=2; i<variable.size(); i++)
  {
    long start[1]={0};
    long count[1]={variable[i].nbvalue};
    ncvarput(outid, i-1, start, count, (void*)variable[i].gettab());
  }
  ncclose(outid);
    
  cout<<"NetCDF file "<<argv[argc-1]<<" is writen"<<endl;
    
  return 0;
}
