! this contains required routines for interfacing the GSM with the LOVECLIM LCice coupler
! and for carrying out the advective precipitation corrections on the 
! precipitation climatology (vPgcm) using the setPorog routine


! cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
MODULE moddim  ! define array dimensions
! GSM field arrays are of the form F(mNA,mNB), F(mNA,mNB,NICE) for declared size
! or F(0:mNAp,0:mNBp) for ghosted grid, mNAp=mNA+1 
! NICE is # of ice sheets
! active dimension for an ice sheet is given by F(1:ni,1:nj)
! or F(0:nip,0:njp) for ghosted grid, nip=ni+1...
! cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

IMPLICIT NONE


#if defined (EAcal)
INTEGER, PARAMETER :: mNA=265,mNB=141,mNAp=mNA+1,mNBp=mNB+1,NICE=1      !EA525f
#elif defined (NGcal) && defined (GRIDres525)
INTEGER, PARAMETER :: mNA=335,mNB=197,mNAp=mNA+1,mNBp=mNB+1,NICE=1      !NGf525
#elif defined (GRcalonly) 
INTEGER, PARAMETER :: mNA=155,mNB=105,mNAp=mNA+1,mNBp=mNB+1,NICE=1      !GR525
#elif defined (NA15cal) 
INTEGER, PARAMETER :: mNA=131,mNB=101,mNAp=mNA+1,mNBp=mNB+1,NICE=1      !NA15
#elif defined (AN40cal)
INTEGER, PARAMETER :: mNA=141,mNB=141,mNAp=mNA+1,mNBp=mNB+1,NICE=1      !AN40
#elif defined (SQgrid) && defined (HEINO)
INTEGER, PARAMETER :: mNA=81,mNB=81,mNAp=mNA+1,mNBp=mNB+1,NICE=1      !SQ81
#elif defined (SQgrid)
INTEGER, PARAMETER :: mNA=61,mNB=61,mNAp=mNA+1,mNBp=mNB+1,NICE=1      !SQ61
#else
INTEGER, PARAMETER :: mNA=335,mNB=197,mNAp=mNA+1,mNBp=mNB+1,NICE=2      !NGf525/EA525f/GR525
#endif

INTEGER, PARAMETER :: Nmonth=12


END MODULE moddim
! cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

! cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
MODULE modMB ! internal: for PDD scheme
! cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
! signorm READIN in readRsiParms() called by main
! PDct SET to PDCUT in initclimsmb
! SETTING: ALL initclimsmb
USE moddim
IMPLICIT NONE
SAVE
REAL,  PARAMETER :: PDCUT = 2.0    !PD rain/snow cutoff temperature (C)

REAL :: signorm ! only remainder of COMMON/huy/
REAL, PARAMETER :: signormMN=1.0, Dsig=0.2, Dsigprec=-0.5 !for PDD sigma input from GCM
INTEGER, PARAMETER :: Nsig=31
!REAL, PARAMETER :: sconv=(1.0/0.910)/12. !m/yr rain to m(ice equiv)/month ! doubled up with modconstants, remove once modconstants -> pn35.f
!@GCM: this initializes the mass-balance calculator (accumr)
!REAL, PARAMETER :: DT = 0.02
REAL, PARAMETER :: DT = 0.01
!INTEGER, PARAMETER :: NPDMAX = 1005, NPDCMAX = 9005 ! = sigmax*5/DT+2
!INTEGER, PARAMETER :: NPDMAX = 1255, NPDCMAX = 505 ! = sigmax*5/DT+2, DT=0.02
!INTEGER, PARAMETER :: NPDMAX = 2705, NPDCMAX = 1005 ! = sigmax*5/DT+2, DT=0.01
INTEGER, PARAMETER :: NPDMAX = INT( 5.0*(signormMN+(Nsig-1)*Dsig)/DT ) +3 ! = sigmax*5/DT+2, DT=0.01
INTEGER, PARAMETER :: NPDCMAX = INT( (PDCUT+ 5.0*(signormMN+Dsigprec+(Nsig-1)*Dsig))/DT ) +3

#if defined (GCM)
REAL ::  pdds(0:NPDMAX,Nsig),pds(0:NPDCMAX,Nsig)
#else
REAL ::  pdds(0:NPDMAX),pds(0:NPDCMAX)
#endif
REAL, PARAMETER ::  SIGlimLW=-20.0, SIGlimUP=5.0, SIGlimCLW=PDCUT-7.0, SIGlimCUP=3.0+PDCUT !nn2002Ho
REAL, PARAMETER :: SIGlim0 =  -SIGlimLW/DT + 0.5,  SIGlim0c =  -SIGlimCLW/DT + 0.5
REAL :: tstep
END MODULE modMB
! cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc


! cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
MODULE modGCM2GSM ! fields for coupling with LOVECLIM
! cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
! INITIALIZATION/reSETTING: readGCMclimIN in main
USE moddim
IMPLICIT NONE
SAVE
 REAL, DIMENSION(mNA,mNB,NICE,Nmonth) :: vTgcm, vPgcm, vevapgcm, vklapsgcm, vTsiggcm

! DIMENSIONS for  output from GCM
! INTEGER, PARAMETER :: mNAgcm=85, mNBgcm=52 ! NEG 525 range
! INTEGER, PARAMETER :: mNAgcm=70, mNBgcm=53 ! NEG 105 range
#if defined (GCM)
 INTEGER, PARAMETER :: mNAgcm=86, mNBgcm=53 ! GCM
#else
 INTEGER, PARAMETER :: mNAgcm=1, mNBgcm=1 ! no GCM
#endif
END MODULE modGCM2GSM
! cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

! cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
MODULE modTocean ! 2D ocean temperature field for subshelf melt
! cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
USE moddim
IMPLICIT NONE
SAVE
 REAL :: ToceanSSMsq(mNA,mNB,NICE)=28.0 
! cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
END MODULE modTocean
! cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc


!****************************************************************************
SUBROUTINE setPorog(hdn, time, ki, iugcm)  ! compute advective precipitation correction 
!****************************************************************************
 ! IN : hdn, time, ki;    INOUT :iugcm
 ! INOUT MODULE: vPgcm  : **** applies orographic correction to vPgcm *********
 ! input files : iugcm slopex,slopey, 
 !               iugcm+1 UwindMean
 !               iugcm+2 UwindSig
 !               iugcm+3 VwindMean
 !               iugcm+4 VwindSig

 ! *** INPUT files need terminating '111' iINPUTENDcheck line ***

 ! call once at beginning of ice dyn for async ice steps
 ! assumes orographic effect proportional to wind velocity (thus wind velocity is not normalized for slope calc)
 ! scale conservation for each GCM grid cell is initially imposed, then full mass con over whole local grid
 ! GCM grid should be representative scale of GCM precip, say 0.5* mode res, 
 !     so for LoveCLIM T21 : 5 degrees lat, 10 deg lon, or 7.5 X 15 degrees lat,long
!****************************************************************************

 USE modgridm !grid coordinates
 USE modconstants, ONLY : Re !radius of earth
 USE modGCM2GSM, ONLY : vPgcm ! precipitation climatology field 
 IMPLICIT NONE

 REAL, INTENT(IN) :: hdn(0:mNAp,0:mNBp), time ! surface elevation (masl) and time (years)
 INTEGER, INTENT(IN) :: ki ! ice sheet index
 INTEGER, INTENT(INOUT) :: iugcm ! base I/O file unit number for coupler passing files


 INTEGER, PARAMETER :: mNAgcm2=19, mNBgcm2=12, Nweight=3
! Note PREG depends on scale/units of uWind, vWind
! PREG should be set to mean scale of (Wind dot slope)
 REAL, PARAMETER :: PREG=0.005, FPorogMX=5.0, FPorogMN=0.2
 REAL, PARAMETER :: fweight(Nweight)=(/0.66,0.17,0.17/)
 REAL, DIMENSION(mNA, mNB) :: slopex, slopey !, slopeB
 REAL :: slope, slopeG, fscale
 REAL :: ylatmaG, xlonmiG, dphidG, dthedG, ylatmiG, xlonmaG, slopesum, slopesumG
 REAL :: fPorog(mNA, mNB, Nmonth)
 REAL, DIMENSION(mNAgcm2, mNBgcm2, Nmonth) :: fsum
 REAL, DIMENSION(mNAgcm2, mNBgcm2) :: slopeGCMx, slopeGCMy
 REAL, DIMENSION(mNAgcm2, mNBgcm2,Nmonth,Nweight) ::  uWind, vWind
 INTEGER :: icnt(mNAgcm2, mNBgcm2), i, j, ig, jg, kw, nig, njg
 INTEGER :: imonth, iugcmp, iINPUTENDcheck
 CHARACTER (LEN=100) :: fdescript

 ! G (gcm) grid bounds are standard cell centered
 CALL readhead(iugcm)
 DO jg=njg,1,-1
   DO ig=1,nig
     READ(iugcm,*) slopeGCMx(ig,jg),slopeGCMy(ig,jg)
   ENDDO
 ENDDO
 READ(iugcm,*) iINPUTENDcheck ! = 111 as check on data read and correct grid
 IF(iINPUTENDcheck/=111) CALL &
    outerrI("iINPUTENDcheck at end of fPorog readin, slope, should be 111, but is",iINPUTENDcheck)


 iugcm=iugcm+1
 iugcmp=iugcm+1
 CALL readhead(iugcm)
 CALL readhead(iugcmp)

 DO jg=njg,1,-1
   DO ig=1,nig
     READ(iugcm,*) (uWind(ig,jg,imonth,1), imonth=1,Nmonth) ! mean wind velocity
     READ(iugcmp,*) (uWind(ig,jg,imonth,2), imonth=1,Nmonth) ! 1 sigma wind velocity, ie stnd. dev.
     uWind(ig,jg,:,3)=SUM(uWind(ig,jg,:,1:2))
     uWind(ig,jg,:,2)=uWind(ig,jg,:,1)-uWind(ig,jg,:,2)
   ENDDO
 ENDDO
 READ(iugcm,*) iINPUTENDcheck ! = 111 as check on data read and correct grid
 IF(iINPUTENDcheck/=111) CALL &
    outerrI("iINPUTENDcheck at end of fPorog readin GCM uWind mean, should be 111, but is",iINPUTENDcheck)
 READ(iugcmp,*) iINPUTENDcheck ! = 111 as check on data read and correct grid
 IF(iINPUTENDcheck/=111) CALL &
    outerrI("iINPUTENDcheck at end of fPorog readin GCM uWind sigma, should be 111, but is",iINPUTENDcheck)

 iugcm=iugcmp+1
 iugcmp=iugcm+1
 CALL readhead(iugcm)
 CALL readhead(iugcmp)

 DO jg=njg,1,-1
   DO ig=1,nig
     READ(iugcm,*) (vWind(ig,jg,imonth,1), imonth=1,Nmonth) ! mean wind velocity
     READ(iugcmp,*) (vWind(ig,jg,imonth,2), imonth=1,Nmonth) ! 1 sigma wind velocity, ie stnd. dev.
     vWind(ig,jg,:,3)=SUM(vWind(ig,jg,:,1:2))
     vWind(ig,jg,:,2)=vWind(ig,jg,:,1)-vWind(ig,jg,:,2)
   ENDDO
 ENDDO
 READ(iugcm,*) iINPUTENDcheck ! = 111 as check on data read and correct grid
 IF(iINPUTENDcheck/=111) CALL &
    outerrI("iINPUTENDcheck at end of fPorog readin GCM vWind mean, should be 111, but is",iINPUTENDcheck)
 READ(iugcmp,*) iINPUTENDcheck ! = 111 as check on data read and correct grid
 IF(iINPUTENDcheck/=111) CALL &
    outerrI("iINPUTENDcheck at end of fPorog readin GCM vWind sigma, should be 111, but is",iINPUTENDcheck)

! slopeB=0.
 slopesum=0.
 fsum=0.
 fPorog(1:ni,1:nj,:)=0.
 icnt=0
 DO j=1,nj
   DO i=1,ni
     ig= 1+ NINT( (REAL(i)*dphid+xcglonmi(ki)-xlonmiG)/dphidG )
     jg= 1+ NINT( (ylatmaG-(ycglatma(ki)-REAL(j)*dthed))/dphidG )
     slopex(i,j)=(hdn(i+1,j)-hdn(i-1,j))/(2.*Re*dphi) ! can't upstream since has to be independent of wind direction
     slopey(i,j)=(hdn(i,j+1)-hdn(i,j-1))/(2.*Re*dthe)
     DO kw=1,3 !OPT could put on outside loop and fiddle, but hopefully loop is unrolled
! improvement to test would be to interpolate GCM fields, this would also avoid cell edge effects
       DO imonth=1,Nmonth
         slopeG=uWind(ig,jg,imonth,kw)*slopeGCMx(ig,jg) + vWind(ig,jg,imonth,kw)*slopeGCMy(ig,jg)
         slope=uWind(ig,jg,imonth,kw)*slopex(i,j) + vWind(ig,jg,imonth,kw)*slopey(i,j)
!         slopeB(i,j)=slopeB(i,j)+slope
         slopesum=ABS(slope)+slopesum
         slopesumG=ABS(slopeG)+slopesumG
         IF(slopeG >= 0.) THEN
           fPorog(i,j,imonth)=MIN(MAX((slope+PREG)/(slopeG+PREG),FPorogMN),FPorogMX)*fweight(kw)+fPorog(i,j,imonth)
         ELSE
           fPorog(i,j,imonth)=MIN(MAX((slope-slopeG+PREG)/PREG,FPorogMN),FPorogMX)*fweight(kw)+fPorog(i,j,imonth)
         ENDIF
       ENDDO
     ENDDO
     fsum(ig,jg,:)=fsum(ig,jg,:)+fPorog(i,j,:)
     icnt(ig,jg)=icnt(ig,jg)+1
   ENDDO
 ENDDO
! slopeB=slopeB/36.0
 
 PRINT *,"mean slope dot Wind for Porog is",slopesum/REAL(ni*nj*36)
 PRINT *,"mean GSMslope dot Wind for Porog is",slopesumG/REAL(ni*nj*36)
!!$ DO j=nj,1,-1
!!$   DO i=1,ni
!!$     WRITE(404,'(f0.6)') slopeB(i,j)
!!$     DO imonth=1,Nmonth
!!$       ig= 1+ NINT( (REAL(i)*dphid+xcglonmi(ki)-xlonmiG)/dphidG )
!!$       jg= 1+ NINT( (ylatmaG-(ycglatma(ki)-REAL(j)*dthed))/dphidG )
!!$       slopesum=REAL(icnt(ig,jg))/(fsum(ig,jg,imonth)+1.e-5)
!!$       WRITE(400,'(f0.6,1x,i7,1x,f0.6,2(1x,f0.4))') fPorog(i,j,imonth),icnt(ig,jg),fsum(ig,jg,imonth), &
!!$      slopesum,fPorog(i,j,imonth)*slopesum
!!$ ENDDO
!!$ ENDDO
!!$ ENDDO

 !want SUM_overGCMcell (fPorog(i,j)) = icnt(cell)  [eg, if fPorog=1.0, ...]
 DO imonth=1,Nmonth
   fsum(1:nig,1:njg,imonth)=REAL(icnt(1:nig,1:njg))/(fsum(1:nig,1:njg,imonth)+1.e-5) !mass not quite conserved, due to cell area =f(lat)
 ENDDO
 
 DO j=1,nj
   DO i=1,ni
     ig= 1+ NINT( (REAL(i)*dphid+xcglonmi(ki)-xlonmiG)/dphidG )
     jg= 1+ NINT( (ylatmaG-(ycglatma(ki)-REAL(j)*dthed))/dphidG )
     fPorog(i,j,:)=fPorog(i,j,:)*fsum(ig,jg,:)
   ENDDO
 ENDDO

!!$ DO j=nj,1,-1
!!$   WRITE(401,'(12(1x,f0.6))') (fPorog(i,j,:),i=1,ni)
!!$ ENDDO
!!$ DO j=nj,1,-1
!!$   WRITE(402,'(12(1x,f0.6))') (vPgcm(i,j,ki,:),i=1,ni)
!!$ ENDDO

! conserve mass over whole grid
 fscale=SUM(vPgcm(1:ni,1:nj,ki,:))
 vPgcm(1:ni,1:nj,ki,:)=fPorog(1:ni,1:nj,:)*vPgcm(1:ni,1:nj,ki,:)
 fscale= fscale/SUM(vPgcm(1:ni,1:nj,ki,:))
 vPgcm(1:ni,1:nj,ki,:)= fscale * vPgcm(1:ni,1:nj,ki,:)

!!$ DO j=nj,1,-1
!!$   WRITE(403,'(12(1x,f0.6))') (vPgcm(i,j,ki,:),i=1,ni)
!!$ ENDDO

 RETURN

CONTAINS
 SUBROUTINE readhead(iugcm)
  INTEGER, INTENT(IN) :: iugcm
  INTEGER :: itimegcm

!  READ(iugcm,'(A)')fdescript
!  IF(ifHEAD==1) WRITE(*,*)'accessing/nocheck ',fdescript
  READ(iugcm,*) nig, njg, xlonmiG, xlonmaG, ylatmiG, ylatmaG, dphidG, dthedG
  READ(iugcm,*) itimegcm

  READ(iugcm,*)
  IF(xlonmiG>xlonmi(ki)) CALL outerrI("xlonmiG>xlonmi(ki) in fPorog for iugcm",iugcm)
  IF(xlonmaG<xlonma(ki)) CALL outerrI("xlonmaG<xlonma(ki) in fPorog for iugcm",iugcm)
  IF(ylatmiG>ylatmi(ki)) CALL outerrI("ylatmiG>ylatmi(ki) in fPorog for iugcm",iugcm)
  IF(ylatmaG<ylatma(ki)) CALL outerrI("ylatmaG<ylatma(ki) in fPorog for iugcm",iugcm)
  IF(nig>mNAgcm2) CALL outerrI2("nig>mNAgcm2 in fPorog, increase mNAgcm2 to",nig,iugcm)
  IF(njg>mNBgcm2) CALL outerrI2("njg>mNAgcm2 in fPorog, increase mNAgcm2 to",njg,iugcm)
  RETURN
 END SUBROUTINE readhead

!****************************************************************************
END SUBROUTINE setPorog
!****************************************************************************


!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
SUBROUTINE readGCMclimIN(tnew,iu0) ! top routine to readin climate fields from coupler
 ! IUin should point to ki=1 fort.IUin for vTgcm
 ! IUin+1 for ki=2..
 !then to vklapsgcm, .., then vPgcm, .., finally vevapgcm
 ! SETS all: modGCM2GSM[vTgcm, vPgcm, vevapgcm, vklapsgcm, vTsiggcm]
 ! GEN grid
 !ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

 !@GCM routine to read in GCM fields on coarse grid
 !to be called from main in pn01.f
 !will need to update rsi* file

 USE modGCM2GSM
 USE modMB, ONLY : signormMN, Nsig, Dsig

 IMPLICIT NONE

 REAL, INTENT(IN)                  :: tnew
 INTEGER, INTENT(INOUT)               :: iu0

! INTEGER :: iu    !iu is fortran unit #
 REAL :: sigmax

 REAL, PARAMETER :: scmconv=(0.01/0.910)/12. !cm/yr rain to m(ice equiv)/month

 !      readvFintGCM advances iu:
 CALL readvFintGCM(vtgcm, tnew, iu0)
 CALL readvFintGCM(vklapsgcm, tnew, iu0)
 CALL readvFintGCM(vpgcm, tnew, iu0)
 CALL readvFintGCM(vevapgcm, tnew, iu0)
 CALL readvFintGCM(vtsiggcm, tnew, iu0)
 sigmax=signormmn+ REAL(Nsig-1)*Dsig
 vtsiggcm=MAX(MIN(vtsiggcm,sigmax),signormmn)
 ! Taimaz's units: -> C/m and m/yr
 vklapsgcm=0.001*vklapsgcm
 !     GSM internal precip/evap in in m(ice)/month
 vpgcm=scmconv*vpgcm
 vevapgcm=scmconv*vevapgcm

 RETURN

END SUBROUTINE readGCMclimIN
!cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

!cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
SUBROUTINE readvFintGCM(vField,tnew, iu0)
 !     interpolate onto highres grid, gets low res. data from file
 ! advances iu: iu=+
 ! GCM -> GEN
 !cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
 USE modgridm
 USE modGCM2GSM, ONLY : mNAgcm, mNBgcm

 IMPLICIT NONE

 REAL, INTENT(OUT) :: vField(mNA,mNB,NICE,Nmonth)

 INTEGER, INTENT(INOUT) :: iu0    !iu is fortran unit #
 REAL, INTENT(IN) :: tnew

 REAL :: vFin(mNAgcm,0:mNBgcm,Nmonth)

 INTEGER :: ki, iu
 INTEGER :: i,it
 ! "l" suffix is for low res input
 INTEGER :: nil,njl, i1, i2, ixl, j, j1, j2, jyl
 INTEGER :: itimegcm
 REAL :: xlonmil,xlonmal,ylatmil,ylatmal
 REAL :: dthedl,dphidl, u, v, xlon, ylat, x, y, xixl, yjyl
 REAL :: tmpF(mLLNA,mLLNB), tmpF2(mNA,mNB)


 ixl(x) = INT((x-xlonmil + dphidl)/dphidl)
 jyl(y) = INT((ylatmal + dthedl - y)/dthedl)
 xixl(x) = (x-xlonmil + dphidl)/dphidl
 yjyl(y) = (ylatmal + dthedl - y)/dthedl

 ylat(j) = ylatLLma(ki) + dthed*(0.5 -REAL(j))
 xlon(i) = xlonLLmi(ki) + dphid*(REAL(i) -0.5)

 DO ki=1,NICE
   CALL cdim(ki) ! set active dimensions nLLi, nLLj for ice sheet ki
   iu=iu0+10*(ki-1)
   CALL readvFgcm(vFin,nil,njl,xlonmil,xlonmal,ylatmil,  &
      ylatmal,dphidl,dthedl,itimegcm,iu)

   j2=jyl(ylat(nLLj))
   IF(j2>=njl) THEN
     PRINT *,'Error: readvFintGCM:j2,njl,ylat(1),k,iu: ,'  &
        ,j2,njl,ylat(nLLj),ki,iu
     CALL outerr('pGSMclim/readvFintGCM, jyl(ylat(nLLj))>=njl, input from GCM doesnt cover grid') 
   END IF
   j1=jyl(ylat(1))
   IF(j1<1) THEN
     PRINT *,'Error: readvFintGCM:j1,ylat(nLLj): ',j1,ylat(1),ki,iu
     CALL outerr('pGSMclim/readvFintGCM, jyl(ylat(1))<1, input from GCM doesnt cover grid') 
   END IF

   i2=ixl(xlon(nLLi))
   IF(i2>=nil) THEN
     PRINT *,'Error: readvFintGCM:i2,nil,xlon(nLLi): ,'  &
        ,i2,nil,xlon(nLLi),ki,iu,nLLi,dphid,xlonmil,dphidl,xlonLLmi(ki)
     CALL outerr('pGSMclim/readvFintGCM, ixl(xlon(nLLi))>=nil, input from GCM doesnt cover grid') 
   END IF

   i1=ixl(xlon(1))
   IF(i1<1) THEN
     PRINT *,'Error: readvFintGCM:i1,xlon(1):,',i1,xlon(1),ki,iu
     CALL outerr('pGSMclim/readvFintGCM, ixl(xlon(1))<1, input from GCM doesnt cover grid') 
   END IF


   PRINT *, "about to bilinear interpolate vFin onto hres grid for: "  &
      ,ki,iu, "GCM time:", itimegcm
   DO it = 1,Nmonth
     DO j=1,nLLj
       j1=jyl(ylat(j))
       j2=j1+1
       v=yjyl(ylat(j)) - j1
       DO i=1,nLLi
         i1=ixl(xlon(i))
         i2=i1+1
         u=xixl(xlon(i)) - i1
         vField(i,j,ki,it)= u*( v*vFin(i2,j2,it) + (1.-v)*vFin(i2,j1,it) )  &
            + (1.-u) *( v*vFin(i1,j2,it) + (1.-v)*vFin(i1,j1,it) )
       ENDDO
     ENDDO
   ENDDO !it month loop

 ENDDO                     ! ki loop
 iu0=iu0+1

 RETURN
END SUBROUTINE readvFintGCM
!************************************************************

!cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
SUBROUTINE readvFgcm(vF,nigcm,njgcm,xlonmi,xlonma,  &
   ylatmi,ylatma,dphid,dthed,itimegcm,iu)

 ! HEADER format is:
 !FIRST ascii character string: ID, file name, geographic sector,...
 !SECOND nigcm,njgcm,xlonmi,xlonma,ylatmi,ylatma,dphid, dthed
 !     # x pts, # y pts, lower bound of x axis (on GRID CELL CENTER, < 0 for degrees west), upper bound of x
 !     axis .., long and latitude grid cell resolution in degrees
 !THIRD integer time (year) of GCM output
 ! GCM grid
 !cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
 USE modflags
 USE moddim, ONLY : Nmonth
 USE modGCM2GSM, ONLY : mNAgcm, mNBgcm


 IMPLICIT NONE

 REAL, INTENT(OUT) :: vF(mNAgcm,0:mNBgcm,Nmonth)
 INTEGER, INTENT(OUT) :: nigcm, njgcm
 REAL, INTENT(OUT) :: xlonmi,xlonma,ylatmi,ylatma,dthed,dphid
 INTEGER, INTENT(OUT) :: itimegcm
 INTEGER, INTENT(IN) :: iu

 INTEGER :: ifldc1,  i,j,itime
 CHARACTER (LEN=90) :: fdescript


 !     *******read in vF


 READ(iu,'(A)')fdescript

 IF(ifHEAD==1) PRINT *,'accessing/nocheck ',fdescript
 READ(iu,*) nigcm,njgcm,xlonmi,xlonma,ylatmi,ylatma,dphid, dthed

 IF(nigcm>mNAgcm .OR.njgcm>mNBgcm) THEN
   PRINT *,"readvFintGCM and readvFGCM ERROR: increase mNAgcm,mNBgcm dimensions:"  &
      ,nigcm,mNAgcm,njgcm,mNBgcm
     CALL outerr('pGSMclim/readvFgcm, code dimensions too small')
 END IF

 IF(ifHEAD==1) THEN
   PRINT *,'xlonmi,xlonma,ylatmi,ylatma,dphid,dthed'
   PRINT *, xlonmi,xlonma,ylatmi,ylatma,dphid,dthed
 END IF

 IF( ABS((xlonma-xlonmi)/dphid +1 - nigcm) > 1.e-7) &
    CALL outerr("readvFgcm: GCM2GSM(fort.21?) longitude grid spec is inconsistent, Need grid centered")
 IF( ABS((ylatma-ylatmi)/dthed +1 - njgcm) > 1.e-7) &
    CALL outerr("readvFgcm: GCM2GSM(fort.21?) latitude grid spec is inconsistent, Need grid centered")
 

 READ(iu,*) itimegcm                !can add other stuff later eg. unique run #
 READ(iu,*)           !Taimaz added extra blank line for some reason

 DO  j = njgcm,1,-1
   ! data file uses South to North (latitude) ordering for ease of plotting
   ! but GSM runs on North to South (colatitude) ordering
   ! each data line should have 12 entries for each month:
   READ(iu,*) ((vF(i,j,itime),itime=1,Nmonth),i=1,nigcm)
 END DO

 CLOSE(iu)


 RETURN
END SUBROUTINE readvFgcm
!cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc


!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
SUBROUTINE updateToceanSSM(tym,iuGCM0) ! sets ocean field based on site ocean temperature profiles from coupler
 ! cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
 !     MASKS OUT ELLESMERE
 ! uses hs(:,:,NICE), not H(:,:)

 USE modflags
 USE modgridm
 USE modTocean
 USE modgengrid ! user must define xy2ij(x,y,i,j)

 IMPLICIT NONE

 !     parameter(Betap=0.00087) !K/m basal T grad check*******
 !      parameter(THEH=69.5)      !SE high: lower mask boundary

 INTEGER, PARAMETER :: MAXkregblk=10, IU=37, MAXnLevelsToceanIN=10
 REAL, PARAMETER :: TcalvCut=-2.0

 REAL :: x, y
 REAL, INTENT(IN) :: tym
 INTEGER, INTENT(INOUT) :: iuGCM0
 INTEGER :: i, j, istart(NICE)=1, kij, oceanSpecID(NICE)=0, oceanFortSpecID, ki
 INTEGER :: nLevelsToceanIN, itimegcm, nkreg, kijin, k1depth=0, k2depth=0, iuGCM
 REAL, DIMENSION(MAXkregblk,NICE) :: bndi1=1,bndi2=1,bndj1=1,bndj2=1
 INTEGER :: nkregblk(NICE)=1 ! number Tocean blocks
 INTEGER :: kreg(MAXkregblk,NICE)=1 ! number of Tocean_GCM reference profiles
 INTEGER :: ix1, ix2, jy1, jy2
 CHARACTER*70 dnotes
 REAL :: Toceanin(MAXkregblk)=0.0,ToceanZin(MAXnLevelsToceanIN)

 DO ki=1,NICE
   kiceS=ki ! modgridm pass to xy2ij
   iuGCM=iuGCM0+10*(ki-1)
   IF(istart(ki)==1) THEN
     IF(ki==1) THEN
       PRINT *,"enter k1,k2 ToceanSpec grid index range for ice sheet:",ki
       READ *,k1depth, k2depth
       IF(k2depth<k1depth)  CALL outerrI2("k2depth<k1depth",k2depth,k1depth)
     ENDIF
     CALL askofil(IU,'ToceanSpec file')
     READ(IU,'(A)') dnotes   
     READ(IU,*)
     IF(ifHEAD==1)  PRINT *,"ToceanSpec config:",dnotes
     READ(IU,*) nkregblk(ki), oceanSpecID(ki)
     IF(nkregblk(ki)>MAXkregblk) CALL outerrI2("nkregblk(ki)>MAXkregblk:",nkregblk(ki),MAXkregblk)
     DO kij=1,nkregblk(ki)
       READ(IU,*) kreg(kij,ki),bndi1(kij,ki),bndi2(kij,ki),bndj2(kij,ki),bndj1(kij,ki) ! bndj read in reverse for colat ordering
       IF(bndi1(kij,ki)<xlonLLmi(ki)+0.5*dphid) THEN
         IF(ifHEAD==1) PRINT *,"resetting bndi1(ki) to xlonLLmi(ki)+dphid*0.5"
         bndi1(kij,ki)=xlonLLmi(ki)+0.5*dphid
       ENDIF
       IF(bndi1(kij,ki)>xlonLLma(ki)-0.5*dphid) THEN
         IF(ifHEAD==1) PRINT *,"resetting bndi1(ki) to xlonLLmi(ki)-dphid*0.5"
         bndi1(kij,ki)=xlonLLma(ki)-0.5*dphi
       ENDIF
       IF(bndj2(kij,ki)<ylatLLmi(ki)+0.5*dthed) THEN
         IF(ifHEAD==1) PRINT *,"resetting bndj2(ki) to ylatLLmi(ki)+dthed*0.5"
         bndj2(kij,ki)=ylatLLmi(ki)+0.5*dthed
       ENDIF
       IF(bndj1(kij,ki)>ylatLLma(ki)-0.5*dthed) THEN
         IF(ifHEAD==1) PRINT *,"resetting bndj1(ki) to ylatLLma(ki)+dthed*0.5"
         bndj1(kij,ki)=ylatLLma(ki)-0.5*dthed
       ENDIF
     ENDDO
     istart(ki)=0
     CLOSE(IU)
   ENDIF

   READ(iuGCM,*) itimegcm, nkreg, nLevelsToceanIN, oceanFortSpecID
   IF ( nLevelsToceanIN > MAXnLevelsToceanIN) &
      CALL outerrI2("nLevelsToceanIN > MAXnLevelsToceanIN",nLevelsToceanIN,MAXnLevelsToceanIN)
   IF ( nLevelsToceanIN <k2depth) &
      CALL outerrI2("nLevelsToceanIN <k2depth",nLevelsToceanIN,k2depth)
   IF( oceanFortSpecID /= oceanSpecID(ki)) &
      CALL outerrI2("ToceanSPEC ID mismatch with GCM pass:",oceanFortSpecID,oceanSpecID(ki))
   IF(nkreg<nkregblk(ki)) CALL outerrI("nkreg(coupler)<nkregblk(ToceanSpec) at ki=",ki)
   READ(iuGCM,*)
   DO kij=1,nkreg
     READ(iuGCM,*) kijIN,ToceanZin(1:nLevelsToceanIN)
     IF(kijIN /= kij) CALL outerrI("Toceainin kijIN /= kij at kreg=",kij)
     Toceanin(kij)=SUM(ToceanZin(k1depth:k2depth))/REAL(1+k2depth-k1depth)
   ENDDO


   DO kij=1,nkregblk(ki)
     CALL xy2ij(bndi1(kij,ki),bndj1(kij,ki), ix1, jy1) !model dependent code to convert geographic coordinates to matrix indices
     CALL xy2ij(bndi2(kij,ki),bndj2(kij,ki), ix2, jy2)
!     DO i=ix(bndi1(kij,ki),bndj1(kij,ki)), ix(bndi2(kij,ki),bndj2(kij,ki))
!       DO j=jy(bndi1(kij,ki),bndj1(kij,ki)), jy(bndi2(kij,ki),bndj2(kij,ki))
     DO j=jy1,jy2
       DO i=ix1,ix2
         ToceanSSMsq(i,j,ki)=(Toceanin(kreg(kij,ki))-TcalvCut)**1.6
       ENDDO
     ENDDO
   ENDDO

!old   iuGCM=iuGCM+1
 ENDDO !k=1,NICE

 RETURN
END SUBROUTINE updateToceanSSM
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
SUBROUTINE outerr(smessage)
 !ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc


 CHARACTER (LEN=*), INTENT(IN)        :: smessage

 PRINT *,"ERROR:::",smessage
 STOP 666
 RETURN

END SUBROUTINE outerr
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc


!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
SUBROUTINE outerrI(smessage,idat)
 !ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc


 CHARACTER (LEN=*), INTENT(IN)        :: smessage
 INTEGER, INTENT(IN) :: idat

 PRINT *,"ERROR:::",smessage,idat
 STOP 666
 RETURN

END SUBROUTINE outerrI
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
SUBROUTINE outerrI2(smessage,idat,idat2)
 !ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc


 CHARACTER (LEN=*), INTENT(IN)        :: smessage
 INTEGER, INTENT(IN) :: idat,idat2

 PRINT *,"ERROR:::",smessage,idat,idat2
 STOP 666
 RETURN

END SUBROUTINE outerrI2
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
